/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2009
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* ============================================================================
 *   @file  Emac_dual_emac_example.c
 *
 *   @path  $(CSLPATH)\example\emac\emac_dual_emac\src
 *
 *   @brief  Example for EMAC CSL supporting multiple instance
 *
 *  ============================================================================
 *  ============================================================================
 *   @n <b> Example Description </b>
 *    1. This example is demonstrating the dual emac feature.
 *    2. This example does not configure MDIO module.
 *	  3. Core 0 is the master core i.e. EMAC configuration is done by core 0.
 *    4. Packet size used in this example is 1600 bytes
 *	  5. One MAC addresses allocated for each EMAC
 *                              EMAC0                EMAC1 
 *       MAC address:     00:01:02:03:04:05     60:61:62:63:64:65
 *	  6. Uses CPPI RAM as EMAC descriptor memory
 *	  7. Selects the receive filter as unicast
 *	  8. Sends total number of 1055 packets from EMAC1 to EMAC0
 *	  9. Sends total number of 1055 packets from EMAC0 to EMAC1
 *
 *    Results of running this example:
 *  -------------------------------------------------------------------------------
 *    Dual EMAC example

 *    Configure and open EMAC0
 *    Configure and open EMAC1

 *    Starting packet transfer...

 *    EMAC1 send and EMAC0 receive:
 *    Calling EMAC_setReceiveFilter() for EMAC0
 *    Sent 1055 of 1055 packets, received 1055 packets

 *    EMAC0 send and EMAC1 receive
 *    Calling EMAC_setReceiveFilter() for EMAC1
 *    Sent 1055 of 1055 packets, received 1055 packets

 *    Calling EMAC_close() for EMAC0
 *    Calling EMAC_close() for EMAC1

 *    Example Passed
 * =============================================================================
 *
 *   <b> Procedure to run the example </b>
 *   @verbatim
 *      1. Configure the CCS setup to work with.
 *      2. Launch CCS window.
 *      3. Open project Emac_dual_emac_example.pjt
 *      4. Build the project and load the .out file of the project.
 *      5. Run the example.
 *   @endverbatim
 *
 * =============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <csl_emac.h>
#include <csl_psc.h>
#include <csl_chip.h>

#define MASTER_CORE     0       /* Which core is responsible for EMAC common initialization */

Uint32 coreNum;
extern Uint32  CSL_chipReadReg( CSL_ChipReg  reg);

/*
 * We'll use INT9 for the EMAC0 Rx and 10 for Tx- this is an arbitrary choice
 * We'll use INT11 for the EMAC1 Rx and 12 for Tx- this is an arbitrary choice
 */
#define EMAC_INT_VAL    9
#define EMAC_INT_FLAG   (0xF<<EMAC_INT_VAL)

/*
 * Raw Data Buffers
 *
 * This example uses internal memory, so we could just use 1518 as our
 * buffer size. However, if the buffers were in cacheable external
 * memory, we'd have to make sure each was cache aligned, and
 * thus use a size of at least 1536 because a buffer must fill an
 * the entire cache line.
 */
#define PKT_MAX 160
EMAC_Pkt packet_header[PKT_MAX];
Uint8 packet_buffer[PKT_MAX][1600];
Uint8 packet_sourcedata[1600];


/*
 * Packet Queue
 *
 * This simple queue object is just for our example packet
 * queue. It is not required for using EMAC/MDIO
 *
 */
typedef struct app_pktq {
  Uint32            Count;      /**< Number of packets in queue */
  EMAC_Pkt          *pHead;     /**< Pointer to first packet    */
  EMAC_Pkt          *pTail;     /**< Pointer to last packet     */
} APP_PKTQ;

/*
 * Queue Helper Functions
 */
static EMAC_Pkt *App_pqPop( APP_PKTQ *pq );
static void App_pqPush( APP_PKTQ *pq, EMAC_Pkt *pPktHdr );

/*
 * Declare our packet queues
 */
APP_PKTQ     FreeQueue;             /* Free packets for RX or TX */
APP_PKTQ     RxQueue;               /* Received packets */

/*
 * Declare EMAC device and core instance
 */
#define NUM_EMAC_PORTS  2
#define EMAC1_100MBPS
EMAC_Device emacPort[NUM_EMAC_PORTS];
EMAC_Core   emacCore[NUM_EMAC_PORTS];

/*
 * Declare some local status variables
 */
volatile Uint32   LinkStatus[NUM_EMAC_PORTS] = {0,0};  /* Our current link status */
volatile Uint32   LocalTicks = 0;  /* Current time in 100ms ticks */

extern cregister volatile unsigned int IER;
#define OUREMAC_enter() IER &= ~EMAC_INT_FLAG
#define OUREMAC_exit()  IER |= EMAC_INT_FLAG

/*
 * Declare some local functions
 */
static EMAC_Pkt *GetPacket(Handle hApplication);
static void     FreePacket(Handle hApplication, EMAC_Pkt *pPKT);
static EMAC_Pkt *RxPacket( Handle hApplication, EMAC_Pkt *pPKT);
static void     StatusUpdate(Handle hApplication);
static void     StatisticsUpdate(Handle hApplication);
static int      verify_packet( EMAC_Pkt *pPkt, Uint32 size );

static EMAC_Status status[NUM_EMAC_PORTS];

/*
 * Declare some external functions
 */
extern void DoTimerInitialization(void);
extern void DoInterruptsInitialization();
extern void DoInterruptEnd(void);
void enableEMAC(int instNum);
int EMACInit(int instNum, EMAC_Config *ecfg);
int DualEMACSendReceive(int txInstNum, EMAC_Config *ecfgTx, EMAC_Config *ecfgRx);
/*
 * =============================================================================
 *   @func   Main 
 *
 *   @desc
 *      Main Example Entry Point.  
 *      
 *   @arg
 *      None
 *      
 *   @expected result
 *      If the example passes, it displays the message "Example Passed"
 *      If the example fails, it displays the messae "Example Failed"
 *
 *   @eg
 *      exampleMain(); // Read description.
 * =============================================================================
 */
void main()
{
    EMAC_Config     ecfg0, ecfg1;
	Uint32          i;

    coreNum = CSL_chipReadReg(CSL_CHIP_DNUM);

    /*
     * Basic Initialization
     */                    
    printf("Dual EMAC example\n\n");

    /* Init packet data array */
    for( i=0; i<1600; i++ ) {
		packet_sourcedata[i] = i;
	}

    /* Initialize our buffer pool */
    memset( &FreeQueue, 0, sizeof(APP_PKTQ) );
    memset( &RxQueue, 0, sizeof(APP_PKTQ) );
    for( i=0; i<PKT_MAX; i++ ) {
        /*
         * Init the buffer headers. Note that the buffer pointer
         * and buffer length are fixed and to not change
         */
        memset( &packet_header[i], 0, sizeof(EMAC_Pkt) );
        packet_header[i].pDataBuffer = packet_buffer[i];
        packet_header[i].BufferLen   = 1600;
        App_pqPush( &FreeQueue, &packet_header[i] );
    }
    
	/*Configure and open EMAC0*/
    printf("Configure and open EMAC0\n");
    if (EMACInit(CSL_EMAC_0, &ecfg0))
    {
        printf("Returned error %08x\n",i);
        goto exitFail;
    }

	/*Configure and open EMAC1*/
    printf("Configure and open EMAC1\n");
    if (EMACInit(CSL_EMAC_1, &ecfg1))
    {
        printf("Returned error %08x\n",i);
        goto exitFail;
    }

    /* Do timer initialization */
    DoTimerInitialization();

    /* Do interrupt setup */
	DoInterruptsInitialization();


    /* Start packet transfer */
    printf("\nStarting packet transfer...\n\n");

    /* Set the receive filter for EMAC0 */
    printf("EMAC1 send and EMAC0 receive:\nCalling EMAC_setReceiveFilter() for EMAC0\n");
    OUREMAC_enter();
    i = EMAC_setReceiveFilter((Handle)&emacPort[0], EMAC_RXFILTER_DIRECT, 0 );
    OUREMAC_exit();
    if(i)
    {
        printf("Returned error %08x\n",i);
        goto close_emac;
    }

    /* EMAC1 send and EMAC0 receive */
    if(DualEMACSendReceive(CSL_EMAC_1, &ecfg1, &ecfg0))
        goto exitFail;

    /* Set the receive filter for EMAC1 */
    printf("EMAC0 send and EMAC1 receive\nCalling EMAC_setReceiveFilter() for EMAC1\n");
    OUREMAC_enter();
    i = EMAC_setReceiveFilter((Handle)&emacPort[1], EMAC_RXFILTER_DIRECT, 0 );
    OUREMAC_exit();
    if(i)
    {
        printf("Returned error %08x\n",i);
        goto close_emac;
    }

    /* EMAC0 send and EMAC1 receive */
    if(DualEMACSendReceive(CSL_EMAC_0, &ecfg0, &ecfg1))
    	goto exitFail;
    /*
     * Close EMAC
     */
close_emac:

    /* Since we're done, we leave the interrupt disabled */
	DoInterruptEnd();
   
    printf("Calling EMAC_close() for EMAC0\n");
    i = EMAC_close((Handle)&emacPort[0]);
    if(i) {
        printf("Returned error %08x\n",i);
        goto exitFail;
    }

    printf("Calling EMAC_close() for EMAC1\n");
    i = EMAC_close((Handle)&emacPort[1]);
    if(i) {
        printf("Returned error %08x\n",i);
        goto exitFail;
    } else goto exitSuccess;

exitFail:
    printf("\nExample Failed\n");
    exit (0);
exitSuccess:
    printf("\nExample Passed\n");
    exit (0);
}

/*
 * =============================================================================
 *   @func   EMACInit
 *
 *   @desc
 *      Configure and open an EMAC device
 *      
 *
 *   @arg
 *      instNum: EMAC Peripheral ID to identify the EMAC controller to be opened
 *      ecfg: pointer for EMAC's configuration structure
 *      
 *   @eg
 *      int EMACInit(int instNum, EMAC_Config *ecfg); // Refer description
 * =============================================================================
 */
int EMACInit(int instNum, EMAC_Config *ecfg)
{
    EMAC_AddrConfig *addrCfg;
    Uint32          i, j;

    /*
     * Initialize Configuration and Open EMAC
     */
	if(coreNum == MASTER_CORE) 
	{
		/* Not using the MDIO configuration */
		ecfg->EMACCommonConfig.UseMdio = 0;
		/* core 0 is master core in terms of EMAC */
		ecfg->EMACCommonConfig.CoreNum = 0; 
		/* packet size */
		ecfg->EMACCommonConfig.PktMTU = 1600;
		/* Initialize our EMAC Dev structure.*/

		if (instNum == CSL_EMAC_0) {
			memset(&emacPort[0], 0, sizeof(EMAC_Device));
		}
		else if (instNum == CSL_EMAC_1) {
			memset(&emacPort[1], 0, sizeof(EMAC_Device));
		}

		/* Setup the EMAC configuration */
    	ecfg->EMACCommonConfig.ModeFlags   = EMAC_CONFIG_MODEFLG_FULLDUPLEX | EMAC_CONFIG_MODEFLG_GMIIEN
										   | EMAC_CONFIG_MODEFLG_GIGABIT | EMAC_CONFIG_MODEFLAG_GIGFORCE;

#ifdef EMAC1_100MBPS
        if (instNum == CSL_EMAC_1) {
			ecfg->EMACCommonConfig.ModeFlags   = EMAC_CONFIG_MODEFLG_FULLDUPLEX | EMAC_CONFIG_MODEFLG_GMIIEN
			                                   | EMAC_CONFIG_MODEFLAG_RMIISPEED100;
		}
#endif		
		ecfg->EMACCommonConfig.MdioModeFlags    = MDIO_MODEFLG_FD1000;

        /* Enable EMAC in PSC control module */
        enableEMAC(instNum);
	}

    /* Initialize EMAC core instance structure.*/
	if (instNum == CSL_EMAC_0) {
		memset(&emacCore[0], 0, sizeof(EMAC_Core));
	}
	else if (instNum == CSL_EMAC_1) {
		memset(&emacCore[1], 0, sizeof(EMAC_Core));
	}

    switch(coreNum) {
        default:
        case 0:                   // core 0 use channel 0 
            ecfg->EMACCoreConfig.ChannelInfo.TxChanEnable = 1;
            ecfg->EMACCoreConfig.ChannelInfo.RxChanEnable = 1;
            break;
        case 1:                   // core 1 use channel 1 
            ecfg->EMACCoreConfig.ChannelInfo.TxChanEnable = 2;
            ecfg->EMACCoreConfig.ChannelInfo.RxChanEnable = 2;
            break;
        case 2:                   // core 2 use channel 2 
            ecfg->EMACCoreConfig.ChannelInfo.TxChanEnable = 4;
            ecfg->EMACCoreConfig.ChannelInfo.RxChanEnable = 4;
            break;
    }

    ecfg->EMACCoreConfig.RxMaxPktPool              = 8;
    ecfg->EMACCoreConfig.pfcbGetPacket             = &GetPacket;
    ecfg->EMACCoreConfig.pfcbFreePacket            = &FreePacket;
    ecfg->EMACCoreConfig.pfcbRxPacket              = &RxPacket;
    ecfg->EMACCoreConfig.pfcbStatus                = &StatusUpdate;
    ecfg->EMACCoreConfig.pfcbStatistics            = &StatisticsUpdate;

    /* Total 1 MAC addresses allocated */
    ecfg->EMACCoreConfig.NumOfMacAddrs = 1;
    /* selects CPPI RAM for Descriptor memory */
    ecfg->EMACCoreConfig.DescBase = EMAC_DESC_BASE_CPPI;

    ecfg->EMACCoreConfig.MacAddr = (EMAC_AddrConfig **)
                                   malloc(ecfg->EMACCoreConfig.NumOfMacAddrs * sizeof(EMAC_AddrConfig *));

    for (j=0; j<ecfg->EMACCoreConfig.NumOfMacAddrs; j++){
        ecfg->EMACCoreConfig.MacAddr[j] = (EMAC_AddrConfig *)malloc(sizeof(EMAC_AddrConfig));
    }

    for(j=0; (Uint8)j<(ecfg->EMACCoreConfig.NumOfMacAddrs); j++)
    {
        addrCfg = ecfg->EMACCoreConfig.MacAddr[j];
        addrCfg->ChannelNum = j;
        for (i=0; i<6; i++)
        {
            addrCfg->Addr[i] = j * 0x10 + i + 0x60*instNum;
        }
    }

    /* Open EMAC instance */
    if (instNum == CSL_EMAC_0) {
        i = EMAC_open( instNum, (Handle)0x12345678, ecfg, (Handle)&emacPort[0], (Handle)&emacCore[0]);
	}
    else if (instNum == CSL_EMAC_1) {
        i = EMAC_open( instNum, (Handle)0x12345678, ecfg, (Handle)&emacPort[1], (Handle)&emacCore[1]);
	}

	return i;
}

/*
 * =============================================================================
 *   @func   DualEMACSendReceive
 *
 *   @desc
 *      Packet Transmit between two EMACs
 *      
 *
 *   @arg
 *      txInstNum: Peripheral ID of transmit EMAC
 *      ecfgRx: pointer for receive EMAC's configuration structure
 *      
 *   @eg
 *      int DualEMACSendReceive(int txInstNum, EMAC_Config *ecfgRx); // Refer description
 * =============================================================================
 */
int DualEMACSendReceive(int txInstNum, EMAC_Config *ecfgTx, EMAC_Config *ecfgRx)
{
	EMAC_AddrConfig *addrCfg;
	Uint32          i, size;
    Uint32          TxCount=0;
	Uint32		    RxCount=0;
	Uint32		    count = (1514-460)+1;
    EMAC_Pkt        *pPkt;

  	addrCfg = ecfgRx->EMACCoreConfig.MacAddr[0];

	for( i=0; i<6; i++ ) {
		packet_sourcedata[i] = addrCfg->Addr[i];
	}

  	addrCfg = ecfgTx->EMACCoreConfig.MacAddr[0];

	for( i=6; i<12; i++ ) {
		packet_sourcedata[i] = addrCfg->Addr[i-6];
	}

    LocalTicks = 0;
    while( RxCount<count ) {
        /* Check for timeout */
	  	if(LocalTicks > 500) {
	   	    printf("ERROR: Timeout waiting for TX/RX\n");
        	break;
		}

        /* See if we should send a packet */
        if( TxCount<count && TxCount<(RxCount+8) ) {
            /* First get a packet buffer from our private queue */
            pPkt = App_pqPop( &FreeQueue );
            if( pPkt ) {
                /* Using 1200 for size */
				size = 1200;
                memcpy( pPkt->pDataBuffer, packet_sourcedata, size);

                /* Send the packet */
                pPkt->Flags      = EMAC_PKT_FLAGS_SOP | EMAC_PKT_FLAGS_EOP;
                pPkt->ValidLen   = size;
                pPkt->DataOffset = 0;

				switch(coreNum) {
					default:
					case 0:pPkt->PktChannel = 0; break; /* core 0 use channel 0 */
					case 1:pPkt->PktChannel = 1; break; /* core 1 use channel 1 */
					case 2:pPkt->PktChannel = 2; break; /* core 2 use channel 2 */
				}

                pPkt->PktLength  = size;
                pPkt->PktFrags   = 1;

                /*
                 * IMPORTANT: If our data packet were in EXTERNAL memory,
                 * we would have to clean it from the CACHE before submitting
                 * it to the EMAC module!!!
                 *
                 * We should probably do the clean operation right here.
                 */

                /* Use our re-entrancy gate to call EMAC functions */
                OUREMAC_enter();

			    if (txInstNum == CSL_EMAC_0) {
			    	i = EMAC_sendPacket((Handle)&emacPort[0], pPkt);
				}
			    else if (txInstNum == CSL_EMAC_1) {
			    	i = EMAC_sendPacket((Handle)&emacPort[1], pPkt);
				}

                OUREMAC_exit();
                if(i) {
                    printf("EMAC_sendPacket() returned error %08x\n",i);
                    break;
                }
                /* Bump the TX count */
                TxCount++;
            }
        }

        /*
         * See if we received a packet
         */
        if( pPkt = App_pqPop( &RxQueue ) ) {
            /* Validate this packet */
            if( !verify_packet( pPkt, 1200) ) {
                printf("ERROR: Verify failure on packet %d\n",RxCount);
                break;
            }

            /* Free the packet */
            App_pqPush( &FreeQueue, pPkt );

            /* Bump the RX count */
            RxCount++;
        }
    }

    printf("Sent %d of %d packets, received %d packets\n\n",
            TxCount, count, RxCount);

    if( TxCount == count && RxCount == count )
        return 0;
    else 
        return 1;
}

/*
 * =============================================================================
 *   @func   Hw0RxInt
 *
 *   @desc
 *      Interrupt service routine called as a result of the EMAC0 hardware Rx
 *      interrupt.
 *
 *   @arg
 *      None
 *      
 *   @eg
 *      Hw0RxInt(); // Refer description
 * =============================================================================
 */
void interrupt Hw0RxInt(void)
{
    Uint32 i;

    if (emacPort[0].DevMagic)
	{
        i = EMAC_RxServiceCheck((Handle)&emacPort[0]);
        if (i) 
        {
            return;
         	// printf("EMAC_RxServiceCheck() returned error %08x\n",i);
        }
	}
    return;
}

/*
 * =============================================================================
 *   @func   Hw0TxInt
 *
 *   @desc
 *      Interrupt service routine called as a result of the EMAC0 hardware Tx
 *      interrupt.
 *
 *   @arg
 *      None
 *      
 *   @eg
 *      Hw0TxInt(); // Refer description
 * =============================================================================
 */
void interrupt Hw0TxInt(void)
{
    Uint32 i;
    if (emacPort[0].DevMagic)
	{
        i = EMAC_TxServiceCheck((Handle)&emacPort[0]);
        if (i) 
        {
            return;
         	// printf("EMAC_TxServiceCheck() returned error %08x\n",i);
        }
	}
    return;
}

/*
 * =============================================================================
 *   @func   Hw1RxInt
 *
 *   @desc
 *      Interrupt service routine called as a result of the EMAC1 hardware Rx
 *      interrupt.
 *
 *   @arg
 *      None
 *      
 *   @eg
 *      Hw1RxInt(); // Refer description
 * =============================================================================
 */
void interrupt Hw1RxInt(void)
{
    Uint32 i;

    if (emacPort[1].DevMagic)
	{
        i = EMAC_RxServiceCheck((Handle)&emacPort[1]);
        if (i) 
        {
            return;
         	// printf("EMAC_RxServiceCheck() returned error %08x\n",i);
        }
	}
    return;
}

/*
 * =============================================================================
 *   @func   Hw1TxInt
 *
 *   @desc
 *      Interrupt service routine called as a result of the EMAC1 hardware Tx
 *      interrupt.
 *
 *   @arg
 *      None
 *      
 *   @eg
 *      HwTx1Int(); // Refer description
 * =============================================================================
 */
void interrupt Hw1TxInt(void)
{
    Uint32 i;
    if (emacPort[1].DevMagic)
	{
        i = EMAC_TxServiceCheck((Handle)&emacPort[1]);
        if (i) 
        {
            return;
         	// printf("EMAC_TxServiceCheck() returned error %08x\n",i);
        }
	}
    return;
}

/*
 * =============================================================================
 *   @func   TimerTick
 *
 *   @desc
 *      Timer tick service routine - called from 100mS PRD
 *
 *   @arg
 *      None
 *      
 *   @eg
 *      TimerTick(); // Refer description
 * =============================================================================
 */
void interrupt TimerTick(void)
{

   	/* Keep track of time for our local timeout loops */
    LocalTicks++;
    /* Poll the EMAC */
    if (emacPort[0].DevMagic)
	{
        EMAC_timerTick((Handle)&emacPort[0]);
	}
    if (emacPort[1].DevMagic)
	{
		EMAC_timerTick((Handle)&emacPort[1]);
	}
}
/*
 * =============================================================================
 *   @func   GetPacket - Get empty packet for RX.
 *
 *   @desc
 *      This function is called from the EMAC module to get an empty packet 
 *      buffer. We need to return a packet buffer to the EMAC or return NULL
 *      if there are no buffers available.
 *
 *   @arg
 *      hApplication    Handle to the application
 *      
 *   @eg
 *      Handle  hApp;
 *      ... // Initialize and get a value to handle- done elsewhere.
 *      GetPacket(hApp); // Refer description
 *      ...
 * =============================================================================
 */
static EMAC_Pkt *GetPacket( Handle hApplication )
{
    EMAC_Pkt *pPkt;

    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        //printf("GetPacket(): Bad App Handle!\n");
        return(0);
    }

    /* Pop a packet off our local free queue */
    pPkt = App_pqPop(&FreeQueue);

    if( pPkt ) {
        /*
         * Here we tell the EMAC what offset to use by setting
         * the DataOffset field in the packet.
         */
        pPkt->DataOffset = 0;

        /*
         * IMPORTANT: If our data packet were in EXTERNAL memory,
         * we would have to invalidated it from the CACHE before
         * returning it to the EMAC module!!!
         *
         * We should probably do the clean operation right here.
         * However, we are most likely inside an ISR right now.
         * It may be better to invalidate the buffer before it
         * is pushed on the FreeQueue.
         */
    }
    return( pPkt );
}
/*
 * =============================================================================
 *   @func   FreePacket
 *
 *   @desc
 *      Free packet that originated from TX or GetPacket(). This function is
 *      called from the EMAC module to free a packet buffer after a TX 
 *      operation (or RX in the case of a RX shutdown).
 *
 *   @arg
 *      hApplication    handle to the application
 *      pPKT            pointer to the EMAC packet
 *      
 *   @eg
 *      Handle   hApp;
 *      EMAC_Pkt *emacPkt;
 *      ...
 *      FreePacket(hApp, emacPkt); // Refer description
 *      ...
 * =============================================================================
 */
static void FreePacket( Handle hApplication, EMAC_Pkt *pPKT )
{
    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        return;
    }
    App_pqPush( &FreeQueue, pPKT );
}

/*
 * 
 *
 */
/*
 * =============================================================================
 *   @func   RxPacket - Received packet from the Network.
 *
 *   @desc
 *      This function is called by the EMAC to indicate a receive. Here
 *      we just push the packet onto our receive queue.
 *      
 *      This function returns a free packet to replace the RX packet on
 *      the EMAC queue. If there are no free packets available, we
 *      return NULL.
 *
 *   @arg
 *      hApplication    handle to the application
 *      pPKT            pointer to the EMAC packet
 *      
 *   @eg
 *      Handle   hApp;
 *      EMAC_Pkt *emacPkt;
 *      ...
 *      RxPacket(hApp, emacPkt); // Refer description
 *      ...
 * =============================================================================
 */
static EMAC_Pkt *RxPacket( Handle hApplication, EMAC_Pkt *pPKT )
{
    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        return(0);
    }

    /* Push this packet onto our local receive queue */
    App_pqPush( &RxQueue, pPKT );

    /* We must return a free packet to replace the one we were given */
    return( GetPacket(hApplication) );
}

/*
 * =============================================================================
 *   @func   StatusUpdate
 *
 *   @desc
 *      The EMAC or MDIO status has changed 
 *
 *   @arg
 *      hApplication    handle to the application
 *      
 *   @eg
 *      Handle   hApp;
 *      ...
 *      StatusUpdate(hApp); // Refer description
 *      ...
 * =============================================================================
 */
static void StatusUpdate( Handle hApplication )
{

    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        return;
    }

    /*
     * The status update function is called for a variety of
     * reasons, including LINK change events and EMAC errors.
     * We'll get the current status and print it out. Note we
     * don't start sending packets until we have a good link.
     *
     */
    EMAC_getStatus((Handle)&emacPort[0], &status[0]);
    EMAC_getStatus((Handle)&emacPort[1], &status[1]);

    LinkStatus[0] = status[0].MdioLinkStatus;
    LinkStatus[1] = status[1].MdioLinkStatus;
}

/*
 * =============================================================================
 *   @func   StatisticsUpdate
 *
 *   @desc
 *      The EMAC statistics are in danger of overflow
 *
 *   @arg
 *      hApplication    handle to the application
 *      
 *   @eg
 *      Handle   hApp;
 *      ...
 *      StatisticsUpdate(hApp); // Refer description
 *      ...
 * =============================================================================
 */
static void StatisticsUpdate( Handle hApplication )
{
    /* Verify our handle came back OK. We don't use it in this example */
    if( (Uint32)hApplication != 0x12345678 ) {
        return;
    }

    /*
     * Here we could call the EMAC_getStatistics() function and pass
     * them to whoever wants them. This example app doesn't have
     * anything to do with them so we do nothing.
     */
}

/*
 * =============================================================================
 *   @func   verify_packet()
 *
 *   @desc
 *      Perform a sanity check validation on the packet
 *
 *   @arg
 *      pPkt    EMAC packet
 *      size    size to validate
 *      
 *   @eg
 *      EMAC_Pkt    *pkt;
 *      Uint32      size;
 *      ... ; // variables to get value elsewhere.
 *      verify_packet(pkt, size);
 *      ...
 * =============================================================================
 */
static int verify_packet( EMAC_Pkt *pPkt, Uint32 size )
{
    int i;
    Uint8 *pBuf;

    if( (pPkt->Flags & (EMAC_PKT_FLAGS_SOP | EMAC_PKT_FLAGS_EOP)) !=
        (EMAC_PKT_FLAGS_SOP | EMAC_PKT_FLAGS_EOP) ) {
        printf("Verify: Bad Flags %08x\n",pPkt->Flags);
        return(0);
    }

    if( pPkt->ValidLen != size ) {
        printf("Verify: Bad ValidLen %d %d\n",pPkt->ValidLen,size);
        return(0);
    }

    if( pPkt->DataOffset != 0 ) {
        printf("Verify: Bad DataOffset %d\n",pPkt->DataOffset);
        return(0);
    }


    if( pPkt->PktChannel != coreNum ) {
        printf("Verify: Bad Channel %d\n",pPkt->PktChannel);
        return(0);
    }

    if( pPkt->PktLength != size ) {
        printf("Verify: Bad PktLength %d\n",pPkt->PktLength);
        return(0);
    }

    if( pPkt->PktFrags != 1 ) {
        printf("Verify: Bad Frag Count %d\n",pPkt->PktFrags);
        return(0);
    }

    pBuf = pPkt->pDataBuffer+pPkt->DataOffset;
	pBuf += 12;
    for( i=12; i<size-12; i++, pBuf++ )
        if( *pBuf != (i&0xFF) ) {
            printf("Verify: Bad data at %d (%d)(%d)\n",i,*pBuf,i&0xff);
            return(0);
        }

    return(1);
}

/* ============================================================================
 *  @func App_pqPop()
 *
 *  @desc
 *      Pop a desc buffer off a queue
 *
 *   @arg
 *      pq    Application packet
 *      
 *   @eg
 *      APP_PKTQ    *pq;
 *      ... ; // variables to get value elsewhere.
 *      App_pqPop(pq);
 *      ...
 * ============================================================================
 */
static EMAC_Pkt *App_pqPop( APP_PKTQ *pq )
{
    EMAC_Pkt *pPktHdr;
    //Uint32   intmask;

   //intmask = C64_disableIER( EMAC_INT_FLAG );
   OUREMAC_enter();

    pPktHdr = pq->pHead;

    if( pPktHdr ) {
        pq->pHead = pPktHdr->pNext;
        pq->Count--;
        pPktHdr->pPrev = pPktHdr->pNext = 0;
    }

    //C64_enableIER( intmask );
	OUREMAC_exit();

    return( pPktHdr );
}

/* ============================================================================
 *  @func App_pqPush()
 *
 *  @desc
 *      Push a desc buffer onto a queue
 *
 *   @arg
 *      pq      Application packet
 *      pPktHdr pointer to EMAC packet header
 *      
 *   @eg
 *      APP_PKTQ    *pq;
 *      EMAC_Pkt    *pPktHdr;
 *      ... ; // variables to get value elsewhere.
 *      App_pqPush(pq, pPktHdr);
 *      ...
 * ============================================================================
 */
static void App_pqPush( APP_PKTQ *pq, EMAC_Pkt *pPktHdr )
{
    //Uint32   intmask;

    //intmask = C64_disableIER( EMAC_INT_FLAG );
	OUREMAC_enter();

    pPktHdr->pNext = 0;

    if( !pq->pHead ) {
        /* Queue is empty - Initialize it with this one packet */
        pq->pHead = pPktHdr;
        pq->pTail = pPktHdr;
    }
    else {
        // Queue is not empty - Push onto END
        pq->pTail->pNext = pPktHdr;
        pq->pTail        = pPktHdr;
    }
    pq->Count++;

    //C64_enableIER( intmask );
	OUREMAC_exit();
}

void enableEMAC(int instNum)
{
    CSL_PscObj pscObj;
	CSL_PscHandle hPsc;
	CSL_Status status;
	CSL_PscPowerDomain pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
	Uint32 response;
    CSL_PscPeripherals module;

	memset(&pscObj, 0, sizeof(CSL_PscObj));      

    /* Init PSC module */
	CSL_pscInit(NULL);

    /* Open PSC module to get a Handle */
	hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status);

    /* Enable clock for the specified module SRIO */
    if (instNum == CSL_EMAC_0) {
       module = CSL_PSC_MODULE_EMAC0;
    }
    else if (instNum == CSL_EMAC_1) {
       module = CSL_PSC_MODULE_EMAC1;
    }  
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
	/*Enable SRIO Power domain GO transition */
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);

	do{
    	CSL_pscGetHwStatus(hPsc, CSL_PSC_QUERY_PWRDMN_TRANS_STAT, (void *)&response);
		response &= (1 << CSL_PSC_PWRDMN_ALWAYSON);
	}while(response != 0x0);

	/* Close the PSC module */
	CSL_pscClose(hPsc);    
}

