/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005,2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** ============================================================================
 *   @file  Pllc_example.c
 *
 *  @path $(CSLPATH)\example\pllc\src
 *
 *   @desc  Example of PLLC module CSL
 *
 *  ============================================================================
 *  ============================================================================
 *   @n <b> Example Description </b>
 *   @n The example shows PLLC module CSL usage
 *      1. Initalize the PLLC hardware setup data structure with default values
 *      2. Intialize PLLC module
 *      3. Set new values for divider 10
 *      4. Set the value using hardware control command
 *      5. Close the module
 *  ============================================================================
 *
 *   <b> Procedure to run the example </b>
 *   @verbatim
 *      1. Launch CCS window.
 *      2. Open project pllc_example.pjt.
 *      3. Build the project and load the .out file of the project.
 *
 *   @endverbatim
 *
 *
 *  ============================================================================
 */

/*  ============================================================================
 *  Revision History
 *  ===============
 *  17-May-2006 RR  File created  for Tomahawk
 *
 *  ============================================================================
 */
#include <stdio.h>
#include <string.h>
#include <csl_pllc.h>

/* PLLC module handle structure */
CSL_PllcHandle           hPllc = NULL;

/* forward declarations*/
void pllc_csl_example (void);
void pllc_error_exit (void);
/*
 *  ============================================================================
 *  @func   main
 *
 *  @desc
 *     This is the main routine for the file.
 *
 *  @arg
 *      NONE
 *
 *  @return
 *      NONE
 *  ============================================================================
*/
void main (
    void
)
{
    /* Call the example routine */ 
    pllc_csl_example ();

}

/*
 *  ============================================================================
 *  @func   pllc_csl_example
 *
 *  @desc
 *      1. Initalize the PLLC hardware setup data structure with default values
 *      2. Intialize PLLC module
 *      3. Set new values for divider 13
 *      4. Set the value using hardware control command
 *      5. Close the module
 *
 *  @arg
 *      NONE
 *
 *  @return
 *      NONE
 *  ============================================================================
*/
void pllc_csl_example (
    void
)
{
    CSL_PllcObj              pllcObj;
    CSL_Status               status;
    CSL_PllcHwSetup          hwSetup;
    CSL_PllcHwSetup          hwSetupRead;
    CSL_PllcDivideControl    arg;
    CSL_PllcDivRatio         arg1;
    CSL_BitMask32            response;   
    CSL_BitMask32            argPllCtrl;    

    /* Initialize the PLLC CSL module */
    status = CSL_pllcInit(NULL);

    if (status != CSL_SOK) {
        printf ("PLLC: Initialization... Failed.\n");
        printf ("\tReason: CSL_pllcInit failed. [status = 0x%x].\n", status);
        return;
    }

    /* Clear local data structures */
    memset(&pllcObj, 0, sizeof(CSL_PllcObj));

    /* Open the PLLC CSL module */
    hPllc = CSL_pllcOpen (&pllcObj, CSL_PLLC_0, NULL, &status);

    if ((hPllc == NULL) || (status != CSL_SOK)) {
        printf ("PLLC: Open... Failed.\n");
        printf ("\tReason: Error opening the instance. [status = 0x%x, "\
                "hPllc = 0x%x].\n", status, hPllc);
        return;
    }

    /* Setup PLLC hardware parameters */
    hwSetup.divEnable  = (CSL_BitMask32) CSL_PLLC_DIVEN_PLLDIV10 ;

    hwSetup.pllM       = (Uint32)        0x000000015;
    hwSetup.pllDiv10    = (Uint32)        0x00000002;

    status = CSL_pllcHwSetup (hPllc, &hwSetup);

    if (status != CSL_SOK) {
        printf ("PLLC: HwSetup... Failed.\n");
        printf ("\tReason: Error setting up the hardware."\
                " [status = 0x%x, hPllc = 0x%x].\n", status, hPllc);
        pllc_error_exit ();
        return;
    }

    /* Read back */

    status = CSL_pllcGetHwSetup (hPllc, &hwSetupRead);

    if (status != CSL_SOK) {
        printf ("PLLC: Hardware setup parameters reading... Failed.\n");
        printf ("\tReason: Error setting in hardware validation."\
                " [status = 0x%x, hPllc = 0x%x].\n", status, hPllc);
        pllc_error_exit ();

    }

    
    /* Wait if another GO operation is currently in progress.*/
    do {
        CSL_pllcGetHwStatus (hPllc, CSL_PLLC_QUERY_STATUS, &response);
    } while (response & CSL_PLLC_STATUS_GO);

    /* Command to enable PLLDIV10 */
    arg.divNum = CSL_PLLC_DIVSEL_PLLDIV10;
    arg.divCtrl = CSL_PLLC_PLLDIV_ENABLE;

    status = CSL_pllcHwControl (hPllc, CSL_PLLC_CMD_PLLDIV_CONTROL,
                                   &arg);
    if (status != CSL_SOK) {
        printf ("PLLC: HwControl for PLL Divider Control... Failed.\n");
        printf ("\tReason: Error in Divider Control."\
                " [status = 0x%x, hPllc = 0x%x].\n", status, hPllc);
        pllc_error_exit ();
        return;
    }

    /* Command to change PLLDIV10 Divider Ratio */
    arg1.divNum = CSL_PLLC_DIVSEL_PLLDIV10;    
    arg1.divRatio = 0x00000008;

    status = CSL_pllcHwControl (hPllc, CSL_PLLC_CMD_SET_PLLRATIO,
                                   &arg1);

    if ((status != CSL_SOK) || (status == CSL_ESYS_INVPARAMS))
    {
        printf ("PLLC: HwControl for PLL Divider Ratio Set Command "\
                "Control... Failed.\n");
        printf ("\tReason: Error in Control for Divider Ratio Set Command."\
                " [status = 0x%x, hPllc = 0x%x].\n", status, hPllc);
        pllc_error_exit ();
        return;
    }

    /* Set GOSET bit to change/set the dividers */
    argPllCtrl = 0x010000;   /* value to set the GOSET bit */ 
    status = CSL_pllcHwControl (hPllc, CSL_PLLC_CMD_PLLCONTROL,
                                   &argPllCtrl);
    if (status != CSL_SOK) {
        printf ("PLLC: HwControl for PLL command control... Failed.\n");
        printf ("\tReason: Error in PLL Command Control."\
                " [status = 0x%x, hPllc = 0x%x].\n", status, hPllc);
        pllc_error_exit ();
        return;
    }

    /* Poll for divide-ratio change and clock alignment to complete */
    do {
        CSL_pllcGetHwStatus (hPllc, CSL_PLLC_QUERY_STATUS, &response);
    } while (response & CSL_PLLC_STATUS_GO);

    /* Close the module */
    status = CSL_pllcClose (hPllc);
    if (status != CSL_SOK) {
        printf ("PLLC: Close the instance ... Failed\n");
    }

    printf("PLLC example complete \n");

    return;
}

/*
 *  ============================================================================
 *  @func   pllc_error_exit
 *
 *  @desc
 *     This calls the module close function in case of error
 *
 *  @arg
 *      NONE
 *
 *  @return
 *      NONE
 *  ============================================================================
 */
void pllc_error_exit (
    void
)
{
    CSL_Status               status;

    /* Closes the module */
    status = CSL_pllcClose (hPllc);
    if (status != CSL_SOK) {
        printf ("PLLC: Close Pllc instance... Failed\n");
    }
}
