/* ============================================================================
 * Copyright (c) Texas Instruments Incorporated 2010
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ===========================================================================
 *   @file Srio_serdes_loopbk_example.c
 *
 *   @path  $(CSLPATH)\example\srio\srio_serdes_loopbk_example\src
 *
 *   @desc This is a simple serdes lookback example of usage of CSL APIs 
 *         of the SRIO module.
 *
 *  ============================================================================
 *  ============================================================================
 *
 *   @n <b> Example Description </b>
 *   @n This is an example of the CSL SRIO usage for a LSU port.
 *   @verbatim
       1. Setup SRIO module for serdes loop back transfer on LSU0.
       2. Setup the destination and source data areas.
       3. Configure LSU0 registers.
       4. Start transfer.
       5. Wait for completion of the transfer.
       6. Verify the destination contains the expected data.

     @endverbatim
 *
 *=============================================================================
 *      
 *   <b> Procedure </b>
 *   @verbatim
      1. Configure the CCS setup
      2. Please refer CCS manual for setup configuration and loading 
         proper GEL file
      3. Launch CCS window
      4. Open project Srio_serdes_loopbk_example.pjt
      5. Build the project and load the .out file of the project.

     @endverbatim
 *       
 *   
 * ============================================================================
 */
#include <csl_srio.h>
#include <csl_srioAux.h>
#include <csl_psc.h>
#include <cslr_dev.h>
#include <soc.h>
#include <stdio.h>
#include <string.h>

/***********************************************************************
 ************************** LOCAL Definitions **************************
 ***********************************************************************/

/* Device Identifiers (8bit and 16bit) for the Source Application. */
#define LARGE_DEV_ID         0xBEEF
#define SMALL_DEV_ID         0xAB

/* RIO NWrite_R Command Identifier.  */
#define SRIO_PKT_TYPE_NWRITER 0x55

/* SRIO Port Number which is being used. */
#define SRIO_PORT_NUMBER     1

/* LSU Number which is being used. */
#define SELECTED_LSU         0

/* Length of data being transferred. */
#define TRANSFER_SIZE        4096

#define REQ_MAINT_WR         0x81 //0b10000001  // ftype=8


/************************************************************************
 ************************** GLOBAL Definitions **************************
 ************************************************************************/

/* Global Source and Destination buffers. */
static Uint8 src[TRANSFER_SIZE];
static Uint8 dst[TRANSFER_SIZE];

/* Global SRIO Hardware Setup is used */
static CSL_SrioHwSetup setup = CSL_SRIO_HWSETUP_DEFAULTS;

/************************************************************************
 ************************** LSU Example Functions ***********************
 ************************************************************************/

/*
 * ============================================================================
 *   @func   delay
 *
 *   @desc
 *     This routine is used to spin in a delay loop for delay iterations
 * ============================================================================
 */
static void delay(Uint32 delay)
{
    volatile Uint32 i, n;

    n = 0;
    for (i = 0; i < delay; i++)
    {
        n = n + 1;
    }
}

/*
 * ============================================================================
 *   @func   enableSRIO
 *
 *   @desc
 *     This routine is used to power up the SRIO IP Module.
 * ============================================================================
 */
static void enableSRIO()
{
    CSL_PscObj pscObj;
	CSL_PscHandle hPsc;
	CSL_Status status;
	CSL_PscPowerDomain pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
	CSL_PscPeripherals module = CSL_PSC_MODULE_SRIO;
	Uint32 response = 0;

	memset(&pscObj, 0, sizeof(CSL_PscObj));      
    
    /* Init PSC module */
	CSL_pscInit(NULL);

    /* Open PSC module to get a Handle */
	hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &status);

    /* Enable clock for the specified module SRIO */
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
	/*Enable SRIO Power domain GO transition */
	CSL_pscHwControl(hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);

	do{
    	CSL_pscGetHwStatus(hPsc, CSL_PSC_QUERY_PWRDMN_TRANS_STAT, (void *)&response);
		response &= (1 << CSL_PSC_PWRDMN_ALWAYSON);
	}while(response != 0x0);

	/* Close the PSC module */
	CSL_pscClose(hPsc);
}


/*
 * ============================================================================
 *   @func   srio_Create_Setup
 *
 *   @desc
 *     This routine sets up a structure with the required setup to initialize
 *     the SRIO IP block.
 * ============================================================================
 */
static void srio_Create_Setup (CSL_SrioHwSetup *pSetup)
{
    Uint32 index;
    
    /* Peripheral enable */
    pSetup->perEn = 1;

    /* All packets, regardless of the destination ID, are forwarded to the application */
    pSetup->periCntlSetup.logicalLayerDisable = 1;

    /* While in shutdown, put memories in sleep mode */
    pSetup->periCntlSetup.swMemSleepOverride = 1;
    
    /* Use SerDes lookback mode. The configured SerDes loopback mode will be
       enabled for all SerDes channels which have ENTEST = 1 */
    pSetup->serDesLoopback = 1;

    /* Boot process is over (complete) */
    pSetup->periCntlSetup.bootComplete = 1;
    
    /* Process TX requests of priority 2, when credit is 1 or greater */
    pSetup->periCntlSetup.txPriority2Wm = CSL_SRIO_TX_PRIORITY_WM_1;
    
    /* Process TX requests of priority 1, when credit is 1 or greater */
    pSetup->periCntlSetup.txPriority1Wm = CSL_SRIO_TX_PRIORITY_WM_2;

    /* Process TX requests of priority 0, when credit is 1 or greater */
    pSetup->periCntlSetup.txPriority0Wm = CSL_SRIO_TX_PRIORITY_WM_3;

    /* Set internal bus priority to 4 */
    pSetup->periCntlSetup.busTransPriority = CSL_SRIO_BUS_TRANS_PRIORITY_4;
    
    /* UDI buffers are port based not proirity based */
    pSetup->periCntlSetup.bufferMode = CSL_SRIO_1X_MODE_PORT;
    
    /* VBUS clock is of 333 MHz  from TPCC document section 2.3 */
    pSetup->periCntlSetup.prescalar = CSL_SRIO_CLK_PRESCALE_1;

    /* Enable clocks to all domains */
    pSetup->gblEn = 1;

    /* Enable clock in all the domains */
    for (index=0; index<9; index++)
        pSetup->blkEn[index] = 1;

    /* 8-bit id is 0xAB and 16-bit id is 0xBEEF */
    pSetup->deviceId1 = CSL_FMK(SRIO_BASE_ID_BASE_DEVICEID, SMALL_DEV_ID) | 
                        CSL_FMK(SRIO_BASE_ID_LARGE_BASE_DEVICEID, LARGE_DEV_ID);

    /* 8-bit id is 0xAB and 16-bit id is 0xBEEF for multi-cast*/
    pSetup->deviceId2 = CSL_FMK(SRIO_BASE_ID_BASE_DEVICEID, SMALL_DEV_ID) | 
                        CSL_FMK(SRIO_BASE_ID_LARGE_BASE_DEVICEID, LARGE_DEV_ID);

	/* SERDES PLL configuration */
	pSetup->serDesPllCfg.pllEnable      = TRUE;
	pSetup->serDesPllCfg.pllMplyFactor  = CSL_SRIO_SERDES_PLL_MPLY_BY_12_5;
	pSetup->serDesPllCfg.loopBandwidth  = CSL_SRIO_SERDES_LOOP_BANDWIDTH_FREQ_DEP;

    /* SERDES RX channel 0 enable */
    pSetup->serDesRxChannelCfg[0].enRx              = TRUE; 
    pSetup->serDesRxChannelCfg[0].enTest            = TRUE;  /* Set enTest=1 to enable Serdes loopback for the channel */
    pSetup->serDesRxChannelCfg[0].symAlign          = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
	pSetup->serDesRxChannelCfg[0].los               = CSL_SRIO_SERDES_LOS_DET_DISABLE;
	pSetup->serDesRxChannelCfg[0].clockDataRecovery = 0x00;  /* first order */
	pSetup->serDesRxChannelCfg[0].equalizer         = 0x01;
    pSetup->serDesRxChannelCfg[0].busWidth          = CSL_SRIO_SERDES_BUS_WIDTH_10_BIT;
    pSetup->serDesRxChannelCfg[0].rate              = CSL_SRIO_SERDES_RATE_FULL;

    /* SERDES TX channel 0 enable */
    pSetup->serDesTxChannelCfg[0].enTx              = TRUE;
    pSetup->serDesTxChannelCfg[0].enTest            = TRUE;  /* Set enTest=1 to enable Serdes loopback for the channel */
    pSetup->serDesTxChannelCfg[0].commonMode        = CSL_SRIO_SERDES_COMMON_MODE_RAISED;
    pSetup->serDesTxChannelCfg[0].outputSwing       = CSL_SRIO_SERDES_SWING_AMPLITUDE_750;
    pSetup->serDesTxChannelCfg[0].rate              = CSL_SRIO_SERDES_RATE_FULL; 

    /* SERDES RX channel 1 enable */
    pSetup->serDesRxChannelCfg[1].enRx              = TRUE; 
    pSetup->serDesRxChannelCfg[1].enTest            = TRUE;  /* Set enTest=1 to enable Serdes loopback for the channel */
    pSetup->serDesRxChannelCfg[1].symAlign          = CSL_SRIO_SERDES_SYM_ALIGN_COMMA; 
	pSetup->serDesRxChannelCfg[1].los               = CSL_SRIO_SERDES_LOS_DET_DISABLE;
	pSetup->serDesRxChannelCfg[1].clockDataRecovery = 0x00;  /* first order */
	pSetup->serDesRxChannelCfg[1].equalizer         = 0x01;
    pSetup->serDesRxChannelCfg[1].busWidth          = CSL_SRIO_SERDES_BUS_WIDTH_10_BIT;
    pSetup->serDesRxChannelCfg[1].rate              = CSL_SRIO_SERDES_RATE_FULL;

    /* SERDES TX channel 1 enable */
    pSetup->serDesTxChannelCfg[1].enTx              = TRUE; 
    pSetup->serDesTxChannelCfg[1].enTest            = TRUE;  /* Set enTest=1 to enable Serdes loopback for the channel */
    pSetup->serDesTxChannelCfg[1].commonMode        = CSL_SRIO_SERDES_COMMON_MODE_RAISED;
    pSetup->serDesTxChannelCfg[1].outputSwing       = CSL_SRIO_SERDES_SWING_AMPLITUDE_750;
    pSetup->serDesTxChannelCfg[1].rate              = CSL_SRIO_SERDES_RATE_FULL; 

    /* Select flow control ID length 16-bit */
    pSetup->flowCntlIdLen[0] = 1; 

    /* Destination ID of flow n, same ids as we are doing loopback */
    pSetup->flowCntlId[0] = LARGE_DEV_ID; 

    /* Sets the number of address bits generated by the PE as a source and 
     * processed by the PE as the target of an operation as 34 bits */
    pSetup->peLlAddrCtrl = CSL_SRIO_ADDR_SELECT_34BIT;

    /* Base device configuration */
    pSetup->devIdSetup.smallTrBaseDevId  = SMALL_DEV_ID;
    pSetup->devIdSetup.largeTrBaseDevId  = LARGE_DEV_ID;
    pSetup->devIdSetup.hostBaseDevId     = LARGE_DEV_ID;

    /* Port General configuration */
    pSetup->portGenSetup.portLinkTimeout = 0xFFFFFF;  /* 3.4 s */
    pSetup->portGenSetup.portRespTimeout = 0xFFFFFF;  /* 3.4 s */
    pSetup->portGenSetup.hostEn          = 1;        /* Agent/Slave Device           */
    pSetup->portGenSetup.masterEn        = 1;        /* Device cannot issue requests */

    /* Port control configuration for PORT 0 */
    pSetup->portCntlSetup[0].portDis           = 0;         /* Do not disable Port 0 */
    pSetup->portCntlSetup[0].outPortEn         = 1;         /* Output on Port 0 enabled */
    pSetup->portCntlSetup[0].inPortEn          = 1;         /* Input on Port 0 enabled */
    pSetup->portCntlSetup[0].portWidthOverride = CSL_SRIO_PORT_WIDTH_NO_OVERRIDE;
    pSetup->portCntlSetup[0].errCheckDis       = 0;         /* Err check enabled */
    pSetup->portCntlSetup[0].multicastRcvEn    = 0;         /* Multicast receive Disabled */
    pSetup->portCntlSetup[0].stopOnPortFailEn  = 0;         /* Dont Stop on fail */
    pSetup->portCntlSetup[0].dropPktEn         = 0;         /* Dont Drop PKT */
    pSetup->portCntlSetup[0].portLockoutEn     = 0;         /* Send any PKT */

    /* Port control configuration for PORT 1 */
    pSetup->portCntlSetup[1].portDis           = 0;        /* Do not disable Port 1 */
    pSetup->portCntlSetup[1].outPortEn         = 1;        /* Output on Port 1 enabled */
    pSetup->portCntlSetup[1].inPortEn          = 1;        /* Input on Port 1 enabled */
    pSetup->portCntlSetup[1].portWidthOverride = CSL_SRIO_PORT_WIDTH_NO_OVERRIDE;
    pSetup->portCntlSetup[1].errCheckDis       = 0;        /* Err check enabled */
    pSetup->portCntlSetup[1].multicastRcvEn    = 0;        /* Multicast receive Disabled. */
    pSetup->portCntlSetup[1].stopOnPortFailEn  = 0;        /* Dont Stop on fail */
    pSetup->portCntlSetup[1].dropPktEn         = 0;        /* Dont Drop PKT */
    pSetup->portCntlSetup[1].portLockoutEn     = 0;        /* Send any PKT */

    /* Enable all logical/transport errors */
    pSetup->lgclTransErrEn = CSL_SRIO_IO_ERR_RESP_ENABLE            |
                             CSL_SRIO_ILL_TRANS_DECODE_ENABLE       |
                             CSL_SRIO_ILL_TRANS_TARGET_ERR_ENABLE   |
                             CSL_SRIO_PKT_RESP_TIMEOUT_ENABLE       |
                             CSL_SRIO_UNSOLICITED_RESP_ENABLE       |
                             CSL_SRIO_UNSUPPORTED_TRANS_ENABLE;

    /* Enable all Port errors for port 0 */    
    pSetup->portErrSetup[0].portErrRateEn =
                            CSL_SRIO_ERR_IMP_SPECIFIC_ENABLE            |
                            CSL_SRIO_CORRUPT_CNTL_SYM_ENABLE            |
                            CSL_SRIO_CNTL_SYM_UNEXPECTED_ACKID_ENABLE   |
                            CSL_SRIO_RCVD_PKT_NOT_ACCPT_ENABLE          |
                            CSL_SRIO_PKT_UNEXPECTED_ACKID_ENABLE        |
                            CSL_SRIO_RCVD_PKT_WITH_BAD_CRC_ENABLE       |
                            CSL_SRIO_RCVD_PKT_OVER_276B_ENABLE          |
                            CSL_SRIO_NON_OUTSTANDING_ACKID_ENABLE       |
                            CSL_SRIO_PROTOCOL_ERROR_ENABLE              |
                            CSL_SRIO_UNSOLICITED_ACK_CNTL_SYM_ENABLE    |
                            CSL_SRIO_LINK_TIMEOUT_ENABLE;

    /* Decrement error rate counter every second */ 
    pSetup->portErrSetup[0].prtErrRtBias = CSL_SRIO_ERR_RATE_BIAS_1S;

    /* Allow only 2 errors after error threshold is reached */ 
    pSetup->portErrSetup[0].portErrRtRec = CSL_SRIO_ERR_RATE_COUNT_2;

    /*  Port error setup */ 
    pSetup->portErrSetup[0].portErrRtFldThresh   = 0xFF; /* Err threshold = 0xFF */
    pSetup->portErrSetup[0].portErrRtDegrdThresh = 0xFF; /* Err degrade threshold = 0xFF */
    
    /* Enable all Port errors for port 1*/    
    pSetup->portErrSetup[1].portErrRateEn =
                            CSL_SRIO_ERR_IMP_SPECIFIC_ENABLE            |
                            CSL_SRIO_CORRUPT_CNTL_SYM_ENABLE            |
                            CSL_SRIO_CNTL_SYM_UNEXPECTED_ACKID_ENABLE   |
                            CSL_SRIO_RCVD_PKT_NOT_ACCPT_ENABLE          |
                            CSL_SRIO_PKT_UNEXPECTED_ACKID_ENABLE        |
                            CSL_SRIO_RCVD_PKT_WITH_BAD_CRC_ENABLE       |
                            CSL_SRIO_RCVD_PKT_OVER_276B_ENABLE          |
                            CSL_SRIO_NON_OUTSTANDING_ACKID_ENABLE       |
                            CSL_SRIO_PROTOCOL_ERROR_ENABLE              |
                            CSL_SRIO_UNSOLICITED_ACK_CNTL_SYM_ENABLE    |
                            CSL_SRIO_LINK_TIMEOUT_ENABLE;

    /* Decrement error rate counter every second */ 
    pSetup->portErrSetup[1].prtErrRtBias = CSL_SRIO_ERR_RATE_BIAS_1S;

    /* Allow only 2 errors after error threshold is reached */ 
    pSetup->portErrSetup[1].portErrRtRec = CSL_SRIO_ERR_RATE_COUNT_2;

    /*  Port error setup */ 
    pSetup->portErrSetup[1].portErrRtFldThresh   = 0xFF; /* Err threshold = 0xFF */
    pSetup->portErrSetup[1].portErrRtDegrdThresh = 0xFF; /* Err degrade threshold = 0xFF */
    
    /* Configure the Port IP Mode (SP_IP_MODE) Register. */
    pSetup->portIpModeSet =
            CSL_SRIO_PW_DISABLE            |  /* Disable Port-Write Error reporting */
            CSL_SRIO_SRC_TGT_ID_DISABLE    |  /* Packet accepted with any DESTID and forwarded to the logical layer.*/
            CSL_SRIO_F8_TGT_ID_DISABLE     |  /* Regardless of Ftype8 packet destination ID, the physical layer 
                                                 accepts and handles the maintenance request packets. */
            CSL_SRIO_MLTC_ENABLE           |  /* Multicast-Event Interrupt Enable */  
            CSL_SRIO_RST_ENABLE            |  /* Reset Interrupt Enable */
            CSL_SRIO_PW_ENABLE;               /* Port-Write-In Interrupt Enable */                                                            

    /* Configure the SP_IP_PRESCALE register */
    pSetup->portIpPrescalar = 9; 

    /*  Port-Write Timer. The timer defines a period to repeat sending an error
     *  reporting Port-Write request for software assistance. The timer stopped 
     *  by software writing to the error detect registers 900 ms
     */
    pSetup->pwTimer = CSL_SRIO_PW_TIME_8;

    /* Configure port control independent error reporting. */
    pSetup->portCntlIndpEn[0].softRecovery = FALSE;/* Hardware controlled error recovery */
    pSetup->portCntlIndpEn[0].forceReInit  = FALSE;/* Reinitialization process NOT forced */
    pSetup->portCntlIndpEn[0].debug        = TRUE; /* Debug enable */
    pSetup->portCntlIndpEn[0].sendDebugPkt = FALSE;/* Do not force a debug packet */
    pSetup->portCntlIndpEn[0].illTransEn   = TRUE; /* Illegal Transfer Error reporting Enable*/
    pSetup->portCntlIndpEn[0].maxRetryEn   = TRUE; /* Max_retry_error report enable */
    pSetup->portCntlIndpEn[0].maxRetryThr  = 1;    /* Maximum Retry Threshold Trigger */
    pSetup->portCntlIndpEn[0].irqEn        = TRUE; /* Interrupt error report Enable */

    pSetup->portCntlIndpEn[1].softRecovery = FALSE;/* Hardware controlled error recovery */
    pSetup->portCntlIndpEn[1].forceReInit  = FALSE;/* Reinitialization process NOT forced */
    pSetup->portCntlIndpEn[1].debug        = TRUE; /* Debug enable */
    pSetup->portCntlIndpEn[1].sendDebugPkt = FALSE;/* Do not force a debug packet */
    pSetup->portCntlIndpEn[1].illTransEn   = TRUE; /* Illegal Transfer Error reporting Enable*/
    pSetup->portCntlIndpEn[1].maxRetryEn   = TRUE; /* Max_retry_error report enable */
    pSetup->portCntlIndpEn[1].maxRetryThr  = 1;    /* Maximum Retry Threshold Trigger */
    pSetup->portCntlIndpEn[1].irqEn        = TRUE; /* Interrupt error report Enable */
}

/*
 * ============================================================================
 *   @func   srio_start_maintenance
 *
 *   @desc
 * ============================================================================
 */
srio_start_maintenance (CSL_SrioHandle hSrio, int xmtAddress, int rcvAddress, int byte_count, int type, int port, int Lsuno)
{
    CSL_SrioDirectIO_ConfigXfr  lsu_conf = {0};
    lsu_conf.srcNodeAddr           = (Uint32)xmtAddress;     /* Source address          */
    lsu_conf.dstNodeAddr.addressHi = 0;
    lsu_conf.dstNodeAddr.addressLo = (Uint32)rcvAddress;     /* Destination address     */
    lsu_conf.byteCnt               = byte_count;          /* Size of data               */
    lsu_conf.idSize                = 1;                   /* 16 bit device id           */
    lsu_conf.priority              = 0;                   /* PKT priority is 0          */
    lsu_conf.xambs                 = 0;                   /* Not an extended address    */
    lsu_conf.dstId                 = LARGE_DEV_ID;        /* Destination ID             */
    lsu_conf.intrReq               = 1;                   /* Interrupt requested        */
    lsu_conf.pktType               = type;                /* Packet type                */
    lsu_conf.hopCount              = 0;                   /* Valid for maintainance pkt */
    lsu_conf.doorbellInfo          = 0;                   /* Not a doorbell pkt         */
    lsu_conf.outPortId             = port;                /* Tx on specified SRIO Port  */
    
    /* Configure the LSU and start transmission */
    CSL_srioLsuSetup (hSrio, &lsu_conf, Lsuno);
}



/*
 * ============================================================================
 *   @func   ackID_sync
 *
 *   @desc
 * ============================================================================
 */
void ackID_sync (CSL_SrioHandle hSrio)
{

    CSL_SrioPortData    responseSrio;
	Uint32              count, response, ackID, response_valid;
	volatile Int32 		LSUNO = SELECTED_LSU;
	Uint32 				rcvAddress;
	Uint32 				partner_inbound_ackid;
    CSL_SrioLsuCompStat completionCode;
    CSL_DevRegs*        devRegs = (CSL_DevRegs*)CSL_DEV_REGS;
	response_valid = 0;

	while (!response_valid)
	{
		/***** Send Link Request Control Symbols *****/
		hSrio->regs->PORT[SRIO_PORT_NUMBER].SP_LM_REQ = 0x04;
       
		for (count=0; count<1000; count++)
			asm(" nop 5");

		response = hSrio->regs->PORT[SRIO_PORT_NUMBER].SP_LM_RESP;
		printf ("Response = 0x%08x\n", response);

		response_valid = (response & 0x80000000) >> 31;
	}
	ackID = (response & 0x3E0) >> 5;

	/***** Copy the expected INBOUND_ACKID sent by other device into OUTBOUND_ACKID *****/
	hSrio->regs->PORT[SRIO_PORT_NUMBER].SP_ACKID_STAT = (hSrio->regs->PORT[SRIO_PORT_NUMBER].SP_ACKID_STAT & 0xFFFFFFE0) | ackID;


	/***** Send Maintenance packet to link partner to set his OUTBOUND/INBOUND ACKID  *****/
	/***Value written should be: outstanding = outbound = DSPs expected inbound ACKID
	value, and the inbound = 1 + ACKID from the LM response above***/

    /* Write the Inbound AckID */
	partner_inbound_ackid = ackID + 1;
	if(CSL_FEXT(devRegs->DEVSTAT, DEV_DEVSTAT_LENDIAN) == 0)
	    partner_inbound_ackid = partner_inbound_ackid << 24;

	/* SPn_ACKID_STAT register offset is 0x1148+0x20*n. When we are writing the Maintenance packet which
	 * will not reach to the logical layer but will be addressed at physical layer, we need to remove 
	 * 0x1000 offset since in actual physical layer the registers are starting from address offset 0x000 */
	rcvAddress = 0x00000148 + 0x20 * SRIO_PORT_NUMBER;
	srio_start_maintenance(hSrio, (int) &partner_inbound_ackid,(int) rcvAddress, 4, REQ_MAINT_WR, SRIO_PORT_NUMBER, LSUNO);

    responseSrio.index = LSUNO;
    while (1)
    {
        /* Poll and check if the LSU has completed */
        CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_LSU_BSY_STAT, &responseSrio);
        /* Is the LSU not busy */
        if (responseSrio.data == 0)
            break;
    }

    /* Get the completion Code for the specific LSU. */
    completionCode.lsuNum = LSUNO;
    CSL_srioLsuCompCodeStat(hSrio, &completionCode);
    if (completionCode.lsuCompCode != 0x0)
    {
        /* Failed Transfer. */
        printf ("Error: Maintenance has a completion code of %d\n", completionCode.lsuCompCode);
        return;
    }

	printf ("Partner_Inbound_AckID = 0x%08x\n", partner_inbound_ackid);
}


/*
 * ============================================================================
 *   @func   main
 *
 *   @desc
 *     This is the main routine of this program which invokes the different
 *     components to complete a simple LSU transfer.
 * ============================================================================
*/
void main (void)
{
    volatile Uint32             index;
    Uint8                       lsu_no;
    CSL_SrioContext             context;
    CSL_Status                  status;
    CSL_SrioObj                 srioObj;
    CSL_SrioParam               srioParam;
    CSL_SrioDirectIO_ConfigXfr  lsu_conf = {0};
    CSL_SrioPortData            response;
    CSL_SrioCntlSym             cntrlSym;
    CSL_SrioLsuCompStat         completionCode;
    CSL_SrioHandle              hSrio;

    /* Enable SRIO in PSC control module */
    enableSRIO();
    
	printf("SRIO Example ...\n");
    /* Initialization and Open of the SRIO */
    status = CSL_srioInit (&context);
    hSrio = CSL_srioOpen (&srioObj, 0, &srioParam, &status);
    if (status != CSL_SOK) 
    {
        printf("SRIO: ... Cannot open SRIO, failed\n");
        return;
    }

    /* Create the setup structure */
    srio_Create_Setup (&setup);

    /* Setup the SRIO with the selected setup in the last step */
    status = CSL_srioHwSetup (hSrio, &setup);
    if (status != CSL_SOK) 
    {
        printf("SRIO: ... Hardware setup, failed\n");
        return;
    }

    /* Setup the source and destination */
    for(index=0; index<TRANSFER_SIZE; index++) 
    {
	    src[index] = index + 1;
	    dst[index] = 0;
	}

    /* Make sure that the SRIO Port is Initialized. */
    printf ("Debug: Waiting for SRIO Port %d to be initialized\n", SRIO_PORT_NUMBER);

    /* Loop around till the PORT is initialized. */
    response.index = SRIO_PORT_NUMBER;
    do 
    {
 		delay(10000);  //check the response every 100 cpu clcok = 100 ns
		CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_SP_ERR_STAT, &response);
    }while(response.data & 0x1);

    /* Clear any errors... */
    cntrlSym.portNum = (CSL_SrioPortNum)SRIO_PORT_NUMBER;
    cntrlSym.stype0  = 0x2;
    cntrlSym.par0    = 0x0;
    cntrlSym.par1    = 0x1F;
    cntrlSym.stype1  = 0x4;
    cntrlSym.cmd     = 0x4;
    cntrlSym.emb     = 0x0;
    CSL_srioHwControl (hSrio, CSL_SRIO_CMD_CNTL_SYM_SET, &cntrlSym);
	delay(10000);

    /* Read the Port Error Status register. */
    response.index = SRIO_PORT_NUMBER;
    CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_SP_ERR_STAT, &response);

    /* Was there an error detected? */
    if ((response.data & (CSL_SRIO_ERR_OUTPUT_ERROR_STP | CSL_SRIO_ERR_INPUT_ERROR_STP)) != 0)
    {
        /* Error was detected; we need to clear this before we can proceed... */
        printf ("Error: Port %d has Input/Output Error... clearing\n", SRIO_PORT_NUMBER);

        /* Clearing the errors... */
		response.data &= (CSL_SRIO_ERR_OUTPUT_PKT_DROP  |
			              CSL_SRIO_ERR_OUTPUT_FLD_ENC   |
						  CSL_SRIO_ERR_OUTPUT_DEGRD_ENC |
						  CSL_SRIO_ERR_OUTPUT_RETRY_ENC |
						  CSL_SRIO_OUTPUT_ERROR_ENC     |
						  CSL_SRIO_INPUT_ERROR_ENC      |
						  CSL_SRIO_PORT_WRITE_PND       |
						  CSL_SRIO_PORT_ERROR);
        CSL_srioHwControl (hSrio, CSL_SRIO_CMD_SP_ERR_STAT_CLEAR, &response);
		delay(10000);
    }
    ackID_sync (hSrio);

    /***********************************************************************************
     ********************************* DATA Transfer ***********************************
     ***********************************************************************************/

    /* Create an LSU configuration */
    lsu_conf.srcNodeAddr           = (Uint32)&src[0];     /* Source address             */
    lsu_conf.dstNodeAddr.addressHi = 0;
    lsu_conf.dstNodeAddr.addressLo = (Uint32)&dst[0];     /* Destination address        */
    lsu_conf.byteCnt               = TRANSFER_SIZE;       /* Size of data               */
    lsu_conf.idSize                = 1;                   /* 16 bit device id           */
    lsu_conf.priority              = 2;                   /* PKT priority is 2          */
    lsu_conf.xambs                 = 0;                   /* Not an extended address    */
    lsu_conf.dstId                 = LARGE_DEV_ID;        /* Destination ID             */
    lsu_conf.intrReq               = 0;                   /* No interrupts              */
    lsu_conf.pktType               = SRIO_PKT_TYPE_NWRITER;/* Write with response       */
    lsu_conf.hopCount              = 0;                   /* Valid for maintainance pkt */
    lsu_conf.doorbellInfo          = 0;                   /* Not a doorbell pkt         */
    lsu_conf.outPortId             = SRIO_PORT_NUMBER;    /* Tx on specified SRIO Port  */

    /* Configure the LSU and start transmission */
    lsu_no = SELECTED_LSU;
    CSL_srioLsuSetup (hSrio, &lsu_conf, lsu_no);

    /* Wait for the completion of transfer */
    response.index = lsu_no;
    while (1)
    {
        /* Poll and check if the LSU has completed */
        CSL_srioGetHwStatus (hSrio, CSL_SRIO_QUERY_LSU_BSY_STAT, &response);
        /* Is the LSU not busy */
        if (response.data == 0)
            break;
    }

    /* Get the completion Code for the specific LSU. */
    completionCode.lsuNum = lsu_no;
    CSL_srioLsuCompCodeStat(hSrio, &completionCode);
    if (completionCode.lsuCompCode != 0x0)
    {
        /* Failed Transfer. */
        printf ("Error: SRIO Write has a completion code of %d\n", completionCode.lsuCompCode);
        return;
    }

    printf ("Debug: SRIO Transfer was completed; validating...\n");

    /* Validate the received data to ensure data was transferred correctly. */
    for(index=0; index<TRANSFER_SIZE; index++) 
    {
	    if (src[index] != dst[index])
		    break;
    }
    if (index == TRANSFER_SIZE) 
	    printf("Debug: Data Transfer Validated\n");
    else
	    printf("Error: Data Transfer Corrupted\n");
}

