/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ============================================================================
 *   @file  tsip_data.c
 *
 *   @path  $(CSLPATH)\example\tsip_dlb_example\tsip\src
 *
 *   @desc  This is a utility program for TSIP example.  
 *
 *  ============================================================================
 */
#include <csl_types.h>
#include <csl_psc.h>
#include <cslr_psc.h>
#include <soc.h>
#include "example_config.h"

/* For PSC */
CSL_PscHandle   hPsc;
CSL_PscObj      pscObj;
CSL_Status      psc_status;

/*
 * =============================================================================
 *   @func   Init_Mem
 *
 *   @desc
 *  @n This function initializes memory with data passed.
 *
 *  @arg  
 *          Addr       - Address from which memory has to be initialized.
 *          NoofArrays - Lengh 
 *          Count      - Number of data
 *          IncrData   - data to be incremented
 *
 *  @return
 *      None
 * =============================================================================
 */
void Init_Mem (
    Uint32*Addr, 
    Uint32 Slots, 
    Uint32 Frames, 
    Uint32 StartData,
    Uint32 IncrData
)
{
    Uint32 i,j,temp;
    Uint32 Data = StartData;

    temp = (Slots + 8) /4;
    for(i= 0; i<Frames; i++) {
        for (j=0; j< temp; j++) {
            *(Uint32*)(Addr + i*temp+j) = Data;
            Data += IncrData;
        }
    }
}   




/*
 * =============================================================================
 *   @func   Verify_Transfer
 *
 *   @desc
 *  @n This function Verifies TSIP CFG A transfer
 *
 *  @arg  
 *      FrameCnt  - Frame count
 *      FrameSize - Frame size
 *      srcBuff   - Source buffer
 *      dstBuff   - Destination buffer
 *
 *  @return
 *      None
 * =============================================================================
 */
#define LOOPBACK_DATA 1
extern volatile int loopback_mode;

Bool Verify_Transfer (
    Uint32 FrameCnt, 
    Uint32 FrameSize, 
    Uint32 *srcBuff,
    Uint32 *dstBuff
)
{
    Uint8* ExpArrayPtr = (Uint8*)(srcBuff );
    Uint8* dstArrayPtr = (Uint8*)(dstBuff );
    
    int i,j,offset;

    if(LOOPBACK_MODE==LOOPBACK_DATA) offset = 4;
    else offset = 3;

    ExpArrayPtr = ((Uint8*)srcBuff + FrameSize *offset +4 );
    dstArrayPtr = ((Uint8*)dstBuff +4 );

    for (i = 0; i < FrameCnt + 1 ; i++) {
        for (j = 0; j < FrameSize-8;j++) {
            if(*dstArrayPtr++ != *ExpArrayPtr++) {
                 return(1);
             }
        }
        dstArrayPtr += 8;
        if(i == FrameCnt - offset ) 
            ExpArrayPtr = ((Uint8*)srcBuff + 4 );
        else 
            ExpArrayPtr += 8;
    }           
  
    return (0);
}


 /* =============================================================================
 *   @func   setup_PSC
 *
 *   @desc
 *  @n This function is to setup PSC
 *
 *  @arg  
 *      None
 *
 *  @return
 *      None
 * =============================================================================
 */
void setup_PSC (
    void
)
{
    hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &psc_status);
}

/*
 * =============================================================================
 *   @func   Close_PSC_handle
 *
 *   @desc
 *  @n This function is to close PSC instance.
 *
 *  @arg  
 *      None
 *
 *  @return
 *      None
 * =============================================================================
 */
void Close_PSC_handle (
    void
)
{
    CSL_pscClose(hPsc);
}

/*
 * =============================================================================
 *   @func   TSIP_Domain_Enable
 *
 *   @desc
 *  @n This function is to enables the TSIP PSC domains
 *
 *  @arg  
 *      None
 *
 *  @return
 *      None
 * =============================================================================
 */
void TSIP_Domain_Enable (
    void
)
{
    Uint32 pwrDmn;
    Uint32 module;
    Uint32 count;
    
    setup_PSC();
    
    /* Enabling TSIP0 Clock */
    module = CSL_PSC_MODULE_TSIP0;
    CSL_pscHwControl (hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
    
    /* set Go for Power domain transition*/
    pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
    CSL_pscHwControl (hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);
    
    /* Enabling TSIP1 Clock */
    module = CSL_PSC_MODULE_TSIP1;
    CSL_pscHwControl (hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
    
    /* set Go for Power domain transition*/
    pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
    CSL_pscHwControl (hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);

    /* Enabling TSIP2 Clock */
    module = CSL_PSC_MODULE_TSIP2;
    CSL_pscHwControl (hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
    
    /* set Go for Power domain transition*/
    pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
    CSL_pscHwControl (hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);
    
    /* Add Delay for power domains to be enabled */
    for(count = 0; count < 0x100000; count++);
    
    Close_PSC_handle();
}


