/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ============================================================================
 *   @file  utopia_util.c
 *
 *   @path  $(CSLPATH)\example\utopia2\src
 *
 *   @desc  This file contains the utility function for utopia2 example
 *
 * =============================================================================
 */  
#include <utopia_util.h>
#include <csl_psc.h>
#include <cslr_psc.h>
#include <csl_gpio.h>
#include <stdio.h>
#include <string.h>

#define PASS         1
#define FAIL         0

CSL_GpioHandle hGpio;
CSL_PscHandle  hPsc;
CSL_PscObj     pscObj;
CSL_Status     psc_status;

/*
 * =============================================================================
 *   @func   Init_Mem
 *
 *   @arg Uint32
       Addr           - Address 
       NoofArrays     - Number of array
       Count          - count
       IncrData       - data value to increment
 *
 *   @desc
 *     This function initialize the buffer
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */
void Init_Mem (
    Uint8  *Addr, 
    Uint32 NoofArrays, 
    Uint32 Count, 
    Uint8  IncrData
)
{
    Uint32 i;
    Uint32 Data = 0x1;
    for (i=0; i < Count * NoofArrays; i++) {
        *(Uint8*)(Addr + i) = Data;
        Data += IncrData;
    }
}

/*
 * =============================================================================
 *   @func   Verify_Result
 *
 *   @arg Uint32
         src_addr       - Source buffer
         dst_addr       - Destination buffer
         Cell_Num       - cell number
         Pad_Cell_Size  - pad cell size
         ATM_Cell_Size  - atm cell size
         Udc            - user defined cell 
         UtopiaBitMode  - utopia2 bit mode ie., 8 bit/16 bit
 *
 *   @desc
 *     This function is to verify the data transfer from UTOPIA and received
 *     data from the UTOPIA master (test bench acts as master.) by comparing
 *     source buffer and destination buffer i.e., src_addr, dst_addr.
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */
Uint32 Verify_Result ( 
    Uint32 src_addr,
    Uint32 dst_addr,
    Uint32 Cell_Num,
    Uint32 Pad_Cell_Size,
    Uint32 ATM_Cell_Size,
    Uint32 Udc,
    Uint32 UtopiaBitMode
)
{
	Uint8* srcArrayPtr = (Uint8*)src_addr;
	Uint8* dstArrayPtr = (Uint8*)dst_addr;
        Uint32 i, j;
	Uint32 dummy_bytes;

 	dummy_bytes = (Pad_Cell_Size -(Udc +(UtopiaBitMode?54:53)));
	
        for (i = 0; i < Cell_Num; i++)
	{
     		for (j=	dummy_bytes ; j < Pad_Cell_Size; j++)
		{
         	        if (srcArrayPtr[(i*Pad_Cell_Size) + j] != 
                        dstArrayPtr[(i*Pad_Cell_Size) + j])
			{
				return (1);
			}
              }
	}
	return(0);
}


/*
 * =============================================================================
 *   @func   setup_PSC
 *
 *   @arg 
 *      NONE
 *
 *   @desc
 *     This function opens the PSC handle.
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */
void setup_PSC (
    void
)
{
    hPsc = CSL_pscOpen(&pscObj, CSL_PSC, NULL, &psc_status);
}
    
/*
 * =============================================================================
 *   @func   Close_PSC_handle
 *
 *   @arg 
 *      NONE
 *
 *   @desc
 *     This function closes the PSC handle.
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */
void Close_PSC_handle (
    void
) 
{
    CSL_pscClose(hPsc);
}

/*
 * =============================================================================
 *   @func   UTOPIA_Domain_Enable
 *
 *   @arg 
 *      NONE
 *
 *   @desc
 *     This function enables the utopia2 domain.
 *
 *   @return
 *      NONE
 *
 * =============================================================================
 */
void UTOPIA_Domain_Enable (
)
{
    CSL_PscPeripherals  module;
    CSL_PscPowerDomain  pwrDmn;
    
    setup_PSC();
    
    /* Enabling UTOPIA Clock */
    module = CSL_PSC_MODULE_UTOPIA;
    CSL_pscHwControl (hPsc, CSL_PSC_CMD_ENABLE_MODULE, &module);
    
    /* set Go for Power domain transition*/
    pwrDmn = CSL_PSC_PWRDMN_ALWAYSON;
    CSL_pscHwControl (hPsc, CSL_PSC_CMD_PWRDMN_TRNS, &pwrDmn);
    
    Close_PSC_handle();
}


/* End of utopia_util.c */

