/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** ============================================================================
 *   @file  csl_ectl.h
 *
 *   PATH:  \$(CSLPATH)\\inc
 *
 *   @brief  ECTL CSL Implementation on DSP side
 */

/* =============================================================================
 *  Revision History
 *  ===============
 *  17-Feb-06 PSK file created
 * =============================================================================
 */

/** @mainpage ECTL Documentation
 *
 * @section Introduction
 *
 * @subsection xxx Purpose and Scope
 * The purpose of this document is to detail the  CSL APIs for the ECTL
 * Module. The CSL developer is expected to refer to this document
 * while designing APIs for the modules which use ECTL module. Some of the
 * listed APIs may not be applicable to a given module. While other cases
 * this list of APIs may not be sufficient to cover all the features required
 * for a particular Module. The CSL developer should use his discretion
 * designing new APIs or extending the existing ones to cover these.
 *
 * @subsection aaa Terms and Abbreviations
 *   -# CSL:  Chip Support Library
 *   -# API:  Application Programmer Interface
 *   -# ECTL: Ethernet Multi core interrupt Control
 *
 * @subsection References
 *   -# CSL 3.x Technical Requirements Specifications Version 0.5, dated
 *      May 14th, 2003
 *   -# EMAC_Intr_combiner_fs.pdf (Revision 0.11)
 *
 * @subsection Assumptions
 *    The abbreviations ECTL, ectl and Ectl have been used throughout this
 *    document to refer to Ethernet Multi core interrupt Controller
 */

#ifndef _CSL_ECTL_H_
#define _CSL_ECTL_H_

#ifdef __cplusplus
extern "C" {
#endif

/* Include the register layer ECTL header file */
#include <cslr_ectl.h>
#include <soc.h>

/**************************************************************************\
* Peripheral Base Address
\**************************************************************************/
/** ECTL Module registers */
#define ECTL0_REGS         ((CSL_EctlRegs *)CSL_ECTL_0_REGS)
#define ECTL1_REGS         ((CSL_EctlRegs *)CSL_ECTL_1_REGS)

/* Error return macro */
#define ECTL_ERROR_INVALID   4 
  
/* Total number of cores */
#define NUM_OF_CORES 6

/* Total number of channels */
#define NUM_OF_CHANNELS 8

/**< Function or calling parameter is invalid  */

/**
 *  \defgroup   ECTLModule  ECTL
 *
 *  Constants, objects and interfaces for ECTL
 *  @{
 */

/**
 *  \defgroup   ECTLConstants   ECTL Constants
 *  @{
 */

/**
 *  \defgroup  CommonInterrupts   Core(0 to 5) common interrupts enable/disable/status macros
 *  @{
 */
#define ECTL_COMMON_INTR_HOST          0x00000002
/**< Host Error interrupt */
#define ECTL_COMMON_INTR_STAT          0x00000004
/**< Statistics interrupt */
#define ECTL_COMMON_INTR_MDIO_LINT     0x00000008
/**< Serial interface link change interrupt, Indicates achange in the
 *   state of the PHY link  */
#define ECTL_COMMON_INTR_MDIO_USER     0x00000010
/**< Serial interface user command event complete interrupt */
/* @} */

/**
 * \defgroup   ReceiveInterrupts Core(0 to 5) Rx interrupts enable/disable/status macros
 *
 *  @{
 */
#define ECTL_RECEIVE_INTR_RX7     0x00800000
/**< Channel 7 Receive Queue Interrupt */
#define ECTL_RECEIVE_INTR_RX6     0x00400000
/**< Channel 6 Receive Queue Interrupt */
#define ECTL_RECEIVE_INTR_RX5     0x00200000
/**< Channel 5 Receive Queue Interrupt */
#define ECTL_RECEIVE_INTR_RX4     0x00100000
/**< Channel 4 Receive Queue Interrupt */
#define ECTL_RECEIVE_INTR_RX3     0x00080000 
/**< Channel 3 Receive Queue Interrupt */
#define ECTL_RECEIVE_INTR_RX2     0x00040000 
/**< Channel 2 Receive Queue Interrupt */
#define ECTL_RECEIVE_INTR_RX1     0x00020000 
/**< Channel 1 Receive Queue Interrupt */
#define ECTL_RECEIVE_INTR_RX0     0x00010000 
/**< Channel 0 Receive Queue Interrupt */
/* @} */

/**
 * \defgroup   TransmitInterrupts Core(0 to 5) Tx interrupts enable/disable/status macros
 *
 *  @{
 */
#define ECTL_TRANSMIT_INTR_TX7    0x00008000
/**< Channel 7 Transmit Queue Interrupt */
#define ECTL_TRANSMIT_INTR_TX6    0x00004000
/**< Channel 6 Transmit Queue Interrupt */
#define ECTL_TRANSMIT_INTR_TX5    0x00002000
/**< Channel 5 Transmit Queue Interrupt */
#define ECTL_TRANSMIT_INTR_TX4    0x00001000
/**< Channel 4 Transmit Queue Interrupt */
#define ECTL_TRANSMIT_INTR_TX3    0x00000800
/**< Channel 3 Transmit Queue Interrupt */
#define ECTL_TRANSMIT_INTR_TX2    0x00000400
/**< Channel 2 Transmit Queue Interrupt */
#define ECTL_TRANSMIT_INTR_TX1    0x00000200
/**< Channel 1 Transmit Queue Interrupt */
#define ECTL_TRANSMIT_INTR_TX0    0x00000100
/**< Channel 0 Transmit Queue Interrupt */
/* @} */

/**
 * \defgroup   StateMachine Rx and Tx time delay and div count state machine status macros
 *
 *  @{
 */
#define ECTL_RXTX_DIV_SM_WATING  0x0
/**< Divide by N count SM is in WAITING state */
#define ECTL_RXTX_DIV_SM_COUNT   0x1
/**< Divide by N count SM is in COUNT state */
#define ECTL_RXTX_DIV_SM_OUTPUT  0x2
/**< Divide by N count SM is in OUTPUT state */
#define ECTL_RXTX_TIM_SM_WATING  0x0
/**< Timed Delay SM is in WAITING state */
#define ECTL_RXTX_TIM_SM_DELAY   0x1
/**< Timed Delay SM is in DELAY state */
#define ECTL_RXTX_TIM_SM_OUTPUT  0x2
/**< Timed Delay SM is in OUTPUT state */
/* @} */



/**
 *  \defgroup   ECTLObjects   ECTL Objects
 *  @{
 */


/** \brief This structure is used for configuring the ECTL module */
typedef struct {
   int      coreNumber;
   /**< core number (There are 6 cores from 0 to 5) */
   int      channelNumber;
   /**< Tx or Rx channel number (There are 8 channels from o to 7) */
   Uint32   rxTxIntrCntl;
   /**< Common, Receive and Transmit interrupts control */
   Uint16   rxPaceTimeDelay;          
   /**< Rx pacer time delay configuration value */
   Uint8    rxPaceDivCount;           
   /**< Rx pacer Divide by N count configuration value */
   Uint16   txPaceTimeDelay;          
   /**< Tx pacer time delay configuration value */
   Uint8    txPaceDivCount;                    
   /**< Tx pacer time delay configuration value */
   Uint16   prescale;
   /**< Reload value for the prescalar configuration value */
} ECTL_Config;

/** \brief This structure is used to get the status values of ECTL */
typedef struct {
    Uint8  coreNumber;
    /**< core number (There are 6 cores from 0 to 5) */
    Uint8  channelNumber;
    /**< Tx or Rx channel number (There are 8 channels from o to 7) */
    Uint32 rxTxIntrCntlStatus;
    /**< Common, Receive and Transmit interrupts control status */
    Uint16 rxTimeDelay;
    /**< Rx pacer time delay configuration value status */
    Uint8  rxDivCount; 
    /**< Rx pacer Divide by N count configuration value status */
    Uint16 txTimeDelay;
    /**< Tx pacer Divide by N count configuration value status */
    Uint8  txDivCount;
    /**< Tx pacer Divide by N count configuration value status */
    Uint16 prescale; 
    /**< Reload value for the prescalar configuration value */
} ECTL_Status;
/* @} */

/** This macro resets the Divide by N count for Rx channel */
#define ECTL_rxDivCountReset(channelNumber, instNum)  \
                CSL_FINST(ECTL##instNum_REGS->RPCFG[channelNumber], ECTL_RPCFG_CR, RESET)
                
/** This macro resets the time delay counter for Rx channel */                
#define ECTL_rxTimeCounterReset(channelNumber, instNum)  \
                CSL_FINST(ECTL##instNum_REGS->RPCFG[channelNumber], ECTL_RPCFG_TR, RESET)
                
/** This macro resets the Divide by N count for Tx channel */
#define ECTL_txDivCountReset(channelNumber, instNum)  \
                CSL_FINST(ECTL##instNum_REGS->TPCFG[channelNumber], ECTL_TPCFG_CR, RESET)
                
/** This macro resets the time delay counter for Tx channel */                
#define ECTL_txTimeCounterReset(channelNumber, instNum)  \
                CSL_FINST(ECTL##instNum_REGS->TPCFG[channelNumber], ECTL_TPCFG_TR, RESET)
                
/** This macro reads the state of Divide by N count State Machine for Rx channel*/
#define ECTL_rxDivCountState(channelNumber, instNum)  \
                CSL_FEXT(ECTL##instNum_REGS->RPSTAT[channelNumber], ECTL_RPSTAT_DIV_SM) 
                
/** This macro reads the state of Timed Delay State Machine for Rx channel */                           
#define ECTL_rxTimeCounterState(channelNumber, instNum)  \
                CSL_FEXT(ECTL##instNum_REGS->RPSTAT[channelNumber], ECTL_RPSTAT_TIM_SM)
                
/** This macro reads the state of Divide by N count State Machine for Tx channel */                
#define ECTL_txDivCountState(channelNumber, instNum)  \
                CSL_FEXT(ECTL##instNum_REGS->TPSTAT[channelNumber], ECTL_TPSTAT_DIV_SM) 
                
/** This macro reads the state of Timed Delay State Machine for Tx channel */                
#define ECTL_txTimeCounterState(channelNumber, instNum)  \
                CSL_FEXT(ECTL##instNum_REGS->TPSTAT[channelNumber], ECTL_TPSTAT_TIM_SM)


/**
 *  \defgroup  ECTLInterfaces  ECTL Interfaces
 *
 *  Interfaces for the ECTL.
 *  @{
 */


/** ============================================================================
 *  @n@b ECTL_config()
 *
 *  @b Description
 *  @n Configures the ECTL module for a specific core and channel. Call this function
        multiple times to configure all the cores and channels
 *
 *  @b Arguments
 *  @verbatim
        pEctlConfig      pointer to config structure
    @endverbatim
 *
 *  <b> Return Value </b>  Success (0)
 *  @n      ECTL_ERROR_INVALID   - A calling parameter is invalid
 *
 *  <b> Pre Condition </b>
 *  @n  None
 *
 *  <b> Post Condition </b>
 *  @n  ECTL module will be configured
 *
 *  @b Example
 *  @verbatim
        ECTL_Config *pEctlConfig;
        
        pEctlConfig->coreNumber = 0;
        pEctlConfig->channelNumber = 1;
        
        pEctlConfig->rxTxIntrCntl = ECTL_COMMON_INTR_HOST |ECTL_COMMON_INTR_MDIO_LINT |
                                    ECTL_RECEIVE_INTR_RX1 | ECTL_TRANSMIT_INTR_TX1;
        pEctlConfig->rxPaceTimeDelay = 0xffff;
        pEctlConfig->txPaceTimeDelay = 0xffff;
        pEctlConfig->rxPaceDivCount  = 0x0f;
        pEctlConfig->txPaceDivCount  = 0x0f;                                 
        pEctlConfig->prescale        = 0xff00;
        
        ECTL_config(pEctlConfig, 0);
    @endverbatim
 * ============================================================================
 */ 
Uint32 ECTL_config( ECTL_Config *ectlConfig, int instNum);


/** ============================================================================
 *  @n@b ECTL_getStatus()
 *
 *  @b Description
 *  @n Gets the ECTL module status for a specific core and channel. Call this function
        multiple times to get the status of all the cores and channels
 *
 *  @b Arguments
 *  @verbatim
        pStatus      pointer to status structure
    @endverbatim
 *
 *  <b> Return Value </b>  Success (0)
 *  @n      ECTL_ERROR_INVALID   - A calling parameter is invalid
 *
 *  <b> Pre Condition </b>
 *  @n  None
 *
 *  <b> Post Condition </b>
 *  @n  ECTL status structure contains the values read from the registers 
 *
 *  @b Example
 *  @verbatim
        ECTL_Status *pStatus;
        
        pStatus->coreNumber = 0;
        pStatus->channelNumber = 1;
        
        ECTL_getStatus(pStatus, 0);
    @endverbatim
 * ============================================================================
 */
Uint32 ECTL_getStatus( ECTL_Status *pStatus, int instNum);
/* @} */
/* @} */
#ifdef __cplusplus
}
#endif

#endif  /* _CSL_ECTL_H_ */

