/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** 
 *  @file    csl_hpiAux.h
 * 
 *  @path    $(CSLPATH)\inc
 *
 *  @brief   HPI CSL implementation on DSP side. 
 *
 */
/* =============================================================================
 *  Revision History
 *  ===============
 *  20-Jan-2006 NG  File taken from CSL code base.
 *
 * =============================================================================
 */
#ifndef _CSL_HPIAUX_H_
#define _CSL_HPIAUX_H_

#include <csl_hpi.h>

#ifdef __cplusplus
extern "C" {
#endif

/** Hardware Control functions of the hpi */

/** ============================================================================
 *   @n@b CSL_hpiSetDspInt
 *
 *   @b Description
 *   @n Gets the HPIC Host-to-DSP Interrupt status of the hpi.
 *
 *   @b Arguments
 *   @verbatim
            hHpi           Handle to the HPI instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n Sets the DSP_INT bit of HPIC register
 *
 *   @b Example
 *   @verbatim
        CSL_HpiHandle          hHpi;

        CSL_hpiSetDspInt(hHpi);

     @endverbatim
 * =============================================================================
 */
static inline
void CSL_hpiSetDspInt (
    CSL_HpiHandle    hHpi
)
{
    CSL_FINS(hHpi->regs->HPIC, HPI_HPIC_DSP_INT, TRUE); 
}   
    
/** ============================================================================
 *   @n@b CSL_hpiResetDspInt
 *
 *   @b Description
 *   @n Gets the Reset HPIC Host-to-DSP Interrupt status of the hpi.
 *
 *   @b Arguments
 *   @verbatim
            hHpi           Handle to the HPI instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n Resets the DSP_INT bit of HPIC register
 *
 *   @b Example
 *   @verbatim
        CSL_HpiHandle          hHpi;

        CSL_hpiResetDspInt(hHpi);

     @endverbatim
 * =============================================================================
 */
static inline
void CSL_hpiResetDspInt (
    CSL_HpiHandle    hHpi
)
{
    CSL_FINS(hHpi->regs->HPIC, HPI_HPIC_DSP_INT, FALSE);
}       

/** ============================================================================
 *   @n@b CSL_hpiSetHint
 *
 *   @b Description
 *   @n Gets the HPIC DSP-to-Host Interrupt status of the hpi.
 *
 *   @b Arguments
 *   @verbatim
            hHpi           Handle to the HPI instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n Sets the HINT bit of HPIC register
 *
 *   @b Example
 *   @verbatim
        CSL_HpiHandle          hHpi;

        CSL_hpiSetHint(hHpi);

     @endverbatim
 * =============================================================================
 */
static inline
void CSL_hpiSetHint (
    CSL_HpiHandle    hHpi
)
{
    CSL_FINS(hHpi->regs->HPIC, HPI_HPIC_HINT, TRUE);    
}       

/** ============================================================================
 *   @n@b CSL_hpiResetHint
 *
 *   @b Description
 *   @n Gets the Reset HPIC DSP-to-Host Interrupt status of the hpi.
 *
 *   @b Arguments
 *   @verbatim
            hHpi           Handle to the HPI instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n Resets the HINT bit of HPIC register
 *
 *   @b Example
 *   @verbatim
        CSL_HpiHandle          hHpi;

        CSL_hpiSetHint(hHpi);

     @endverbatim
 * =============================================================================
 */
static inline
void CSL_hpiResetHint (
    CSL_HpiHandle    hHpi
)
{
    CSL_FINS(hHpi->regs->HPIC, HPI_HPIC_HINT, FALSE);   
}       

/** ============================================================================
 *   @n@b CSL_hpiResetDSPint
 *
 *   @b Description
 *   @n Gets the Reset HPIC Host-to-DSP Interrupt status of the hpi.
 *
 *   @b Arguments
 *   @verbatim
            hHpi           Handle to the HPI instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n Resets the DSP_INT bit of HPIC register
 *
 *   @b Example
 *   @verbatim
        CSL_HpiHandle          hHpi;

        CSL_hpiResetDSPint(hHpi);

     @endverbatim
 * =============================================================================
 */
static inline
void CSL_hpiResetDSPint (
    CSL_HpiHandle    hHpi
)
{
    CSL_FINS(hHpi->regs->HPIC, HPI_HPIC_DSP_INT, 1);   
}       


/** ============================================================================
 *   @n@b CSL_hpiGetPidRev
 *
 *   @b Description
 *   @n This function  gets the Peripheral ID Revision.
 *
 *   @b Arguments
 *   @verbatim

            hHpi           Handle to HPI instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_hpiGetPidRev(hHpi, &response);

     @endverbatim
 * =============================================================================
 */
static inline
void CSL_hpiGetPidRev (
    CSL_HpiHandle    hHpi,
    void              *response
)
{
    /* Read the peripheral revision from PID */
    *((Uint32 *)response) = CSL_FEXT(hHpi->regs->PID, HPI_PID_REV);
}

/** ============================================================================
 *   @n@b CSL_hpiGetPidClass
 *
 *   @b Description
 *   @n This function  gets the Peripheral ID Class.
 *
 *   @b Arguments
 *   @verbatim

            hHpi           Handle to HPI instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>  
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_hpiGetPidClass(hHpi, &response);

     @endverbatim
 * =============================================================================
 */
static inline
void CSL_hpiGetPidClass (
    CSL_HpiHandle    hHpi,
    void              *response
)
{
    *((Uint32 *)response) = CSL_FEXT(hHpi->regs->PID, HPI_PID_CLASS);
}

/** ============================================================================
 *   @n@b CSL_hpiGetPidType
 *
 *   @b Description
 *   @n This function  gets the Peripheral ID Tupe.
 *
 *   @b Arguments
 *   @verbatim

            hHpi           Handle to HPI instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_hpiGetPidType(hHpi, &response);

     @endverbatim
 * =============================================================================
 */
static inline
void CSL_hpiGetPidType (
    CSL_HpiHandle    hHpi,
    void              *response
)
{
    *((Uint32 *)response) = CSL_FEXT(hHpi->regs->PID, HPI_PID_TYPE);
}

/** ===========================================================================
 *   @n@b CSL_hpiGetHpicHrdy
 *
 *   @b Description
 *   @n This function  gets HRDY bit of the HPIC.
 *
 *   @b Arguments
 *   @verbatim

            hHpi           Handle to HPI instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_hpiGetHpicHrdy(hHpi, &response);

     @endverbatim
 * ===========================================================================
 */
static inline
void CSL_hpiGetHpicHrdy (
    CSL_HpiHandle    hHpi,
    void              *response
)
{
    *((Uint32 *)response) = CSL_FEXT(hHpi->regs->HPIC, HPI_HPIC_HRDY);
}               
        
/** ===========================================================================
 *   @n@b CSL_hpiGetHpicFetch
 *
 *   @b Description
 *   @n This function  gets FETCH bit of the HPIC Register.
 *
 *   @b Arguments
 *   @verbatim

            hHpi           Handle to HPI instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_hpiGetHpicFetch(hHpi, &response);

     @endverbatim
 * ===========================================================================
 */
static inline
void CSL_hpiGetHpicFetch (
    CSL_HpiHandle    hHpi,
    void              *response
)
{
    *((Uint32 *)response) = CSL_FEXT(hHpi->regs->HPIC, HPI_HPIC_FETCH);
}               

/** ===========================================================================
 *   @n@b CSL_hpiGetHpicHpiRst
 *
 *   @b Description
 *   @n This function  gets HPI_RST bit of the HPIC Register.
 *
 *   @b Arguments
 *   @verbatim

            hHpi           Handle to HPI instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_hpiGetHpicHpiRst(hHpi, &response);

     @endverbatim
 * ===========================================================================
 */
static inline
void CSL_hpiGetHpicHpiRst (
    CSL_HpiHandle    hHpi,
    void              *response
)
{
    *((Uint32 *)response) = CSL_FEXT(hHpi->regs->HPIC, HPI_HPIC_RESET);
}

/** ===========================================================================
 *   @n@b CSL_hpiGetHpicHwobStat
 *
 *   @b Description
 *   @n This function  gets HWOB_STAT bit of the HPIC Register.
 *
 *   @b Arguments
 *   @verbatim

            hHpi           Handle to HPI instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_hpiGetHpicHwobStat(hHpi, &response);

     @endverbatim
 * ===========================================================================
 */
static inline
void CSL_hpiGetHpicHwobStat (
    CSL_HpiHandle    hHpi,
    void              *response
)
{
    *((Uint32 *)response) = CSL_FEXT(hHpi->regs->HPIC, HPI_HPIC_HWOB_STAT);
}

/** ===========================================================================
 *   @n@b CSL_hpiGetDspInt
 *
 *   @b Description
 *   @n This function  gets DSP interrupt status.
 *
 *   @b Arguments
 *   @verbatim

            hHpi           Handle to HPI instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_hpiGetDspInt(hHpi, &response);

     @endverbatim
 * ===========================================================================
 */
static inline
void CSL_hpiGetDspInt (
    CSL_HpiHandle    hHpi,
    void              *response
)
{
    *((Uint32 *)response) = CSL_FEXT(hHpi->regs->HPIC, HPI_HPIC_DSP_INT);
}

#ifdef __cplusplus
}
#endif

#endif /* _CSL_HPIAUX_H_ */

