/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** 
 *  @file    csl_pdma.h
 *
 *  @path    $(CSLPATH)\inc
 *
 *  @brief   PDMA functional layer API header file
 *
 */
/* =============================================================================
 *  Revision History
 *  ===============
 *  27-Jan-2006 NG  File taken from TI code base and modified for c6486.
 *                  Updated according coding guidelines (doxygen compliant)
 *
 * =============================================================================
 */

/** @mainpage PDMA CSL 3.x
 *
 * @section Introduction
 *
 * @subsection xxx Purpose and Scope
 * The purpose of this document is to identify a set of common CSL APIs for
 * the PDMA module across various devices. The CSL developer is expected to
 * refer to this document while designing APIs for these modules. Some of the
 * listed APIs may not be applicable to a given PDMA module. While this list
 * of APIs may not be sufficient to cover all the features of a
 * particular PDMA Module. The CSL developer should use his discretion designing
 * new APIs or extending the existing ones to cover these.
 *
 * @subsection aaa Terms and Abbreviations
 *   -# CSL:  Chip Support Library
 *   -# API:  Application Programmer Interface
 *
 * @subsection References
 *    pdmaSpec_1_9.doc - Peripheral DMA Requirement document Release 1.9
 *
 */

#ifndef _CSL_PDMA_H_
#define _CSL_PDMA_H_

#ifdef __cplusplus
 extern "C" {
#endif

#include <soc.h>
#include <tistdtypes.h>
#include <csl_types.h>
#include <csl_error.h>
#include <cslr.h>
#include <cslr_pim.h>

/**************************************************************************\
 * PIM global macro declarations
\**************************************************************************/
 
#define CSL_CLR_STAT            0
/*  Clear FIFO full flag */
#define CLEAR_FIFO_FULL_FLAG    0
#define CSL_PDMA_NUM_CHAN       16
#define CSL_PDMA_0              0	
#define CSL_EVENT_GROUP         8 	

/***********************************************************************/
/* Register Definition  of PDMA module                                  */
/***********************************************************************/

#define PDMA_PIM__REGS_BASE_ADDR  (unsigned long)0x02BC0000

/** PDMA Event group 1 */
#define EVENT_GROUP_ONE 	0
/** PDMA Event group 2 */
#define EVENT_GROUP_TWO		1
/** PDMA Event group 3 */
#define EVENT_GROUP_THREE	2
/** PDMA Event group 4 */
#define EVENT_GROUP_FOUR 	3
/** PDMA Event group 5 */
#define EVENT_GROUP_FIVE	4
/** PDMA Event group 6 */
#define EVENT_GROUP_SIX  	5
/** PDMA Event group 7 */
#define EVENT_GROUP_SEVEN	6
/** PDMA Event group 8 */
#define EVENT_GROUP_EIGHT	7

 
/**************************************************************************\
 * PIM global typedef declarations
\**************************************************************************/

/** @brief Pdma global control select enumeration */
typedef enum {
    /** Pdma global disable */
    CSL_PDMA_GLOBAL_DIS   = 0,
    /** Pdma global enable */
    CSL_PDMA_GLOBLE_ENB   = 1
} CSL_PdmaEnable;

/** @brief Pdma emulation select enumeration */
typedef enum {
    /** Pdma emulation stop */
    CSL_PDMA_EMU_STOP       = 0,
    /** free running mode  */
    CSL_PDMA_EMU_FREERUN    = 1
} CSL_PdmaEmu;

/** @brief Pdma Endianness select enumeration */
typedef enum {
    /** Pdma little endian mode */
    CSL_PDMA_ENDIAN_LITTLE    = 0,
    /** Pdma big endian mode */
    CSL_PDMA_ENDIAN_BIG       = 1
} CSL_PdmaEndian;

/** @brief Pdma frame sync select enumeration */
typedef enum {
    /** Pdma Super-Frame Sync reset */
    CSL_PDMA_SFRMSYNC_RST    = 0,
    /** Pdma Super-Frame Sync set */
    CSL_PDMA_SFRMSYNC_SET    = 1
} CSL_PdmaSframeSync;

/** @brief Pdma peripheral sync select enumeration */
typedef enum {
    /** Pdma no sync */
    CSL_PDMA_NOSYNC_XFER    = 0,
    /** Pdma sync */
    CSL_PDMA_SYNC_XFER      = 1
} CSL_PdmaPerSyncStart;

/** @brief Pdma priority select enumeration */
typedef enum {
    /** Pdma priority 0 lowest priority */
    CSL_PDMA_PRIORITY_0    = 0,
    /** Pdma priority 1 */
    CSL_PDMA_PRIORITY_1    = 1,
    /** Pdma priority 2 */
    CSL_PDMA_PRIORITY_2    = 2,
    /** Pdma priority 3 highest priority */
    CSL_PDMA_PRIORITY_3    = 3
} CSL_PdmaPriority;

/** @brief Pdma auto buffer control select enumeration */
typedef enum {
    /** Pdma Auto-buffering Mode Disable */
    CSL_PDMA_ABU_DIS     = 0,
    /** Pdma sync enable */
    CSL_PDMA_SYNC_ENB    = 1
} CSL_PdmaAutoBufCtrl;

/** @brief Pdma direction control select enumeration */
typedef enum {
    /** Pdma direction control from Sw to Per */
    CSL_PDMA_SW_PER    = 0,
    /** Pdma direction control from Per to Sw */
    CSL_PDMA_PER_SW    = 1
} CSL_PdmaDirCtrl;

/** @brief Pdma address mode select enumeration */
typedef enum {
    /** Pdma address increment */
    CSL_PDMA_ADDR_INCR     = 0,
    /** Pdma address constant */
    CSL_PDMA_ADDR_CONST    = 1
} CSL_PdmaAddrMod;

/** @brief Pdma channel select enumeration */
typedef enum {
    /** Pdma channel XFER disable */
    CSL_PDMA_CH_XFER_DIS    = 0,
    /** Pdma channel XFER enable */
    CSL_PDMA_CH_XFER_ENB    = 1
} CSL_PdmaChXferCtrl;

/** @brief Pdma interrupt mode select enumeration */
typedef enum {
    /** Pdma interrupt mode half buffer */
    CSL_PDMA_INT_HALFBUF    = 0,
    /** Pdma interrupt mode full buffer */
    CSL_PDMA_INT_FULLBUF    = 1,
    /** Pdma interrupt mode block end */
    CSL_PDMA_INT_BLKEND     = 2
} CSL_PdmaIntMod;

/** @brief Pdma interrupt control select enumeration */
typedef enum {
    /** Pdma interrupt disable */
    CSL_PDMA_INT_DIS    = 0,
    /** Pdma interrupt enable */
    CSL_PDMA_INT_ENB    = 1
} CSL_PdmaIntCtrl;

/** @brief Pdma sub system interrupt select enumeration */
typedef enum {
    /** Pdma interrupt for CPU 0 */
    CSL_PDMA_INT_CPU0    = 0,
    /** Pdma interrupt for CPU 1 */
    CSL_PDMA_INT_CPU1    = 1,
    /** Pdma interrupt for CPU 2 */
    CSL_PDMA_INT_CPU2    = 2,
    /** Pdma interrupt for CPU 3 */
    CSL_PDMA_INT_CPU3    = 3,
    /** Pdma interrupt for CPU 4 */
    CSL_PDMA_INT_CPU4    = 4,
    /** Pdma interrupt for CPU 5 */
    CSL_PDMA_INT_CPU5    = 5,
    /** Pdma interrupt for CPU 6 */
    CSL_PDMA_INT_CPU6    = 6,
    /** Pdma interrupt for CPU 7 */
    CSL_PDMA_INT_CPU7    = 7
} CSL_PdmaSubsysIntSel;

/** @brief Pdma data element size select enumeration */
typedef enum {
    /** Pdma 8 bit data element */
    CSL_PDMA_DATA_ELEMENT_8BIT   = 0,
    /** Pdma 16 bit data element */
    CSL_PDMA_DATA_ELEMENT_16BIT  = 1,
    /** Pdma 32 bit data element */
    CSL_PDMA_DATA_ELEMENT_32BIT  = 2
} CSL_PdmaDataElementSize;

/** 
 * @brief Enumeration for queries passed to @a CSL_pdmaGetHwStatus()
 *
 * This is used to get the status of different operations 
 */
typedef enum {
    /** Pdma query of PID */
    /*  CSL_PdmaPerId structure */
    CSL_PDMA_QUERY_PID                   = 0,
    
    /** Pdma query of register statistic */
    CSL_PDMA_QUERY_STATISTIC_REGS        = 1,
    
    /** Pdma query of peripheral control register */
    /*  CSL_PdmaPeriCtrlSetup structure */
    CSL_PDMA_QUERY_PERIPHERAL_CTRL_REGS  = 2,
    
    /** Pdma query of memory control register */
    /** CSL_PdmaMemCtrlSetup structure */
    CSL_PDMA_QUERY_MEMORY_CTRL_REGS      = 3,
    
    /** Pdma query of global status */
    /*  CSL_PdmaGblSetup structure */
    CSL_PDMA_QUERY_GLOBAL_STATUS         = 4,
    
    /** Pdma query of channel information */
    /*  CSL_ChannelContext structure */
    CSL_PDMA_QUERY_CHANNEL_INFO          = 5
} CSL_PdmaHwStatusQuery;

/* *************************************************************************\
 * PIM PDMA Structure declarations
\* *************************************************************************/

/** @brief PDMA specific context information. Present implementation doesn't
 *  have any Context information.
 */
typedef struct {
    /** Context information of Pdma.
     *  The below declaration is just a place-holder for future implementation.
     */
    Uint16 contextInfo;
} CSL_PdmaContext;

/** @brief The structure contains the pdma global setup */ 
typedef struct  {
    /** Start Control bit of PDMA Global Control and Status Register */
    volatile Uint32 strt;
    /** Soft Emulation Halt Control bit of PDMA Global Control and Status 
     *  Register  
     */
    volatile Uint32 soft;
    /** Free Running Mode Control bit of PDMA Global Control and Status Register 
     */
    volatile Uint32 free;
} CSL_PdmaGblSetup;

/** @brief The structure contains the pdma peripheral control setup */ 
typedef struct  {
    /** Pdma big endian peripheral control setup  */
    volatile Uint16 bend;
    /** Clear Statistics Register */
    volatile Uint16 clrs;
    /** Super-Frame Sync Select */
    volatile Uint16 sfrm;
    /** Peripheral Event Control */
    volatile Uint16 pe;
    /** Peripheral Synchronization Start */
    volatile Uint16 sync;
    /** priority */
    volatile Uint16 pri;
    /** Auto-buffering Mode */
    volatile Uint16 abu;
    /** Direction of transfer */
    volatile Uint16 dir;
    /** Peripheral burst length */
    volatile Uint16 pblen;
    /** Peripheral address modification mode */
    volatile Uint16 pmod;
    /** Switch burst length */
    volatile Uint16 sblen;
    /** Switch address modification */
    volatile Uint16 smod;
} CSL_PdmaPeriCtrlSetup;

/** @brief The structure contains the pdma memory control */ 
typedef struct {
    /** Pdma big endian peripheral synchronization start */
    volatile Uint16 sync;
    /** priority  */
    volatile Uint16 pri;
    /** auto buffering mode */
    volatile Uint16 abu;
    /** direction of transfer */
    volatile Uint16 dir;
    /** peripheral burst length */
    volatile Uint16 pblen;
    /** peripheral address modification */
    volatile Uint16 pmod;
    /** switch burst length  */
    volatile Uint16 sblen;
    /** switch address modification */
    volatile Uint16 smod;
}CSL_PdmaMemCtrlSetup;


/** @brief The structure contains the pdma peripheral id */ 
typedef struct CSL_PdmaPerId {
    /** Peripheral Type ID */
    Uint16  tid; 
    /** Peripheral Class ID  */
    Uint16  cid;
    /** Peripheral Revision Number  */
    Uint16  prev; 
} CSL_PdmaPerId;

/** @brief The structure contains the buffer size */ 
typedef struct  {
    /** Peripheral Buffer Size bit */
    volatile Uint16 pbuf;
    /** Switch Buffer Size bit */
    volatile Uint16 sbuf;
} CSL_BufSize;

/** @brief The structure contains the channel transfer control */ 
typedef struct  {
    /** start bit of PDMA Transfer Control Register */
    volatile Uint16 strt;
    /** Interrupt Mode bit of PDMA Transfer Control Register */
    volatile Uint16 imod;
    /** Interrupt Enable bit of PDMA Transfer Control Register */
    volatile Uint16 ie;
    /** Subsystem Interrupt bit of PDMA Transfer Control Register */
    volatile Uint16 sint;
    /** Peripheral element size bit of PDMA Transfer Control Register */
    volatile Uint16 psiz;
    /** Switch element size bit of PDMA Transfer Control Register */
    volatile Uint16 ssiz;
    /** FIFO Full Flag bit of PDMA Transfer Control Register */
    volatile Uint16 ff;
    /** Block Count Equal Zero bit of PDMA Transfer Control Register */
    volatile Uint16 bcz;
    /** Write Pointer bit of PDMA Transfer Control Register */
    volatile Uint16 wrptr;
    /** Read Pointer bit of PDMA Transfer Control Register */
    volatile Uint16 rdptr;
} CSL_ChanTferCtl;

/** @brief The structure contains the channerl context information */ 
typedef struct  {
    /** Source Address Register */
    volatile Uint32 SAR;
    /** Peripheral Address Register */
    volatile Uint32 PAR;
    /** Transfer Count/Size register */
    volatile CSL_BufSize *cnt_buff;
    /** Transfer control register */
    volatile CSL_ChanTferCtl *chTctl;
}CSL_ChannelContext ;

/** @brief The structure contains the statistics information */ 
typedef struct  {
    /** Stall count */
    volatile Uint32 wait;
    /** error count */
    volatile Uint32 error;
}CSL_Statistics;
/** 
 * @brief Enumeration for commands passed to @a CSL_pdmaHwControl()
 *
 * This is used to select the commands to control the operations
 * existing setup of PDMA. The arguments to be passed with each
 * enumeration if any are specified next to the enumeration.
 */
typedef enum {
    /** ^ Pdma global enable */
    CSL_PDMA_CMD_GLOBAL_ENABLE  = 0,
    
    /** ^ Pdma global disable */
    CSL_PDMA_CMD_GLOBAL_DISABLE = 1,
    
    /** ^ Pdma global status */
    CSL_PDMA_CMD_GLOBAL_STATUS  = 2, 
    
    /** ^ Pdma clear global statistics */
    CSL_PDMA_CMD_CLR_STATISTICS = 3,
    
    /** Pdma get channel */
    CSL_PDMA_CMD_GET_CHANNEL    = 4,
    
    /** Pdma close channel */
    CSL_PDMA_CMD_CLOSE_CHANNEL  = 5,
    
    /** Pdma channel enable */
    CSL_PDMA_CMD_CHANNEL_ENABLE = 6,
    
    /** Pdma channel disable */
    CSL_PDMA_CMD_CHANNEL_DISABLE  = 7,
    
    /** ^Pdma interrupt enable */
    CSL_PDMA_CMD_INTERRUPT_ENABLE = 8,
    
    /** ^Pdma interrupt disable */
    CSL_PDMA_CMD_INTERRUPT_DISABLE = 9,
    
    /** ^ Pdma peripheral control sync enable */
    CSL_PDMA_CMD_PERI_CTL_SYNC_EN = 10,
    
    /** ^ Pdma peripheral control sync disable */
    CSL_PDMA_CMD_PERI_CTL_SYNC_DIS = 11,
    
    /** ^ Pdma super frame sync operation enable */
    CSL_PDMA_CMD_PERI_CTL_SUPERFS_EN = 12,
    
    /** ^ Pdma super frame sync operation disable */
    CSL_PDMA_CMD_PERI_CTL_SUPERFS_DIS = 13 
} CSL_PdmaControlCmd;

/** @brief The structure contains the PDMA hardware setup */ 
typedef struct CSL_PdmaHwSetup {
    /** Pdma global setup */
    CSL_PdmaGblSetup         *gbl;
    /** Pdma peripheral control setup */
    CSL_PdmaPeriCtrlSetup    *pctl;
    		
} CSL_PdmaHwSetup;
 
/** @brief The structure contains the channel setup */ 
typedef struct CSL_PdmaChHwSetup {
    /** Pdma channel context hardware setup */
    CSL_ChannelContext   *chContext;
} CSL_PdmaChHwSetup;

/** @brief This structure/object holds the context of the instance of PDMA 
  *        opened using CSL_pdmaOpen() function. Pointer to this object is 
  *        passed as PDMA Handle to all PDMA CSL APIs. CSL_pdmaOpen() function 
  *        initializes this structure based on the parameters passed 
  */
typedef struct CSL_PdmaObj {
    /** Pointer to the register overlay structure of the hpi */
    CSL_PdmaRegsOvly    regs;
    /** Instance of pdma being referred by this object */
    CSL_InstNum 	perNum;
} CSL_PdmaObj;

/** @brief Holds the base-address information for UTOPIA2 peripheral instance */
typedef struct {
    /** Base-address of the Configuration registers of UTOPIA2. */
    CSL_PdmaRegsOvly    regs;
} CSL_PdmaBaseAddress;

/** @brief UTOPIA2 specific parameters. Present implementation doesn't have
 *  any specific parameters.
 */
typedef struct {
    /** Bit mask to be used for module specific parameters.
     *  The below declaration is just a place-holder for future
     *  implementation.
     */
    CSL_BitMask16   flags;
} CSL_PdmaParam;

/** @brief This data type is used to return the handle to the CSL of PDMA */
typedef struct CSL_PdmaObj *CSL_PdmaHandle;

#define CSL_PDMA_HWSETUP_DEFAULTS   \
{ {0x0, 0x0, 0x0}, \
  {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, \
   0x0, 0x0, 0x0, 0x0, 0x0, 0x0} \
}

/******************* PIM PDMA global function declarations ********************/
/** ============================================================================
 *   @n@b CSL_pdmaInit
 *
 *   @b Description
 *   @n This is the initialization function for the PDMA CSL. The function must 
 *      be called before calling any other API from this CSL. This function does 
 *      not modify any registers or check status. It returns status CSL_SOK. It 
 *      has been kept for future use.
 *
 *   @b Arguments
     @verbatim
            pContext   Context information for the instance. Should be NULL

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK - Always returns
 *
 *   <b> Pre Condition </b>
 *   @n  This function should be called before using any of the CSL PDMA APIs
 *
 *   <b> Post Condition </b>
 *   @n  The CSL for PDMA is initialized
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim
        CSL_Status status;
        ...
        status = CSL_pdmaInit(NULL);
        ...
     @endverbatim
 *  ===========================================================================
 */
extern CSL_Status  CSL_pdmaInit (
    CSL_PdmaContext    *pContext
);

/** ============================================================================
 *   @n@b CSL_pdmaOpen
 *
 *   @b Description
 *   @n This function returns the handle to the PDMA instance. The open call 
 *      sets up the data structures for the particular instance of PDMA. The 
 *      handle returned by this call is input argument for rest of the PDMA CSL 
 *      APIs.
 *
 *   @b Arguments
 *   @verbatim            
            pPdmaObj       Pointer to the object that holds reference to the
                           instance of PDMA requested after the call. Memory
                           for this object should be allocated by the user.
 
            pdmaNum        Instance of PDMA to which a handle is requested 
 
            pPdmaParam     Pointer to module specific parameters
             
            pStatus        Pointer to the variable that holds the status of the 
                           open call

     @endverbatim
 *
 *   <b> Return Value </b>  
 *      CSL_PdmaHandle
 *        CSL_PdmaHandle to the requested instance of PDMA if the call is 
 *        successful, otherwise @a NULL is returned.
 *
 *   <b> Pre Condition </b>
 *   @n  The PDMA must be successfully initialized via CSL_pdmaInit() before 
 *       calling this function. 
 *       Memory for the @a CSL_PdmaObj must be allocated outside this call. This 
 *       object must be retained while using this peripheral instance.
 *
 *   <b> Post Condition </b>
 *   @n   1.    The status is returned in the status variable. If status
 *              returned is
 *   @li            CSL_SOK            - Valid PDMA handle is returned
 *   @li            CSL_ESYS_FAIL      - The PDMA instance is invalid
 *   @li            CSL_ESYS_INVPARAMS - Invalid parameter
 *   2. PDMA object structure is populated.
 *
 *   @b  Modifies
 *    1. The status variable
 *    2. object structure
 *
 *   @b Example:
 *   @verbatim
        CSL_PdmaHandle  hPdma;
        CSL_PdmaObj     pdmaObj;
        CSL_Status       status;
        ...
        hPdma = CSL_pdmaOpen(&pdmaObj, CSL_PDMA, NULL, &status);
        ...
     @endverbatim
 *
 * ===========================================================================
 */
extern CSL_PdmaHandle CSL_pdmaOpen (
    CSL_PdmaObj         *pPdmaObj,
    CSL_InstNum         pdmaNum,
    CSL_PdmaParam       *pPdmaParam,
    CSL_Status          *pStatus
);

/** ============================================================================
 *   @n@b CSL_pdmaClose
 *
 *   @b Description
 *   @n Unreserves the PDMA identified by the handle passed.
 *
 *   @b Arguments
 *   @verbatim
            hPdma            Handle to the PDMA

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Close Successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_pdmaInit() and CSL_pdmaOpen() must be called successfully 
 *       in order before calling CSL_pdmaClose(). 
 * 
 *   <b> Post Condition </b>
 *   @n  PDMA CSL APIs can not be called until the PDMA CSL is reopened again 
 *       using CSL_pdmaOpen().
 *
 *   @b Modifies
 *   @n PDMA Handle
 *
 *   @b Example
 *   @verbatim
            CSL_Status        status;
            CSL_pdmaHandle    hPdma;
            ...
            status = CSL_pdmaClose(hPdma);
            ...
     @endverbatim
 *  ============================================================================
 */
extern CSL_Status CSL_pdmaClose (
    CSL_PdmaHandle    hPdma
);

/** ============================================================================
 *   @n@b CSL_pdmaHwSetup
 *
 *   @b Description
 *   @n This function initializes the device registers with the appropriate 
 *      values provided through the HwSetup data structure. For information 
 *      passed through the HwSetup data structure, refer CSL_PdmaHwSetup. 
 *
 *   @b Arguments
 *   @verbatim
            hPdma           PDMA handle returned by successful 'open'

            setup           Pointer to the intialized setup structure 

            eventGroup      Hardware setup for the specified event group 
            
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - HwSetup Successful
 *   @li                    CSL_ESYS_BADHANDLE  - Handle passed is invalid 
 *   @li                    CSL_ESYS_INVPARAMS  - The param passed is invalid
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_pdmaInit() and CSL_pdmaOpen() must be called successfully 
 *       in order before calling CSL_pdmaHwSetup(). 
 *
 *   <b> Post Condition </b>
 *   @n The registers of the specified PDMA instance will be setup
 *      according to value passed.
 *
 *   @b Modifies
 *   @n PDMA registers
 *
 *   @b Example
 * @verbatim
        CSL_PdmaHandle hPdma;
        CSL_PdmaHwSetup setup = CSL_PDMA_HWSETUP_DEFAULTS;
        ...
        // pdma is initialized and opened successfully.
        ...
        CSL_pdmaHwSetup(hPdma, &setup);
        ...
  @endverbatim
 * =============================================================================
 */
 
extern CSL_Status  CSL_pdmaHwSetup (
    CSL_PdmaHandle     hPdma,
    CSL_PdmaHwSetup    *setup,
    Uint32             eventGrp 
);

/** ============================================================================
 *   @n@b CSL_pdmaHwControl
 *
 *   @b Description
 *   @n This function takes an input control command with an optional argument
 *      and accordingly controls the operation/configuration of PDMA.
 *
 *   @b Arguments
 *   @verbatim
            hPdma       PDMA handle returned by successful 'open'

            ctrlCmd     The command to this API indicates the action to be
                        taken on PDMA.
                        Control command, refer @a CSL_PdmaControlCmd for the 
                        list of commands supported

            eventGrp    Hardware setup for the specified event group
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li         CSL_SOK               - Command successful.
 *   @li         CSL_ESYS_BADHANDLE    - Handle passed is invalid
 *   @li         CSL_ESYS_INVCMD       - Command passed is invalid
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_pdmaInit() and CSL_pdmaOpen() must be called successfully 
 *       in order before calling CSL_pdmaHwControl(). 
 *
 *   <b> Post Condition </b>
 *   @n  PDMA registers are configured according to the command passed
 *
 *   @b Modifies
 *   @n The hardware registers of PDMA.
 *
 *   @b Example
 *   @verbatim
       CSL_Status status;
       CSL_BitMask16 ctrlMask;
       CSL_PdmaHandle hPdma;
       ...
       // pdma initialize and opened successfully.
       ...
       status = CSL_pdmaHwControl(hPdma, PDMA_GLOBAL_STATUS, EVENT_GROUP_ONE);
     @endverbatim
 * =============================================================================
 */
extern CSL_Status CSL_pdmaHwControl (
	CSL_PdmaHandle        hPdma,
	CSL_PdmaControlCmd    ctrlCmd,
	Uint32                eventGrp
);

/** ============================================================================
 *   @n@b CSL_pdmaChHwSetup
 *
 *   @b Description
 *   @n This function initializes the pdma channel. 
 *
 *   @b Arguments
 *   @verbatim
            hPdma         PDMA handle returned by successful 'open'

            chsetup       Pointer to the intialized channel setup structure 

            chNum         Pdma channel number 
            
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - channel Hwsetup Successful
 *   @li                    CSL_ESYS_BADHANDLE  - Handle passed is invalid 
 *   @li                    CSL_ESYS_INVPARAMS  - The param passed is invalid
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_pdmaInit() and CSL_pdmaOpen() must be called successfully 
 *       in order before calling CSL_pdmaChHwSetup(). 
 *
 *   <b> Post Condition </b>
 *   @n The registers of the specified PDMA channel will be setup
 *      according to value passed.
 *
 *   @b Modifies
 *   @n PDMA registers
 *
 *   @b Example
 * @verbatim
        CSL_PdmaHandle hPdma;
        CSL_PdmaChHwSetup  chSetup;
        ...
        // pdma Initilize and opened Successfully.
        ...
        CSL_pdmaChHwSetup(hPdma, &chSetup);
        ...
  @endverbatim
 * =============================================================================
 */
extern CSL_Status  CSL_pdmaChHwSetup (
    CSL_PdmaHandle    hPdma,
    CSL_PdmaChHwSetup     *chSetup,
    Uint32 		      chNum 
);

/** ============================================================================
 *   @n@b CSL_pdmaChHwControl
 *
 *   @b Description
 *   @n This function takes an input control command with an optional argument
 *      and accordingly controls the operation/configuration of PDMA channel.
 *
 *   @b Arguments
 *   @verbatim
            hPdma       PDMA handle returned by successful 'open'

            ctrlCmd     The command to this API indicates the action to be
                        taken on PDMA.
                        Control command, refer @a CSL_PdmaControlCmd for the 
                        list of commands supported

            chnum       PDMA channel number


     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li         CSL_SOK               - Command successful.
 *   @li         CSL_ESYS_BADHANDLE    - Handle passed is invalid
 *   @li         CSL_ESYS_INVCMD       - Command passed is invalid
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_pdmaInit() and CSL_pdmaOpen() must be called successfully 
 *       in order before calling CSL_pdmaChHwControl(). 
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n The hardware registers of PDMA.
 *
 *   @b Example
 *   @verbatim
       CSL_Status status;
       CSL_PdmaHandle hPdma;
       CSL_PdmaControlCmd    ctrlCmd = CSL_PDMA_CMD_CHANNEL_ENABLE;
       ...
       // pdma initialized and opened successfully.
       ...
       status = CSL_pdmaChHwControl(hPdma, ctrlCmd, 0);
     @endverbatim
 * =============================================================================
 */
extern CSL_Status CSL_pdmaChHwControl (
	CSL_PdmaHandle        hPdma,
	CSL_PdmaControlCmd    ctrlCmd,
	Uint32                chnum
);

/** ============================================================================
 *   @n@b CSL_pdmaGetHwSetup
 *
 *   @b Description
 *   @n Gets the output status of PDMA
 *
 *   @b Arguments
 *   @verbatim
        hPdma            Handle to the PDMA instance

        setup            Pointer to the hard ware setup 

        eventGrp         Hardware setup for the specified event group

 *   @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - HwSetup query successful.
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle.
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter.
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_pdmaInit() and CSL_pdmaOpen() must be called successfully 
 *       in order before calling this function.
 *
 *   <b> Post Condition </b>
 *   @n  The hardware setup structure is populated with the hardware setup
 *       parameters
 *
 *   @b Modifies
 *   @n hwSetup variable
 *
 *   @b Example
 *   @verbatim
        CSL_PdmaHandle     hPdma;
        CSL_Status         status;
        CSL_PdmaHwSetup    *setup
        ...
        // pdma is initialized and opened successfully.
        ...
        status = CSL_pdmaGetHwSetup (hPdma, setup);
        ...
     @endverbatim
 * ===========================================================================
 */
extern CSL_Status CSL_pdmaGetHwSetup ( 
    CSL_PdmaHandle    hPdma,
    CSL_PdmaHwSetup   *setup,
    Uint32             eventGroup
);

/** ============================================================================
 *   @n@b CSL_pdmaGetHwStatus
 *
 *   @b Description
 *   @n Gets the status of the different operations of PDMA.
 *
 *   @b Arguments
 *   @verbatim
            hPdma            Handle to the PDMA instance


            query           The query to this API of PDMA which indicates the
                            status to be returned.

            response        Placeholder to return the status.
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Status info return successful.
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li                    CSL_ESYS_INVQUERY   - Invalid querry
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_heccInit() and CSL_heccOpen() must be called successfully 
 *       in order before calling CSL_heccGetHwStatus(). 
 *       Refer to @a CSL_HeccHwStatusQuery for the argument to be passed along 
 *       with the corresponding query command.
 *
 *   <b> Post Condition </b>
 *   @n  Data requested by the query is returned through the variable "response"
 *
 *   @b Modifies
 *   @n The Third parameter 'response'.
 *
 *   @b Example
 *   @verbatim
        CSL_PdmaHandle    hPdma;
        void              reponse;
        ...
        status = CSL_pdmaGetHwStatus (hPdma, CSL_PDMA_QUERY_STATISTIC_REGS,
                                      &response);
        ...
     @endverbatim
 * =============================================================================
 */
extern CSL_Status CSL_pdmaGetHwStatus (
    CSL_PdmaHandle          hPdma,
    CSL_PdmaHwStatusQuery   query,
    void                    *response 
);

#ifdef __cplusplus
}
#endif

#endif

