/* =============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ============================================================================
 *   @file  csl_pdmaAux.h
 *
 *   @path  $(CSLPATH)\inc
 *
 *   @desc  Header file for functional layer of CSL
 *    - This decribes the definitions of the inline functions
 *
 */
/* =============================================================================
 *  Revision History
 *  ===============
 *  23-Mar-2006 KNG  Creted for Tomahawk
 *
 * =============================================================================
 */
#ifndef _CSL_PDMAAUX_H_
#define _CSL_PDMAAUX_H_

#include <csl_pdma.h>

#ifdef __cplusplus
extern "C" {
#endif


/* Inline functions for status querry */
/** ============================================================================
 *   @n@b CSL_pdmaGetVersion
 *
 *   @b Description
 *   @n This function gets the peripheral revision
 *
 *   @b Arguments
 *   @verbatim

            hPdma           Handle to PDMA instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Pdma must be initialized and opened properly
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  The parameter response
 *
 *   @b Example
 *   @verbatim
        CSL_pdmaHandle    hPdma;
        void*             responsel;
        ...
        // The pdma is initialized and opened successfully.
        ...
        CSL_pdmaGetVersion(hPdma, response);
        ...
     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_pdmaGetVersion (
    CSL_PdmaHandle    hPdma,
    void              *response
)
{
    ((CSL_PdmaPerId*)response)->tid = 
                        CSL_FEXT(hPdma->regs->PID, PIM_PID_TID);
    ((CSL_PdmaPerId*)response)->cid = 
                        CSL_FEXT(hPdma->regs->PID, PIM_PID_CID);
    ((CSL_PdmaPerId*)response)->prev =
                        CSL_FEXT(hPdma->regs->PID, PIM_PID_PREV); 
}


/** ============================================================================
 *   @n@b CSL_pdmaGetStatistics
 *
 *   @b Description
 *   @n This function gets the statistics
 *
 *   @b Arguments
 *   @verbatim

            hPdma           Handle to PDMA instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Pdma must be initialized and opened properly
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  The parameter response
 *
 *   @b Example
 *   @verbatim
        CSL_pdmaHandle    hPdma;
        void*             responsel;
        ...
        // The pdma is initialized and opened successfully.
        ...
        CSL_pdmaGetStatistics(hPdma, response);
        ...
     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_pdmaGetStatistics (
    CSL_PdmaHandle    hPdma,
    void              *response
)
{
    Uint32 index = (*(Uint32*)response);
    ((CSL_Statistics*)response)->wait = 
                       CSL_FEXT(hPdma->regs->STATR[index], PIM_STATR_WAIT);
    ((CSL_Statistics*)response)->error = 
                       CSL_FEXT(hPdma->regs->STATR[index], PIM_STATR_ERROR);
}

/** ============================================================================
 *   @n@b CSL_pdmaGetPeriCtrl
 *
 *   @b Description
 *   @n This function gets the peripheral control information
 *
 *   @b Arguments
 *   @verbatim

            hPdma           Handle to PDMA instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Pdma must be initialized and opened properly
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  The parameter response
 *
 *   @b Example
 *   @verbatim
        CSL_pdmaHandle    hPdma;
        void*             responsel;
        ...
        // The pdma is initialized and opened successfully.
        ...
        CSL_pdmaGetPeriCtrl(hPdma, response);
        ...
     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_pdmaGetPeriCtrl (
    CSL_PdmaHandle    hPdma,
    void              *response
)
{
    Uint32 index = *((Uint32*)response);

 	((CSL_PdmaPeriCtrlSetup*)response)->bend = 
 	   					CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_BEND);
    ((CSL_PdmaPeriCtrlSetup*)response)->clrs  = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_CLRS);
    ((CSL_PdmaPeriCtrlSetup*)response)->sfrm  = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_SFRM);
    ((CSL_PdmaPeriCtrlSetup*)response)->pe    = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_PE);
    ((CSL_PdmaPeriCtrlSetup*)response)->sync  = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_SYNC);
    ((CSL_PdmaPeriCtrlSetup*)response)->pri   = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_PRI);
    ((CSL_PdmaPeriCtrlSetup*)response)->abu   = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_ABU);
    ((CSL_PdmaPeriCtrlSetup*)response)->dir   = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_DIR);
    ((CSL_PdmaPeriCtrlSetup*)response)->pblen = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_PBLEN);
    ((CSL_PdmaPeriCtrlSetup*)response)->pmod  = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_PMOD);
    ((CSL_PdmaPeriCtrlSetup*)response)->sblen = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_SBLEN);
    ((CSL_PdmaPeriCtrlSetup*)response)->smod  = 
                        CSL_FEXT(hPdma->regs->PCR[index], PIM_PCR_SMOD);
   
}


/** ============================================================================
 *   @n@b CSL_pdmaGetMemCtrl
 *
 *   @b Description
 *   @n This function gets the memory control information
 *
 *   @b Arguments
 *   @verbatim

            hPdma           Handle to PDMA instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Pdma must be initialized and opened properly
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  The parameter response
 *
 *   @b Example
 *   @verbatim
        CSL_pdmaHandle    hPdma;
        void*             responsel;
        ...
        // The pdma is initialized and opened successfully.
        ...
        CSL_pdmaGetMemCtrl(hPdma, response);
        ...
     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_pdmaGetMemCtrl (
    CSL_PdmaHandle    hPdma,
    void              *response
)
{

    Uint16 index = *((Uint16*)response);
    
    ((CSL_PdmaMemCtrlSetup*)response)->sync  = 
                        CSL_FEXT(hPdma->regs->MMCR[index], PIM_MMCR_SYNC);
    ((CSL_PdmaMemCtrlSetup*)response)->pri  = 
                        CSL_FEXT(hPdma->regs->MMCR[index], PIM_MMCR_PRI);
    ((CSL_PdmaMemCtrlSetup*)response)->abu  = 
                        CSL_FEXT(hPdma->regs->MMCR[index], PIM_MMCR_ABU);
    ((CSL_PdmaMemCtrlSetup*)response)->dir  = 
                        CSL_FEXT(hPdma->regs->MMCR[index], PIM_MMCR_DIR);
    ((CSL_PdmaMemCtrlSetup*)response)->pblen  = 
                        CSL_FEXT(hPdma->regs->MMCR[index], PIM_MMCR_PBLEN);
    ((CSL_PdmaMemCtrlSetup*)response)->pmod  = 
                        CSL_FEXT(hPdma->regs->MMCR[index], PIM_MMCR_PMOD);
    ((CSL_PdmaMemCtrlSetup*)response)->sblen  = 
                        CSL_FEXT(hPdma->regs->MMCR[index], PIM_MMCR_SBLEN);
    ((CSL_PdmaMemCtrlSetup*)response)->smod  = 
                        CSL_FEXT(hPdma->regs->MMCR[index], PIM_MMCR_SMOD);
}

/** ============================================================================
 *   @n@b CSL_pdmaGetGlobalStatus
 *
 *   @b Description
 *   @n This function gets the global status
 *
 *   @b Arguments
 *   @verbatim

            hPdma           Handle to PDMA instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Pdma must be initialized and opened properly
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  The parameter response
 *
 *   @b Example
 *   @verbatim
        CSL_pdmaHandle    hPdma;
        void*             responsel;
        ...
        // The pdma is initialized and opened successfully.
        ...
        CSL_pdmaGetGlobalStatus(hPdma, response);
        ...
     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_pdmaGetGlobalStatus (
    CSL_PdmaHandle    hPdma,
    void              *response
)
{
    ((CSL_PdmaGblSetup*)response)->strt  = 
                        CSL_FEXT(hPdma->regs->GCSR, PIM_GCSR_STRT);
    ((CSL_PdmaGblSetup*)response)->soft  = 
                        CSL_FEXT(hPdma->regs->GCSR, PIM_GCSR_SOFT);
    ((CSL_PdmaGblSetup*)response)->free  = 
                        CSL_FEXT(hPdma->regs->GCSR, PIM_GCSR_FREE);
}


/** ============================================================================
 *   @n@b CSL_pdmaGetChannelInfo
 *
 *   @b Description
 *   @n This function gets the channel contex information
 *
 *   @b Arguments
 *   @verbatim

            hPdma           Handle to PDMA instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Pdma must be initialized and opened properly
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  The parameter response
 *
 *   @b Example
 *   @verbatim
        CSL_pdmaHandle    hPdma;
        void*             responsel;
        ...
        // The pdma is initialized and opened successfully.
        ...
        CSL_pdmaGetChannelInfo(hPdma, response);
        ...
     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_pdmaGetChannelInfo (
    CSL_PdmaHandle    hPdma,
    void              *response
)
{
    CSL_BufSize      cntBuf;
    CSL_ChanTferCtl  tctl;
    Uint32           index = *((Uint32*)response);

    ((CSL_ChannelContext*)response)->SAR = hPdma->regs->CHPROXY[index].SAR;
    ((CSL_ChannelContext*)response)->PAR = hPdma->regs->CHPROXY[index].PAR;
     
    cntBuf.pbuf = CSL_FEXT(hPdma->regs->CHPROXY[index].BUFSIZE, 
                           PIM_BUFSIZE_PBUF);
    cntBuf.sbuf = CSL_FEXT(hPdma->regs->CHPROXY[index].BUFSIZE, 
                           PIM_BUFSIZE_SBUF);
    ((CSL_ChannelContext*)response)->cnt_buff = &cntBuf; 

    tctl.strt  = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_STRT);
    tctl.imod  = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_IMOD);
    tctl.ie    = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_IE);
    tctl.sint  = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_SINT);
    tctl.psiz  = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_PSIZ);
    tctl.ssiz  = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_SSIZ);
    tctl.ff    = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_FF);
    tctl.bcz   = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_BCZ);
    tctl.wrptr = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_WRPTR);
    tctl.rdptr = CSL_FEXT(hPdma->regs->CHPROXY[index].TCR, PIM_TCR_RDPTR);
    ((CSL_ChannelContext*)response)->chTctl = &tctl;

}

#ifdef __cplusplus
}
#endif

#endif /* CSL_PDMAAUX_H_ */




