/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** @file  csl_pllc.h
 *
 *  @brief PLLC functional layer API header file
 *
 *  Path: \(CSLPATH)\inc
 */

/** @mainpage PLLC CSL 3.x
 *
 * @section Introduction
 *
 * @subsection xxx Purpose and Scope
 * The purpose of this document is to identify a set of common CSL APIs for
 * the PLLC module across various devices. The CSL developer is expected to
 * refer to this document while designing APIs for these modules. Some of the
 * listed APIs may not be applicable to a given PLLC module. While other cases
 * this list of APIs may not be sufficient to cover all the features of a
 * particular PLLC Module. The CSL developer should use his discretion designing
 * new APIs or extending the existing ones to cover these.
 *
 * @subsection aaa Terms and Abbreviations
 *   -# CSL:  Chip Support Library
 *   -# API:  Application Programmer Interface
 *
 * @subsection References
 *    -# CSL-001-DES, CSL 3.x Design Specification Document Version 1.02
 *
 */

/* =============================================================================
 *  Revision History
 *  ===============
 *  10-Feb-2004 kpn CSL3X Upgradation.
 *  16-May-2006 RR   updated for Tomahawk
 * =============================================================================
 */

#ifndef _CSL_PLLC_H_
#define _CSL_PLLC_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <cslr.h>
#include <soc.h>
#include <csl_error.h>
#include <csl_types.h>
#include <cslr_pllc.h>

/**
\defgroup CSL_PLLC_API PLLC
   \internal Based on <b>PLLC Spec Rev 3.4 </b>
*/
/**
\defgroup CSL_PLLC_DATASTRUCT Data Structures
\ingroup CSL_PLLC_API
*/
/**
\defgroup CSL_PLLC_DEFINE  Defines
\ingroup CSL_PLLC_API
*/
/**
\defgroup CSL_PLLC_ENUM  Enumerated Data Types
\ingroup CSL_PLLC_API
*/

/**
\defgroup CSL_PLLC_FUNCTION  Functions
\ingroup CSL_PLLC_API
*/

/*****************************************************************************\
          PLLC global macro declarations
\*****************************************************************************/

/** \defgroup CSL_PLLC_STATUS_DEFINE PLL Controller Status
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */

/** Set when GO operation (divide-ratio change and
 *   clock alignment) is in progress
 */
#define CSL_PLLC_STATUS_GO CSL_FMKT (PLLC_PLLSTAT_GOSTAT, INPROG)

/** Set when OSCIN/CLKIN is assumed to be stable */
#define CSL_PLLC_STATUS_STABLE CSL_FMKT (PLLC_PLLSTAT_STABLE, YES)

/**
@} */

/** \defgroup CSL_PLLC_DCHANGESTAT_DEFINE PLLC Divider Ratio Modified Status
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */

/** SYSCLK10 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS10 CSL_FMKT (PLLC_DCHANGE_SYS10, YES)
/** SYSCLK9 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS9 CSL_FMKT (PLLC_DCHANGE_SYS9, YES)
/** SYSCLK8 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS8 CSL_FMKT (PLLC_DCHANGE_SYS8, YES)
/** SYSCLK7 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS7 CSL_FMKT (PLLC_DCHANGE_SYS7, YES)
/** SYSCLK18 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS6 CSL_FMKT (PLLC_DCHANGE_SYS6, YES)
/** SYSCLK17 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS5 CSL_FMKT (PLLC_DCHANGE_SYS5, YES)
/** SYSCLK16 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS4 CSL_FMKT (PLLC_DCHANGE_SYS4, YES)
/** SYSCLK15 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS3 CSL_FMKT (PLLC_DCHANGE_SYS3, YES)
/** SYSCLK14 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS2 CSL_FMKT (PLLC_DCHANGE_SYS2, YES)
/** SYSCLK13 divide ratio is modified */
#define CSL_PLLC_DCHANGESTAT_SYS1 CSL_FMKT (PLLC_DCHANGE_SYS1, YES)

/**
@} */


/** \defgroup CSL_PLLC_RESETSTAT_DEFINE PLLC Last Reset Status
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */

/** Power On Reset */
#define CSL_PLLC_RESETSTAT_POR CSL_FMKT (PLLC_RSTYPE_POR, YES)
/** External Warm Reset */
#define CSL_PLLC_RESETSTAT_XWRST CSL_FMKT (PLLC_RSTYPE_XWRST, YES)
/** Maximum Reset */
#define CSL_PLLC_RESETSTAT_MRST CSL_FMKT (PLLC_RSTYPE_MRST, YES)
/** System/Chip Reset */
#define CSL_PLLC_RESETSTAT_SRST CSL_FMKT (PLLC_RSTYPE_SRST, YES)

/**
@} */


/** \defgroup CSL_PLLC_CTRL_DEFINE PLLC Control Mask
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */

/** PreDiv, PLL, and PostDiv are bypassed. SYSCLK
 *   divided down directly from input reference
 *   clock refclk
 */
#define CSL_PLLC_CTRL_BYPASS CSL_FMKT (PLLC_PLLCTL_PLLEN, BYPASS)
/** PLL is used. SYSCLK divided down from PostDiv
 *   output
 */
#define CSL_PLLC_CTRL_ENABLE CSL_FMKT (PLLC_PLLCTL_PLLEN, PLL)
/** clkin_pi is the reference clock */
#define CSL_PLLC_CTRL_SELECT_CLKIN CSL_FMKT (PLLC_PLLCTL_CLKMODE, CLKIN)
/** A write of 1 to this bit signifies that the new
 *   divide ratios in PLLDIV[1:n] are taken into
 *   account at the nearest possible rising edge to
 *   phase align the clocks. The actual SYSCLKx to
 *   be aligned are selected in register ALNCTL
 */
#define CSL_PLLC_CTRL_ALIGN_PHASE (CSL_FMKT (PLLC_PLLCMD_GOSET, SET)<< 16)


/**
@} */

/** \defgroup CSL_PLLC_ALIGNCTL_DEFINE PLLC Align Control
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */

/** SYSCLK10 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK10 CSL_FMKT (PLLC_ALNCTL_ALN10, YES)
/** SYSCLK9 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK9 CSL_FMKT (PLLC_ALNCTL_ALN9, YES)
/** SYSCLK8 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK8 CSL_FMKT (PLLC_ALNCTL_ALN8, YES)
/** SYSCLK7 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK7 CSL_FMKT (PLLC_ALNCTL_ALN7, YES)
/** SYSCLK18 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK6 CSL_FMKT (PLLC_ALNCTL_ALN6, YES)
/** SYSCLK17 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK5 CSL_FMKT (PLLC_ALNCTL_ALN5, YES)
/** SYSCLK16 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK4 CSL_FMKT (PLLC_ALNCTL_ALN4, YES)
/** SYSCLK15 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK3 CSL_FMKT (PLLC_ALNCTL_ALN3, YES)
/** SYSCLK14 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK2 CSL_FMKT (PLLC_ALNCTL_ALN2, YES)
/** SYSCLK13 needs to be aligned with other clocks
 *   selected in this register
 */
#define CSL_PLLC_ALIGNCTL_SYSCLK1 CSL_FMKT (PLLC_ALNCTL_ALN1, YES)

/**
@} */

/** \defgroup CSL_PLLC_DIVEN_DEFINE PLLC Divider Enable
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */

/** Enable divider D1 for SYSCLK13 */
#define CSL_PLLC_DIVEN_PLLDIV1   (1 << 3)
/** Enable divider D2 for SYSCLK14 */
#define CSL_PLLC_DIVEN_PLLDIV2   (1 << 4)
/** Enable divider D3 for SYSCLK15 */
#define CSL_PLLC_DIVEN_PLLDIV3   (1 << 5)
/** Enable divider D4 for SYSCLK16 */
#define CSL_PLLC_DIVEN_PLLDIV4   (1 << 6)
/** Enable divider D5 for SYSCLK17 */
#define CSL_PLLC_DIVEN_PLLDIV5   (1 << 7)
/** Enable divider D6 for SYSCLK18 */
#define CSL_PLLC_DIVEN_PLLDIV6   (1 << 8)
/** Enable divider D7 for SYSCLK7 */
#define CSL_PLLC_DIVEN_PLLDIV7   (1 << 9)
/** Enable divider D8 for SYSCLK8 */
#define CSL_PLLC_DIVEN_PLLDIV8   (1 << 10)
/** Enable divider D9 for SYSCLK9 */
#define CSL_PLLC_DIVEN_PLLDIV9   (1 << 11)
/** Enable divider D10 for SYSCLK10 */
#define CSL_PLLC_DIVEN_PLLDIV10   (1 << 12)


/**
@} */

/** \defgroup CSL_PLLC_DIVSEL_DEFINE Divider Select for SYSCLKs
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */

/** select Divider D1 for SYSCLK13 */
#define CSL_PLLC_DIVSEL_PLLDIV1     (1)
/** Select Divider D2 for SYSCLK14 */
#define CSL_PLLC_DIVSEL_PLLDIV2     (2)
/** Select Divider D3 for SYSCLK15 */
#define CSL_PLLC_DIVSEL_PLLDIV3     (3)
/** Select Divider D4 for SYSCLK16 */
#define CSL_PLLC_DIVSEL_PLLDIV4     (4)
/** Select Divider D5 for SYSCLK17 */
#define CSL_PLLC_DIVSEL_PLLDIV5     (5)
/** Select Divider D6 for SYSCLK18 */
#define CSL_PLLC_DIVSEL_PLLDIV6     (6)
/** Select Divider D6 for SYSCLK7 */
#define CSL_PLLC_DIVSEL_PLLDIV7     (7)
/** Select Divider D6 for SYSCLK8 */
#define CSL_PLLC_DIVSEL_PLLDIV8     (8)
/** Select Divider D6 for SYSCLK9 */
#define CSL_PLLC_DIVSEL_PLLDIV9     (9)
/** Select Divider D10 for SYSCLK10 */
#define CSL_PLLC_DIVSEL_PLLDIV10    (10)

/** \defgroup CSL_PLLC_DIVRATIO_DEFINE Divider Select for SYSCLKs
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */

/** Divider D1 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV1     (0)
/** Divider D2 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV2     (1)
/** Divider D3 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV3     (2)
/** Divider D4 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV4     (3)
/** Divider D5 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV5     (4)
/** Divider D6 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV6     (5)
/** Divider D7 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV7     (6)
/** Divider D8 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV8     (7)
/** Divider D9 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV9     (8)
/** Divider D10 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV10    (9)
/** Divider D11 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV11    (10)
/** Divider D12 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV12    (11)
/** Divider D13 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV13    (12)
/** Divider D14 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV14    (13)
/** Divider D15 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV15    (14)
/** Divider D16 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV16    (15)
/** Divider D17 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV17    (16)
/** Divider D18 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV18    (17)
/** Divider D19 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV19    (18)
/** Divider D20 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV20    (19)
/** Divider D21 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV21    (20)
/** Divider D22 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV22    (21)
/** Divider D23 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV23    (22)
/** Divider D24 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV24    (23)
/** Divider D25 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV25    (24)
/** Divider D26 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV26    (25)
/** Divider D27 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV27    (26)
/** Divider D28 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV28    (27)
/** Divider D29 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV29    (28)
/** Divider D30 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV30    (29)
/** Divider D31 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV31    (30)
/** Divider D32 for SYSCLKn */
#define CSL_PLLC_DIVRATIO_PLLDIV32    (31)

/**
@} */

/**************************************************************************\
* PLLC global typedef declarations                                         *
\**************************************************************************/

/**
@} */

/** \defgroup CSL_PLLC_PLLDIVCTL_ENUM PLL Divide Control
 *  \ingroup CSL_PLLC_ENUM
 @{*/
/** \brief  Enums for PLL divide enable/ disable */
typedef enum {
    /** <b>: PLL Divider Disable</b> */
    CSL_PLLC_PLLDIV_DISABLE = 0,
    /** <b>: PLL Divider Enable</b> */
    CSL_PLLC_PLLDIV_ENABLE = 1
} CSL_PllcDivCtrl;
/**
@} */

/** \defgroup CSL_PLLC_CONTROLCMD_ENUM  Control Commands
 *  \ingroup CSL_PLLC_CONTROL_API
@{*/
/** \brief Enumeration for control commands passed to \a CSL_pllcHwControl()
 *
 *  This is the set of commands that are passed to the \a CSL_pllcHwControl()
 *  with an optional argument type-casted to \a void* .
 *  The arguments to be passed with each enumeration (if any) are specified
 *  next to the enumeration
 */
typedef enum {
    /** \brief Control PLL based on the bits set in the input argument
     *  \param CSL_BitMask32
     *  \return CSL_SOK
     *  \sa CSL_PLLC_CTRL_DEFINE
     */
    CSL_PLLC_CMD_PLLCONTROL,
    /** \brief By setting the ALN bits, the selected SYSCLKx (specified by
     *         input bitmask) will always be phase aligned to other clocks
     *         also selected in this register
     *  \param CSL_BitMask32
     *  \return CSL_SOK
     *  \sa CSL_PLLC_ALIGNCTL_DEFINE
     */
    CSL_PLLC_CMD_SET_PHASEALIGN,
    /** \brief Set PLL multiplier value
     *  \param Uint32
     *  \return CSL_SOK
     */
    CSL_PLLC_CMD_SET_PLLM,
    /** \brief Set PLL divide ratio
     *  \param CSL_PllcDivRatio
     *  \return CSL_SOK
     *  \sa CSL_PllcDivRatio
     */
    CSL_PLLC_CMD_SET_PLLRATIO,
    /** \brief Enable/disable  PLL divider
     *  \param CSL_PllcDivideControl
     *  \return CSL_SOK
     *  \sa CSL_PllcDivideControl
     */
    CSL_PLLC_CMD_PLLDIV_CONTROL
} CSL_PllcHwControlCmd;
/**
@} */

/** \defgroup CSL_PLLC_QUERYCMD_ENUM Query Commands
 *  \ingroup CSL_PLLC_QUERY_API
 @{ */
/** \brief Enumeration for queries passed to \a CSL_PllcGetHwStatus()
 *
 *  This is used to get the status of different operations.The arguments
 *  to be passed with each enumeration if any are specified next to
 *  the enumeration
 */
typedef enum {
    /** \brief Queries PLL Control Peripheral ID
     *  \param (Uint32*)
     *  \return CSL_SOK
     */
    CSL_PLLC_QUERY_PID,
    /** \brief Queries PLL Controller Status
     *  \param (CSL_BitMask32*)
     *  \return CSL_SOK
     *  \sa CSL_PLLC_STATUS_DEFINE
     */
    CSL_PLLC_QUERY_STATUS,
    /** \brief Queries PLLDIV Modified Status
     *  \param (CSL_BitMask32*)
     *  \return CSL_SOK
     *  \sa CSL_PLLC_DCHANGESTAT_DEFINE
     */
    CSL_PLLC_QUERY_DIVRATIO_CHANGE,
    /** \brief Queries PLL SYSCLK Status
     *  \param (CSL_BitMask32*)
     *  \return CSL_SOK
     *  \sa CSL_PLLC_SYSCLKSTAT_DEFINE
     */
    CSL_PLLC_QUERY_SYSCLKSTAT,
    /** \brief Queries Reset Type Status
     *  \param (CSL_BitMask32*)
     *  \return CSL_SOK
     *  \sa CSL_PLLC_RESETSTAT_DEFINE
     */
    CSL_PLLC_QUERY_RESETSTAT,
    /** \brief Queries Fuse Farm Error Status
     *  \param (Uint32*)
     *  \return CSL_SOK
     */
    CSL_PLLC_QUERY_EFUSEERR
} CSL_PllcHwStatusQuery;
/**
@} */

/**
\addtogroup CSL_PLLC_DATASTRUCT
@{ */
/**
 *  \brief Input parameters for setting up PLL Divide ratio
 *  \sa CSL_PLLC_DIVSEL_DEFINE
 */
typedef struct CSL_PllcDivRatio {
    /** \brief Divider number
     *  \param Uint32
     */
    Uint32 divNum;
    /** \brief Divider Ratio
     *  \param Uint32
     */
    Uint32 divRatio;
} CSL_PllcDivRatio;
/**
@} */

/**
\addtogroup CSL_PLLC_DATASTRUCT
@{ */
/**
 *  \brief Input parameters for enabling or disabling PLL Divide ratio
 *  \sa CSL_PLLC_DIVSEL_DEFINE
 */
typedef struct CSL_PllcDivideControl {
    /** \brief Divider Number
     *  \param Uint32
     */
    Uint32 divNum;
    /** \brief Divider Control (Enable/Disable)
     *  \param CSL_PllcDivCtrl
     */
    CSL_PllcDivCtrl divCtrl;
} CSL_PllcDivideControl;
/**
@} */

/**
\addtogroup CSL_PLLC_DATASTRUCT
@{ */
/**
 *  \brief Input parameters for setting up PLL Controller
 *
 *  Used to put PLLC known useful state
 *  /sa CSL_PLLC_DIVEN_DEFINE
 */
typedef struct CSL_PllcHwSetup {
    /** \brief Divider Enable/Disable
     *  \param CSL_BitMask32
     */
    CSL_BitMask32 divEnable;
    /** \brief PLL Multiplier.This is valid only for PLLC instance 1 
     *  \param Uint32
     */
    Uint32        pllM;
    /** \brief Post-Divider
     *  \param Uint32
     */
    Uint32        pllDiv1;
    /** \brief PLL Divider 2.This is valid only for PLLC instance 2 
     *  \param Uint32
     */
    Uint32        pllDiv2;
    /** \brief PLL Divider 3.This is valid only for PLLC instance 2 
     *  \param Uint32
     */
    Uint32        pllDiv3;
    /** \brief PLL Divider 4.This is valid only for PLLC instance 2 
     *  \param Uint32
     */
    Uint32        pllDiv4;
    /** \brief PLL Divider 5.This is valid only for PLLC instance 2 
     *  \param Uint32
     */
    Uint32        pllDiv5;
    /** \brief PLL Divider 6.This is valid only for PLLC instance 2 
     *  \param Uint32
     */
    Uint32        pllDiv6;
    /** \brief PLL Divider 7.This is valid only for PLLC instance 1 
     *  \param Uint32
     */
    Uint32        pllDiv7;
    /** \brief PLL Divider 8.This is valid only for PLLC instance 1 
     *  \param Uint32
     */
    Uint32        pllDiv8;
    /** \brief PLL Divider 9.This is valid only for PLLC instance 1 
     *  \param Uint32
     */
    Uint32        pllDiv9;
    /** \brief PLL Divider 10.This is valid only for PLLC instance 1 
     *  \param Uint32
     */
    Uint32        pllDiv10;
    /** \brief Phase Align Control
     *  \param CSL_BitMask32
     */
    CSL_BitMask32 phaseAlign;
    /** \brief Setup that can be used for future implementation
     *  \param void*
     */
    void*         extendSetup;
} CSL_PllcHwSetup;
/**
@} */

/** \defgroup CSL_PLLC_DEFAULT_HWSETUP_DEFINE PLLC Default HwSetup Structure
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */
/** \brief Default hardware setup parameters
 */
#define CSL_PLLC_HWSETUP_DEFAULTS{  \
    0,                              \
    0,                              \
    0,                              \
    0,                              \
    0,                              \
    0,                              \
    0,                              \
    0,                              \
    0,                              \
    0,                              \
    NULL                            \
}
/**
@} */

/**
\addtogroup CSL_PLLC_DATASTRUCT
@{ */
/** \brief Config-structure
 *
 *  Used to configure the pllc using CSL_pllcHwSetupRaw()
 */
typedef struct {
    /** PLL Multiplier Control register */
    volatile Uint32 PLLM;

    /** PLL Controller Divider 1 register. 
        This should be configured only for instance 2 */
    volatile Uint32 PLLDIV1;

    /** PLL Controller Divider 2 register 
        This should be configured only for instance 2 */
    volatile Uint32 PLLDIV2;

    /** PLL Controller Divider 3 register 
        This should be configured only for instance 2*/
    volatile Uint32 PLLDIV3;

    /** PLL Controller Command register */
    volatile Uint32 PLLCMD;

    /** PLL Controller Clock Align Control register */
    volatile Uint32 ALNCTL;

    /** PLL Controller Divider 4 register 
        This should be configured only for instance 2*/
    volatile Uint32 PLLDIV4;

    /** PLL Controller Divider 5 register 
        This should be configured only for instance 2*/
    volatile Uint32 PLLDIV5;

    /** PLL Controller Divider 6 register 
        This should be configured only for instance 2*/
    volatile Uint32 PLLDIV6;

    /** PLL Controller Divider 7 register 
        This should be configured only for instance 1*/
    volatile Uint32 PLLDIV7;

    /** PLL Controller Divider 8 register 
        This should be configured only for instance 1*/
    volatile Uint32 PLLDIV8;

    /** PLL Controller Divider 9 register 
        This should be configured only for instance 1*/
    volatile Uint32 PLLDIV9;

    /** PLL Controller Divider 10 register 
        This should be configured only for instance 1*/
    volatile Uint32 PLLDIV10;

} CSL_PllcConfig;
/**
@} */

/** \defgroup CSL_PLLC_DEFAULT_CONFIG_DEFINE PLLC Default Config Structure
 *  \ingroup CSL_PLLC_DEFINE
 *
 * @{ */
/** \brief Default values for config structure
 */
#define CSL_PLLC_CONFIG_DEFAULTS {    \
    CSL_PLLC_PLLM_RESETVAL,           \
    CSL_PLLC_PLLDIV1_RESETVAL,        \
    CSL_PLLC_PLLDIV2_RESETVAL,        \
    CSL_PLLC_PLLDIV3_RESETVAL,        \
    CSL_PLLC_PLLCMD_RESETVAL,         \
    CSL_PLLC_ALNCTL_RESETVAL,         \
    CSL_PLLC_PLLDIV4_RESETVAL,        \
    CSL_PLLC_PLLDIV5_RESETVAL,        \
    CSL_PLLC_PLLDIV6_RESETVAL,        \
    CSL_PLLC_PLLDIV7_RESETVAL,        \
    CSL_PLLC_PLLDIV8_RESETVAL,        \
    CSL_PLLC_PLLDIV9_RESETVAL,        \
    CSL_PLLC_PLLDIV10_RESETVAL        \
}
/**
@} */

/**
\addtogroup CSL_PLLC_DATASTRUCT
@{ */
/** \brief Module specific context information. Present implementation of pllc
 *         CSL doesn't have any context information.
 */
typedef struct {
    /** Context information of Pllc CSL.
     *  The below declaration is just a place-holder for future implementation.
     */
    Uint16  contextInfo;
} CSL_PllcContext;
/**
@} */

/**
\addtogroup CSL_PLLC_DATASTRUCT
@{ */
/** \brief Module specific parameters. Present implementation of pllc CSL
 *         doesn't have any module specific parameters.
 */
typedef struct {
    /** Bit mask to be used for module specific parameters. The below
     *  declaration is just a place-holder for future implementation.
     */
    CSL_BitMask16   flags;
} CSL_PllcParam;
/**
@} */

/**
\addtogroup CSL_PLLC_DATASTRUCT
@{ */
/** \brief This structure contains the base-address information for the
 *         peripheral instance of the PLLC
 */
typedef struct {
    /** Base-address of the configuration registers of the peripheral
     */
    CSL_PllcRegsOvly  regs;
} CSL_PllcBaseAddress;
/**
@} */

/**
\addtogroup CSL_PLLC_DATASTRUCT
@{ */
/** \brief This object contains the reference to the instance of PLLC
 *         opened using the @a CSL_pllcOpen()
 *
 *  The pointer to this is passed to all PLLC CSL APIs
 *  This structure has the fields required to configure PLLC for any test
 *  case/application. It should be initialized as per requirements of a
 *  test case/application and passed on to the setup function
 */
typedef struct CSL_PllcObj {
    /** This is a pointer to the registers of the instance of PLLC
     *   referred to by this object
     */
    CSL_PllcRegsOvly regs;
    /** This is the instance of PLLC being referred to by this object */
    CSL_InstNum  pllcNum;
} CSL_PllcObj;
/**
@} */

/**
\addtogroup CSL_PLLC_DATASTRUCT
@{ */
/** \brief This data type is used to return the handle to the pllc functions */
typedef struct CSL_PllcObj *CSL_PllcHandle;
/**
@} */

/*******************************************************************************
 * PLLC function declarations
 ******************************************************************************/

/**
\defgroup CSL_PLLC_INIT_API PLLC Init API
\ingroup CSL_PLLC_FUNCTION
@{*/
/** ============================================================================
 *   @n@b CSL_pllcInit
 *
 *   @b Description
 *   @n This is the initialization function for the pllc CSL. The function
 *      must be called before calling any other API from this CSL. This
 *      function is idem-potent. Currently, the function just return status
 *      CSL_SOK, without doing anything.
 *
 *   @b Arguments
 *   @verbatim
        pContext    Pointer to module-context. As pllc doesn't have
                    any context based information user is expected to pass NULL.
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                        CSL_SOK - Always returns
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n None
 *
 *   @b Example
 *   @verbatim
            CSL_pllcInit(NULL);
     @endverbatim
 * =============================================================================
 */
CSL_Status CSL_pllcInit (
    CSL_PllcContext       *pContext
);
/**
@} */

/**
\defgroup CSL_PLLC_OPEN_API PLLC Open API
\ingroup CSL_PLLC_FUNCTION
@{*/
/** ============================================================================
 *   @n@b CSL_pllcOpen
 *
 *   @b Description
 *   @n This function returns the handle to the PLLC
 *      instance. This handle is passed to all other CSL APIs.
 *
 *   @b Arguments
 *   @verbatim
            pllcObj     Pointer to pllc object.

            pllcNum     Instance of pllc CSL to be opened.
                        There is only one instance of the pllc available.
                        So, the value for this parameter will be
                        CSL_PLLC always.

            pPllcParam  Module specific parameters.

            status      Status of the function call
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_PllcHandle
 *   @n                     Valid pllc handle will be returned if
 *                          status value is equal to CSL_SOK.
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n   1.    The status is returned in the status variable. If status
 *              returned is
 *   @li            CSL_SOK            - Valid pllc handle is returned
 *   @li            CSL_ESYS_FAIL      - The pllc instance is invalid
 *   @li            CSL_ESYS_INVPARAMS - Invalid parameter
 *   @n   2.    PLLC object structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *   @n    2. PLLC object structure
 *
 *   @b Example
 *   @verbatim
            CSL_status           status;
            CSL_PllcObj          pllcObj;
            CSL_PllcHandle       hPllc;
            ...

            hPllc = CSL_pllcOpen(&pllcObj, CSL_PLLC_0, NULL, &status);
            ...

    @endverbatim
 * =============================================================================
 */
CSL_PllcHandle CSL_pllcOpen (
    CSL_PllcObj               *pllcObj,
    CSL_InstNum                pllcNum,
    CSL_PllcParam             *pPllcParam,
    CSL_Status                *status
);
/**
@} */

/**
\defgroup CSL_PLLC_GETBASEADDRESS_API PLLC GetBaseAddress API
\ingroup CSL_PLLC_FUNCTION
@{*/
/** ============================================================================
 *   @n@b CSL_pllcGetBaseAddress
 *
 *   @b Description
 *   @n  Function to get the base address of the peripheral instance.
 *       This function is used for getting the base address of the peripheral
 *       instance. This function will be called inside the CSL_pllcOpen()
 *       function call. This function is open for re-implementing if the user
 *       wants to modify the base address of the peripheral object to point to
 *       a different location and there by allow CSL initiated write/reads into
 *       peripheral. MMR's go to an alternate location.
 *
 *   @b Arguments
 *   @verbatim
            pllcNum         Specifies the instance of the pllc to be opened.

            pPllcParam      Module specific parameters.

            pBaseAddress    Pointer to base address structure containing base
                            address details.
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_OK             - Open call is successful
 *   @li                    CSL_ESYS_FAIL      - The instance number is invalid.
 *   @li                    CSL_ESYS_INVPARAMS - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Base address structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *   @n    2. Base address structure is modified.
 *
 *   @b Example
 *   @verbatim
         CSL_Status              status;
         CSL_PllcBaseAddress  baseAddress;

         ...

         status = CSL_pllcGetBaseAddress(CSL_PLLC_0, NULL,
                  &baseAddress);
         ...
    @endverbatim
 * =============================================================================
 */
CSL_Status CSL_pllcGetBaseAddress (
    CSL_InstNum                      pllcNum,
    CSL_PllcParam                   *pPllcParam,
    CSL_PllcBaseAddress             *pBaseAddress
);
/**
@} */

/**
\defgroup CSL_PLLC_CLOSE_API PLLC Close API
\ingroup CSL_PLLC_FUNCTION
@{*/
/** ============================================================================
 *   @n@b CSL_pllcClose
 *
 *   @b Description
 *   @n This function closes the specified instance of PLLC.
 *
 *   @b Arguments
 *   @verbatim
            hPllc            Handle to the pllc
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Close successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *
 *   @b Example
 *   @verbatim
            CSL_PllcHandle   hPllc;
            CSL_status       status;
            ...

            status = CSL_pllcClose(hPllc, &hwSetup);
     @endverbatim
 * =============================================================================
 */
CSL_Status CSL_pllcClose (
    CSL_PllcHandle         hPllc
);
/**
@} */

/**
\defgroup CSL_PLLC_SETUP_API PLLC Setup API
\ingroup CSL_PLLC_FUNCTION
@{*/
/** ============================================================================
 *   @n@b CSL_pllcHwSetup
 *
 *   @b Description
 *   @n It configures the pllc registers as per the values passed
 *      in the hardware setup structure.
 *
 *   @b Arguments
 *   @verbatim
            hPllc            Handle to the pllc

            hwSetup          Pointer to harware setup structure
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Hardware setup successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li                    CSL_ESYS_INVPARAMS  - Hardware structure is not
                                                  properly initialized
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  PLLC registers are configured according
 *       to the hardware setup parameters
 *
 *   @b Modifies
 *   @n PLLC registers
 *
 *   @b Example
 *   @verbatim
            CSL_PllcHandle   hPllc;
            CSL_PllcObj      pllcObj;
            CSL_PllcHwSetup  hwSetup;
            CSL_status       status;
            ...

            hPllc = CSL_pllcOpen(&pllcObj, CSL_PLLC_0, NULL, &status);
            ...

            status = CSL_pllcHwSetup(hPllc, &hwSetup);
     @endverbatim
 * =============================================================================
 */
CSL_Status CSL_pllcHwSetup (
    CSL_PllcHandle            hPllc,
    CSL_PllcHwSetup          *hwSetup
);
/**
@} */

/**
\defgroup CSL_PLLC_CONTROL_API PLLC Control API
\ingroup CSL_PLLC_FUNCTION
@{*/
/** ============================================================================
 *   @n@b CSL_pllcHwControl
 *
 *   @b Description
 *   @n Takes a command of PLLC with an optional argument & implements it.
 *
 *   @b Arguments
 *   @verbatim
            hPllc           Handle to the PLLC instance

            cmd             The command to this API indicates the action to be
                            taken on PLLC.

            arg             An optional argument.
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li            CSL_SOK               - Status info return successful.
 *   @li            CSL_ESYS_BADHANDLE    - Invalid handle
 *   @li            CSL_ESYS_INVCMD       - Invalid command
 *   @li            CSL_ESYS_INVPARAMS    - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n The hardware registers of PLLC.
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle         hPllc;
        CSL_PllcHwControlCmd   cmd;
        void                      arg;
        ...

        status = CSL_pllcHwControl (hPllc, cmd, &arg);
     @endverbatim
 * =============================================================================
 */
CSL_Status CSL_pllcHwControl (
    CSL_PllcHandle              hPllc,
    CSL_PllcHwControlCmd        cmd,
    void                       *cmdArg
);
/**
@} */

/**
\defgroup CSL_PLLC_QUERY_API PLLC Query API
\ingroup CSL_PLLC_FUNCTION
@{*/
/** ============================================================================
 *   @n@b CSL_pllcGetHwStatus
 *
 *   @b Description
 *   @n Gets the status of the different operations of PLLC.
 *
 *   @b Arguments
 *   @verbatim
            hPllc         Handle to the PLLC instance

            query         The query to this API of PLLC which indicates the
                          status to be returned.

            response      Placeholder to return the status.
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Status info return successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li                    CSL_ESYS_INVQUERY   - Invalid query command
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n None
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle          hPllc;
        CSL_PllcHwStatusQuery   query;
        void                    reponse;
        ...

        status = CSL_pllcGetHwStatus (hPllc, query, &response);
     @endverbatim
 * =============================================================================
 */
CSL_Status CSL_pllcGetHwStatus (
    CSL_PllcHandle                hPllc,
    CSL_PllcHwStatusQuery         query,
    void                         *response
);
/**
@} */

/**
\defgroup CSL_PLLC_HWSETUPRAW_API PLLC HwSetupRaw API
\ingroup CSL_PLLC_FUNCTION
@{*/
/** ============================================================================
 *   @n@b CSL_pllcHwSetupRaw
 *
 *   @b Description
 *   @n This function initializes the device registers with the register-values
 *      provided through the Config Data structure.
 *
 *   @b Arguments
 *   @verbatim
            hPllc        Handle to the PLLC instance

            config       Pointer to config structure
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li            CSL_SOK               - Configuration successful
 *   @li            CSL_ESYS_BADHANDLE    - Invalid handle
 *   @li            CSL_ESYS_INVPARAMS    - Configuration is not
                                            properly initialized
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  The registers of the specified PLLC instance will be setup
 *       according to input configuration structure values.
 *
 *   @b Modifies
 *   @n Hardware registers of the specified PLLC instance.
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle       hPllc;
        CSL_PllcConfig       config = CSL_PLLC_CONFIG_DEFAULTS;
        CSL_Status           status;
        ...

        status = CSL_pllcHwSetupRaw (hPllc, &config);
     @endverbatim
 * =============================================================================
 */
CSL_Status CSL_pllcHwSetupRaw (
    CSL_PllcHandle               hPllc,
    CSL_PllcConfig              *config
);
/**
@} */

/**
\defgroup CSL_PLLC_GETSETUP_API PLLC GetSetup API
\ingroup CSL_PLLC_FUNCTION
@{*/
/** ============================================================================
 *   @n@b CSL_pllcGetHwSetup
 *
 *   @b Description
 *   @n It retrives the hardware setup parameters of the pllc
 *      specified by the given handle.
 *
 *   @b Arguments
 *   @verbatim
            hPllc        Handle to the pllc

            hwSetup      Pointer to the harware setup structure
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Retrieving the hardware setup
 *                                                parameters is successful
 *   @li                    CSL_ESYS_BADHANDLE  - The handle is passed is
 *                                                invalid
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  The hardware setup structure is populated with the hardware setup
 *       parameters
 *
 *   @b Modifies
 *   @n hwSetup variable
 *
 *   @b Example
 *   @verbatim
            CSL_PllcHandle   hPllc;
            CSL_PllcHwSetup  hwSetup;
            ...

            status = CSL_pllcGetHwSetup(hPllc, &hwSetup);
     @endverbatim
 * =============================================================================
 */
CSL_Status CSL_pllcGetHwSetup (
    CSL_PllcHandle               hPllc,
    CSL_PllcHwSetup             *hwSetup
);
/**
@} */

#ifdef __cplusplus
}
#endif

#endif /* _CSL_PLLC_H_ */

