/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** @file  csl_pllcAux.h
 *
 *  @brief API Auxilary header file for PLLC CSL.
 *
 *  Path: \\(CSLPATH)\inc
 */

/* =============================================================================
 *  Revision History
 *  ===============
 *  10-Feb-2004 kpn  File Created.
 *  16-May-2006 RR   updated for Tomahawk
 * =============================================================================
 */

#ifndef _CSL_PPLCAUX_H_
#define _CSL_PLLCAUX_H_

#include <csl_pllc.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 *  Status query functions of the pllc
 */

/** ============================================================================
 *   @n@b CSL_pllcGetPID
 *
 *   @b Description
 *   @n Gets the Peripheral ID of the pllc.
 *
 *   @b Arguments
 *   @verbatim
            hPllc           Handle to the PLLC instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  Uint32
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n None
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle    hPllc;
        Uint32            response;
        ...

        response = CSL_pllcGetPID (hPllc);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
Uint32 CSL_pllcGetPID (
    CSL_PllcHandle      hPllc
)
{
    Uint32 response = 0;
    response = (Uint32) hPllc->regs->PID;
    return response;
}

/** ============================================================================
 *   @n@b CSL_pllcGetStatus
 *
 *   @b Description
 *   @n Gets the Status of the pll controller.
 *
 *   @b Arguments
 *   @verbatim
            hPllc           Handle to the PLLC instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  CSL_BitMask32
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n None
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle    hPllc;
        CSL_BitMask32     response;
        ...

        response = CSL_pllcGetStatus (hPllc);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
CSL_BitMask32 CSL_pllcGetStatus (
    CSL_PllcHandle                hPllc
)
{
    CSL_BitMask32 response;
    response = (CSL_BitMask32) hPllc->regs->PLLSTAT;
    return response;
}

/** ============================================================================
 *   @n@b CSL_pllcGetDivRatioChange
 *
 *   @b Description
 *   @n Gets the Divider Ratio Change Status of the pllc SYSCLK.
 *
 *   @b Arguments
 *   @verbatim
            hPllc           Handle to the PLLC instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  CSL_BitMask32
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n None
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle    hPllc;
        CSL_BitMask32     response;
        ...

        response = CSL_pllcGetDivRatioChange (hPllc);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
CSL_BitMask32 CSL_pllcGetDivRatioChange (
    CSL_PllcHandle                        hPllc
)
{
    CSL_BitMask32 response;
    response = (CSL_BitMask32) hPllc->regs->DCHANGE;
    return response;
}


/** ============================================================================
 *   @n@b CSL_pllcGetSysClkStatus
 *
 *   @b Description
 *   @n Gets the System Clock ON/OFF Status of the pllc.
 *
 *   @b Arguments
 *   @verbatim
            hPllc           Handle to the PLLC instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  CSL_BitMask32
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n None
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle    hPllc;
        CSL_BitMask32     response;
        ...

        response = CSL_pllcGetSysClkStatus (hPllc);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
CSL_BitMask32 CSL_pllcGetSysClkStatus (
    CSL_PllcHandle                      hPllc
)
{
    CSL_BitMask32 response;
    response = (CSL_BitMask32) hPllc->regs->SYSTAT;
    return response;
}

/** ============================================================================
 *   @n@b CSL_pllcGetResetStatus
 *
 *   @b Description
 *   @n Gets the Reset Type Status of the pllc.
 *
 *   @b Arguments
 *   @verbatim
            hPllc           Handle to the PLLC instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  CSL_BitMask32
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n None
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle    hPllc;
        CSL_BitMask32     response;
        ...

        response = CSL_pllcGetResetStatus (hPllc);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
CSL_BitMask32 CSL_pllcGetResetStatus (
    CSL_PllcHandle                     hPllc
)
{
    CSL_BitMask32 response;
    response = (Uint32) hPllc->regs->RSTYPE;
    return response;
}

/** ============================================================================
 *   @n@b CSL_pllcGetFusefarmErrorStatus
 *
 *   @b Description
 *   @n Gets the Fusefarm Error Status of the pllc.
 *
 *   @b Arguments
 *   @verbatim
            hPllc           Handle to the PLLC instance
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  Uint32
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n None
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle    hPllc;
        Uint32            response;
        ...

        response = CSL_pllcGetFusefarmErrorStatus (hPllc);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
Uint32 CSL_pllcGetFusefarmErrorStatus (
    CSL_PllcHandle                      hPllc
)
{
    Uint32 response = 0;
    response = (Uint32) hPllc->regs->FUSERR;
    return response;
}

/*
 *  Control command functions of the pllc
 */

/** ============================================================================
 *   @n@b CSL_pllcCommandCtrl
 *
 *   @b Description
 *      Controls the pllc operation.
 *
 *   @b Arguments
 *   @verbatim
        hPllc        Handle to the PLLC instance
        loadVal      Value to be loaded to pllc PLLCMD register
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  pllc operation is controlled according to value set.
 *
 *   @b Modifies
 *   @n pllc PLLCMD register.
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle      hPllc;
        CSL_BitMask32       loadVal;
        ...

        CSL_pllcCommandCtrl (hPllc, loadVal);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
void CSL_pllcCommandCtrl (
    CSL_PllcHandle         hPllc,
    CSL_BitMask32          loadVal
)
{
    hPllc->regs->PLLCTL = (loadVal & 0xFFFF);
    loadVal = (loadVal & 0xFFFF0000)>>16;
    if (loadVal)
        hPllc->regs->PLLCMD = loadVal;
}


/** ============================================================================
 *   @n@b CSL_pllcSetPhaseAlignCtrl
 *
 *   @b Description
 *      Controls Phase Alignment of System clocks.
 *
 *   @b Arguments
 *   @verbatim
        hPllc        Handle to the PLLC instance
        loadVal      Value to be loaded to pllc ALNCTL register
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Phase alignment of system clocks is controlled.
 *
 *   @b Modifies
 *   @n pllc ALNCTL register.
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle      hPllc;
        CSL_BitMask32       loadVal;
        ...

        CSL_pllcSetPhaseAlignCtrl (hPllc, loadVal);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
void CSL_pllcSetPhaseAlignCtrl (
    CSL_PllcHandle               hPllc,
    CSL_BitMask32                loadVal
)
{
    while (CSL_FEXT (hPllc->regs->PLLSTAT, PLLC_PLLSTAT_GOSTAT)) {
    /* GO operation is in progress */
    }
    hPllc->regs->ALNCTL = loadVal;
}

/** ============================================================================
 *   @n@b CSL_pllcMultiplierCtrl
 *
 *   @b Description
 *      Controls the pllc Multiplier.
 *
 *   @b Arguments
 *   @verbatim
        hPllc        Handle to the PLLC instance
        loadVal      Value to be loaded to pllc PLLM register
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  pllc multiplier is controlled accordingly.
 *
 *   @b Modifies
 *   @n pllc PLLM register.
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle      hPllc;
        Uint32              loadVal;
        ...

        CSL_pllcMultiplierCtrl (hPllc, loadVal);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
void CSL_pllcMultiplierCtrl (
    CSL_PllcHandle            hPllc,
    Uint32                    loadVal
)
{
    CSL_FINS (hPllc->regs->PLLM, PLLC_PLLM_PLLM, loadVal);
}

/** ============================================================================
 *   @n@b CSL_pllcSetPLLDivRatio
 *
 *   @b Description
 *      Sets the pllc Dividers ratios.
 *
 *   @b Arguments
 *   @verbatim
        hPllc        Handle to the PLLC instance
        loadVal      Value to be loaded to pllc divider registers
        status       Status variable
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  pllc dividers ratios are set.
 *
 *   @b Modifies
 *   @n pllc divider registers.
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle      hPllc;
        CSL_PllcDivRatio    loadVal;
        CSL_Status          status;
        ...

        CSL_pllcSetPLLDivRatio (hPllc, loadVal, &status);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
void CSL_pllcSetPLLDivRatio (
    CSL_PllcHandle             hPllc,
    CSL_PllcDivRatio           loadVal,
    CSL_Status                *status
)
{
    switch (loadVal.divNum) {
        case CSL_PLLC_DIVSEL_PLLDIV1:
            CSL_FINS (hPllc->regs->PLLDIV1, PLLC_PLLDIV1_RATIO,
                      (Uint32)loadVal.divRatio);
            break;
        case CSL_PLLC_DIVSEL_PLLDIV2:
            CSL_FINS (hPllc->regs->PLLDIV2, PLLC_PLLDIV2_RATIO,
                      (Uint32)loadVal.divRatio);
            break;
        case CSL_PLLC_DIVSEL_PLLDIV3:
            CSL_FINS (hPllc->regs->PLLDIV3, PLLC_PLLDIV3_RATIO,
                      (Uint32)loadVal.divRatio);
            break;
        case CSL_PLLC_DIVSEL_PLLDIV4:
            CSL_FINS (hPllc->regs->PLLDIV4, PLLC_PLLDIV4_RATIO,
                      (Uint32)loadVal.divRatio);
            break;
        case CSL_PLLC_DIVSEL_PLLDIV5:
            CSL_FINS (hPllc->regs->PLLDIV5, PLLC_PLLDIV5_RATIO,
                      (Uint32)loadVal.divRatio);
            break;
        case CSL_PLLC_DIVSEL_PLLDIV6:
            CSL_FINS (hPllc->regs->PLLDIV6, PLLC_PLLDIV6_RATIO,
                      (Uint32)loadVal.divRatio);
            break;
        case CSL_PLLC_DIVSEL_PLLDIV10:
            CSL_FINS (hPllc->regs->PLLDIV10, PLLC_PLLDIV10_RATIO,
                      (Uint32)loadVal.divRatio);
            break;
            
        default:
            *status = CSL_ESYS_INVPARAMS;
            break;
    }
}

/** ============================================================================
 *   @n@b CSL_pllcPLLDivCtrl
 *
 *   @b Description
 *      Controls the pllc dividers.
 *
 *   @b Arguments
 *   @verbatim
        hPllc        Handle to the PLLC instance
        loadVal      Value to be loaded to pllc dividers register.
        status       Status variable
     @endverbatim
 *
 *   <b> Return Value </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  pllc dividers are controlled.
 *
 *   @b Modifies
 *   @n pllc dividers register.
 *
 *   @b Example
 *   @verbatim
        CSL_PllcHandle              hPllc;
        CSL_PllcDivideControl       loadVal;
        CSL_Status                  status;
        ...

        CSL_pllcPLLDivCtrl (hPllc, loadVal, &status);
     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
void CSL_pllcPLLDivCtrl (
    CSL_PllcHandle         hPllc,
    CSL_PllcDivideControl  loadVal,
    CSL_Status            *status
)
{
    switch (loadVal.divNum) {
        case CSL_PLLC_DIVSEL_PLLDIV1:
            CSL_FINS (hPllc->regs->PLLDIV1, PLLC_PLLDIV1_D1EN,
                      (CSL_PllcDivCtrl)loadVal.divCtrl);
            break;

        case CSL_PLLC_DIVSEL_PLLDIV2:
            CSL_FINS (hPllc->regs->PLLDIV2, PLLC_PLLDIV2_D2EN,
                      (CSL_PllcDivCtrl)loadVal.divCtrl);
            break;

        case CSL_PLLC_DIVSEL_PLLDIV3:
            CSL_FINS (hPllc->regs->PLLDIV3, PLLC_PLLDIV3_D3EN,
                      (CSL_PllcDivCtrl)loadVal.divCtrl);
            break;

        case CSL_PLLC_DIVSEL_PLLDIV4:
            CSL_FINS (hPllc->regs->PLLDIV4, PLLC_PLLDIV4_D4EN,
                      (CSL_PllcDivCtrl)loadVal.divCtrl);
            break;

        case CSL_PLLC_DIVSEL_PLLDIV5:
            CSL_FINS (hPllc->regs->PLLDIV5, PLLC_PLLDIV5_D5EN,
                      (CSL_PllcDivCtrl)loadVal.divCtrl);
            break;

        case CSL_PLLC_DIVSEL_PLLDIV6:
            CSL_FINS (hPllc->regs->PLLDIV6, PLLC_PLLDIV6_D6EN,
                      (CSL_PllcDivCtrl)loadVal.divCtrl);
            break;

        case CSL_PLLC_DIVSEL_PLLDIV10:
            CSL_FINS (hPllc->regs->PLLDIV10, PLLC_PLLDIV10_D10EN,
                      (CSL_PllcDivCtrl)loadVal.divCtrl);
            break;

        default:
            *status = CSL_ESYS_INVPARAMS;
            break;
    }
}

#ifdef __cplusplus
}
#endif

#endif /* _CSL_PLLCAUX_H_ */

