/* ============================================================================
 * Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 *
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/** ===========================================================================
 * @file csl_psc.h
 *
 * @brief Header file for functional layer of CSL
 *
 * @Path $(CSLPATH)\inc
 *
 * @desc The different enumerations, structure definitions
 *       and function declarations
 * ============================================================================
 * @defgroup CSL_PSC_API PSC
 *
 * @section Introduction
 *
 * @subsection xxx Overview
 * This page describes the Functions, Data Structures, Enumerations and Macros
 * within PSC module. The PSC manages clocks to the various modules. The PSC
 * allows software to gracefully gate clocks to any unused modules and to reset
 * these modules during device operation.
 * The PSC consists of one global power/sleep controller (GPSC) and many local,
 * permodule, power/sleep controllers (LPSCs). Each modules reset and clocks
 * are controlled by the LPSC.
 * 
 * Tomahawk comprises several power domains to enable minimizing power
 * dissipation for unused logic on the device. The GPSC manages each of the
 * power domains. Additionally, clock gating to each of the logic blocks is
 * manages by the LPSCs of each module
 *
 *
 * ============================================================================
 */

/* ============================================================================
 * Revision History
 * ===============
 * 28-June-2006 TRR File Created.
 *
 *  03-April-2008   Update for Doxygen
 * ============================================================================
 */

#ifndef _CSL_PSC_H_
#define _CSL_PSC_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <csl.h>
#include <cslr_psc.h>

/**
@defgroup CSL_PSC_DATASTRUCT  PSC Data Structures
@ingroup CSL_PSC_API
*/
/**
@defgroup CSL_PSC_FUNCTION  PSC Functions
@ingroup CSL_PSC_API
*/
/**
@defgroup CSL_PSC_ENUM  PSC Enumerated Data Types
@ingroup CSL_PSC_API
*/
/**
 * General purpose global typedef declarations
 */

/** @addtogroup CSL_PSC_ENUM
 @{ */
/** @brief PSC module Enumeration
 * This provides the name of peripherals controlled by the PSC module.
 * 
 */
typedef enum {
    /** GEM_0 module*/
    CSL_PSC_MODULE_GEM0        =   0,
    /** GEM_1 module*/
    CSL_PSC_MODULE_GEM1        =   1,
    /** GEM_2 module*/
    CSL_PSC_MODULE_GEM2        =   2,
    /** GEM_3 module*/
    CSL_PSC_MODULE_GEM3        =   3,
    /** GEM_4 module*/
    CSL_PSC_MODULE_GEM4        =   4,
    /** GEM_5 module*/
    CSL_PSC_MODULE_GEM5        =   5,
    /** SRIO module */
    CSL_PSC_MODULE_SRIO        =   6,
    /** EMAC0 module */
    CSL_PSC_MODULE_EMAC0       =   7,
    /** EMAC1 module */
    CSL_PSC_MODULE_EMAC1       =   8,
    /** TSIP0 module */
    CSL_PSC_MODULE_TSIP0       =   9,
    /** TSIP1 module */
    CSL_PSC_MODULE_TSIP1       =   10,
    /** TSIP2 module */
    CSL_PSC_MODULE_TSIP2       =   11,
    /** HPI module */
    CSL_PSC_MODULE_HPI         =   12,
    /** UTOPIA module */
    CSL_PSC_MODULE_UTOPIA      =   13
    
} CSL_PscPeripherals;

/** @brief PSC Power domain Enumeration
 * This provides the Power Domains on the SOC.
 * 
 */
typedef enum {
    /** Always on power domain*/
    CSL_PSC_PWRDMN_ALWAYSON        =   0,
    /** Proxy A power domain*/
    CSL_PSC_PWRDMN_PROXYA          =   1,
    /** Proxy B power domain*/
    CSL_PSC_PWRDMN_PROXYB          =   2,
    /** Proxy C power domain*/
    CSL_PSC_PWRDMN_PROXYC          =   3,
    /** Proxy D power domain*/
    CSL_PSC_PWRDMN_PROXYD          =   4,
    /** Proxy E power domain*/
    CSL_PSC_PWRDMN_PROXYE          =   5,
    /** Proxy F power domain*/
    CSL_PSC_PWRDMN_PROXYF          =   6,
    /** Proxy G power domain*/
    CSL_PSC_PWRDMN_PROXYG          =   7,
    /** Proxy H power domain*/
    CSL_PSC_PWRDMN_PROXYH          =   8

}CSL_PscPowerDomain;



/** Enumeration to describe the state of the module */
typedef enum {
    
    /** module software reset disable */
    CSL_PSC_MODULE_SWRST_DISABLE   = 0,
    /** Module sync reset */
    CSL_PSC_MODULE_SYNCRST         = 1,
    /** Module disable */
    CSL_PSC_MODULE_DISABLE         = 2,
    /** Module enable */
    CSL_PSC_MODULE_ENABLE          = 3
    
} CSL_PscPeriState;
/**
@}
*/

/** @addtogroup CSL_PSC_DATASTRUCT
 @{ */
/** @brief PSC Power Domain status structure 
 * This provides the status of the specified Power Domain.
 * 
 */
typedef struct {
    
    /** Power Domain to be selected */
    CSL_PscPowerDomain  pwrDmn;
    /** Status of the Power Domain*/
    Uint32              state;
    
}CSL_PscPwrdmnState;

/** @brief PSC module status structure 
 * This provides the status of the specified module.
 * 
 */
typedef struct {
    
    /** Module to be selected */
    CSL_PscPeripherals module;
    /** Status of the module*/
    CSL_PscPeriState   state;
    
}CSL_PscmoduleState;
/**
@}
*/

/** @addtogroup CSL_PSC_ENUM
 @{ */
/**
 *  @brief This enum describes the commands used to control the power/sleep 
 *  controller through CSL_pscHwControl()
 */
typedef enum {
    /**
     * @brief   Enable clock for the specified module
     * @param   CSL_PscPeripherals *
     */
    CSL_PSC_CMD_ENABLE_MODULE = 0,

    /**
     * @brief   Disable clock for the specified module
     * @param   CSL_PscPeripherals *
     */
    CSL_PSC_CMD_DISABLE_MODULE = 1,

    /**
     * @brief   Enable Power domain-n GO transition
     * @param   CSL_PscPowerDomain *
     */
    CSL_PSC_CMD_PWRDMN_TRNS = 2

} CSL_PscHwControlCmd;


/**
 *  @brief This enum describes the commands used to get status of various 
 *  parameters of the power/sleep controller. These values are used in 
 *  CSL_pscGetHwStatus()
 */
typedef enum {
    /**
     * @brief   Gets the current status of the specified module 
     * @param   CSL_PscmoduleState *
     */
    CSL_PSC_QUERY_MODULE_STAT       = 0,

    /**
     * @brief   Gets the transition status of the selected power domain
     * @param   CSL_PscPowerDomain *
     */
    CSL_PSC_QUERY_PWRDMN_TRANS_STAT  = 1

    
} CSL_PscHwStatusQuery;
/**
@}
*/

/** @addtogroup CSL_PSC_DATASTRUCT
 @{ */
/**
 * @brief Module specific context information. Present implementation of 
 *        Power and Sleep Controller CSL doesn't have any context information.
 */
typedef struct {
    /** Context information of Power and Sleep Controller CSL.
     *  The below declaration is just a place-holder for future implementation.
     */
    Uint16 contextInfo;
} CSL_PscContext;

/** @brief Module specific parameters. Present implementation of power/sleep 
 *         controller CSL doesn't have any module specific parameters.
 */
typedef struct {
    /** Bit mask to be used for module specific parameters. The below
     *  declaration is just a place-holder for future implementation.
     */
    CSL_BitMask16 flags;
} CSL_PscParam;

/** @brief This structure contains the base-address information for the
 *         peripheral instance
 */
typedef struct {
    /** Base-address of the configuration registers of the peripheral
     */
    CSL_PscRegsOvly regs;
} CSL_PscBaseAddress;

/**
 * @brief  PSC object structure.
 */
typedef struct {
    /** Pointer to the register overlay structure of the 
      * power/sleep controller 
      */
    CSL_PscRegsOvly regs;

    /** Instance of PSC being referred by this object */
    CSL_InstNum perNum;
} CSL_PscObj;


/**
 * @brief This data type is used to return the handle to the CSL of the 
 *        power/sleep controller
 */
typedef CSL_PscObj *CSL_PscHandle;
/**
@}
*/

/** @addtogroup CSL_PSC_FUNCTION
 @{ */
/*******************************************************************************
 * Power and Sleep Controller global function declarations
 ******************************************************************************/

/** ============================================================================
 *   @n@b CSL_pscInit
 *
 *   @b Description
 *   @n This is the initialization function for the  PSC CSL.
 *      The function must be called before calling any other API from this CSL.
 *      This function is idem-potent. Currently, the function just return
 *      status CSL_SOK, without doing anything.
 *
 *   @b Arguments
 *   @verbatim
        pContext    Pointer to module-context. As PSC doesn't
                    have any context based information user is expected to pass
                    NULL.
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                        CSL_SOK - Always returns
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  The CSL for PSC is initialized
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim
            CSL_pscInit();
     @endverbatim
 * =============================================================================
 */
CSL_Status CSL_pscInit (
    CSL_PscContext *pContext
);



/** ============================================================================
 *   @n@b CSL_pscOpen
 *
 *   @b Description
 *   @n This function populates the peripheral data object for the PSC instance
 *      and returns a handle to the instance.
 *      The open call sets up the data structures for the particular instance
 *      of PSC device. The device can be re-opened anytime after it has been
 *      normally closed if so required. The handle returned by this call is
 *      input as an essential argument for rest of the APIs described
 *      for this module.
 *
 *   @b Arguments
 *   @verbatim
            pscObj          Pointer to PSC object.

            pscNum          Instance of PSC CSL to be opened.
                            There are twelve instances of the PSC
                            available. So, the value for this parameter will be
                            based on the instance.

            pPscParam       Module specific parameters.

            status          Status of the function call
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_PscHandle
 *   @n                         Valid PSC handle will be returned if
 *                              status value is equal to CSL_SOK.
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n   1.    The status is returned in the status variable. If status
 *              returned is
 *   @li            CSL_SOK             Valid PSC handle is returned
 *   @li            CSL_ESYS_FAIL       The PSC instance is invalid
 *
 *        2.    Psc object structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *
 *         2. Psc object structure
 *
 *   @b Example
 *   @verbatim
            CSL_status              status;
            CSL_PscObj              pscObj;
            CSL_PscHandle           hPsc;

            hPsc = CSL_pscOpen(&pscObj, CSL_PSC_1, NULL, &status);
            ...
    @endverbatim
 * =============================================================================
 */
CSL_PscHandle CSL_pscOpen (
    CSL_PscObj   *pscObj,
    CSL_InstNum  pscNum,
    CSL_PscParam *pPscParam,
    CSL_Status   *status
);



/** ============================================================================
 *   @n@b CSL_pscGetBaseAddress
 *
 *   @b Description
 *   @n  This function gets the base address of the given PSC
 *       instance.
 *
 *   @b Arguments
 *   @verbatim
            pscNum        Specifies the instance of the PSC to be opened

            pPscParam     Psc module specific parameters

            pBaseAddress  Pointer to base address structure containing base
                          address details
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li					CSL_SOK             Open call is successful
 *   @li					CSL_ESYS_FAIL      PSC instance is not
 *                                             available.
 *   @li                    CSL_ESYS_INVPARAMS Invalid Parameters
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Base address structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *
 *         2. Base address structure is modified.
 *
 *   @b Example
 *   @verbatim
        CSL_Status            status;
        CSL_PscBaseAddress    baseAddress;

        ...
        status = CSL_pscGetBaseAddress(CSL_PSC_1, NULL, &baseAddress);
        ...

    @endverbatim
 * ===========================================================================
 */
CSL_Status CSL_pscGetBaseAddress (
    CSL_InstNum        pscNum,
    CSL_PscParam       *pPscParam,
    CSL_PscBaseAddress *pBaseAddress
);



/** ============================================================================
 *   @n@b CSL_pscClose
 *
 *   @b Description
 *   @n This function marks that CSL for the power and sleep controller instance is closed.
 *      CSL for the power and sleep controller instance need to be reopened before using any
 *      power and sleep controller CSL API.
 *
 *   @b Arguments
 *   @verbatim
            hPsc         Handle to the power and sleep controller instance
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK            - PSC is closed
 *                                               successfully
 *
 *   @li                    CSL_ESYS_BADHANDLE - The handle passed is invalid
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  1. The power/sleep controller CSL APIs can not be called until the PSC
 *          CSL is reopened again using CSL_pscOpen()
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim
            CSL_pscClose(hPsc);
    @endverbatim
 * ===========================================================================
 */
CSL_Status CSL_pscClose (
    CSL_PscHandle hPsc
);

/** ============================================================================
 *   @n@b CSL_pscHwControl
 *
 *   @b Description
 *   @n This function performs various control operations on the PSC instance,
 *      based on the command passed.
 *
 *   @b Arguments
 *   @verbatim
            hPsc         Handle to the PSC instance

            cmd          Operation to be performed on the PSC

            cmdArg       Argument specific to the command

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK            - Command execution successful.
 *   @li                    CSL_ESYS_BADHANDLE - Invalid handle
 *   @li                    CSL_ESYS_INVCMD    - Invalid command
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Registers of the PSC instance are configured according to the command
 *       and the command arguments. The command determines which registers are
 *       modified.
 *
 *   @b Modifies
 *   @n Registers determined by the command
 *
 *   @b Example
 *   @verbatim
        CSL_Status status;

        status  = CSL_pscHwControl(hPsc, CSL_PSC_CMD_MODULE_ENABLE, NULL);

     @endverbatim
 * ============================================================================
 */
CSL_Status CSL_pscHwControl(
    CSL_PscHandle       hPsc,
    CSL_PscHwControlCmd cmd,
    void                *cmdArg
);



/** ===========================================================================
 *   @n@b CSL_pscGetHwStatus
 *
 *   @b Description
 *   @n This function is used to get the value of various parameters of the
 *      PSC instance. The value returned depends on the query passed.
 *
 *   @b Arguments
 *   @verbatim
            hPsc            Handle to the PSC instance

            query           Query to be performed

            response        Pointer to buffer to return the data requested by
                            the query passed
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK            - Successful completion of the
 *                                               query
 *
 *   @li                    CSL_ESYS_BADHANDLE - Invalid handle
 *
 *   @li                    CSL_ESYS_INVQUERY  - Query command not supported
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *       Data requested by the query is returned through the variable "response"
 *
 *   @b Modifies
 *   @n  The input argument "response" is modified
 *
 *   @b Example
 *   @verbatim

        CSL_status    status;
        Uint8         response;

        status = CSL_pscGetHwStatus(hPsc, CSL_PSC_QUERY_PWRDMN_STAT,
                                    &response);
     @endverbatim
 * ===========================================================================
 */
CSL_Status CSL_pscGetHwStatus (
    CSL_PscHandle        hPsc,
    CSL_PscHwStatusQuery query,
    void                 *response
);
/**
@}
*/


#ifdef __cplusplus
}
#endif

#endif  /* _CSL_PSC_H_ */

