/* ============================================================================
 * Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** ===========================================================================
 * @file csl_pscAux.h
 *
 * @brief API Auxilary header file for PSC CSL
 *
 * @Path $(CSLPATH)\inc
 *
 * @desc It gives the definitions of the status query & control functions.
 * ============================================================================
 */

/* ============================================================================
 * Revision History
 * ===============
 * 29-Jun-2006 TRR File Created.
 * ============================================================================
 */

#ifndef _CSL_PSCAUX_H_
#define _CSL_PSCAUX_H_

#include <csl_psc.h>

#ifdef __cplusplus
extern "C" {
#endif

/*  Command Functions of Power and Sleep Controller */

/** ============================================================================
 *   @n@b CSL_PscEnableClock
 *
 *   @b Description
 *   @n This function enables the clock for the selected module
 *
 *   @b Arguments
 *   @verbatim
         hPsc                 Pointer to the object that holds reference to the
                              instance of PSC requested after the call

         CSL_PscPeripherals   pointer to enum
     @endverbatim
 *
 *   <b> Return Value </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None 
 *
 *   <b> Post Condition </b>
 *   @n  Clock is Enabled for the module
 *
 *   @b  Modifies
 *   @n  Module Control Register
 *
 *   @b  Example
 *   @verbatim
        CSL_PscHandle          hPsc;
        CSL_PscPeripherals     *pscModule;
        ...
        CSL_PscEnableClock(hPsc, &pscModule);
        ...
     @endverbatim
 * ============================================================================
 */
static inline
 CSL_Status CSL_PscEnableClock (
    CSL_PscHandle       hPsc,
    CSL_PscPeripherals  *pscModule
)
{
    switch(*pscModule){
    
       	case CSL_PSC_MODULE_GEM0 :
       		CSL_FINS(hPsc->regs->MDCTL0, PSC_MDCTL0_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_GEM1 :
       		CSL_FINS(hPsc->regs->MDCTL1, PSC_MDCTL1_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_GEM2 :
       		CSL_FINS(hPsc->regs->MDCTL2, PSC_MDCTL2_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_GEM3 :
       		CSL_FINS(hPsc->regs->MDCTL3, PSC_MDCTL3_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_GEM4 :
       		CSL_FINS(hPsc->regs->MDCTL4, PSC_MDCTL4_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_GEM5 :
       		CSL_FINS(hPsc->regs->MDCTL5, PSC_MDCTL5_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_SRIO :
       		CSL_FINS(hPsc->regs->MDCTL6, PSC_MDCTL6_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_EMAC0 :
       		CSL_FINS(hPsc->regs->MDCTL7, PSC_MDCTL7_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_EMAC1 :
       		CSL_FINS(hPsc->regs->MDCTL8, PSC_MDCTL8_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_TSIP0 :
       		CSL_FINS(hPsc->regs->MDCTL9, PSC_MDCTL9_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_TSIP1 :
       		CSL_FINS(hPsc->regs->MDCTL10, PSC_MDCTL10_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_TSIP2 :
       		CSL_FINS(hPsc->regs->MDCTL11, PSC_MDCTL11_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_HPI :
       		CSL_FINS(hPsc->regs->MDCTL12, PSC_MDCTL12_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_UTOPIA :
       		CSL_FINS(hPsc->regs->MDCTL13, PSC_MDCTL13_NEXT, 0x3);
       		break;
       	default :
       		return CSL_ESYS_INVPARAMS;
    
  }

  return CSL_SOK;

}



/** ============================================================================
 *   @n@b CSL_PscDisableClock
 *
 *   @b Description
 *   @n This function disables the clock for the selected module
 *
 *   @b Arguments
 *   @verbatim
         hPsc                 Pointer to the object that holds reference to the
                              instance of PSC requested after the call

         CSL_PscPeripherals   pointer to enum
     @endverbatim
 *
 *   <b> Return Value </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None 
 *
 *   <b> Post Condition </b>
 *   @n  Clock is Disabled for the module
 *
 *   @b  Modifies
 *   @n  Module Control Register
 *
 *   @b  Example
 *   @verbatim
        CSL_PscHandle          hPsc;
        CSL_PscPeripherals     *pscModule;
        ...
        CSL_PscDisableClock(hPsc, &pscModule);
        ...
     @endverbatim
 * ============================================================================
 */
static inline
 CSL_Status CSL_PscDisableClock (
    CSL_PscHandle       hPsc,
    CSL_PscPeripherals  *pscModule
)
{
    switch(*pscModule){
    
       	case CSL_PSC_MODULE_GEM0 :
       		CSL_FINS(hPsc->regs->MDCTL0, PSC_MDCTL0_NEXT, 0x3);
       		break;
       	case CSL_PSC_MODULE_GEM1 :
       		CSL_FINS(hPsc->regs->MDCTL1, PSC_MDCTL1_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_GEM2 :
       		CSL_FINS(hPsc->regs->MDCTL2, PSC_MDCTL2_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_GEM3 :
       		CSL_FINS(hPsc->regs->MDCTL3, PSC_MDCTL3_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_GEM4 :
       		CSL_FINS(hPsc->regs->MDCTL4, PSC_MDCTL4_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_GEM5 :
       		CSL_FINS(hPsc->regs->MDCTL5, PSC_MDCTL5_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_SRIO :
       		CSL_FINS(hPsc->regs->MDCTL6, PSC_MDCTL6_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_EMAC0 :
       		CSL_FINS(hPsc->regs->MDCTL7, PSC_MDCTL7_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_EMAC1 :
       		CSL_FINS(hPsc->regs->MDCTL8, PSC_MDCTL8_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_TSIP0 :
       		CSL_FINS(hPsc->regs->MDCTL9, PSC_MDCTL9_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_TSIP1 :
       		CSL_FINS(hPsc->regs->MDCTL10, PSC_MDCTL10_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_TSIP2 :
       		CSL_FINS(hPsc->regs->MDCTL11, PSC_MDCTL11_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_HPI :
       		CSL_FINS(hPsc->regs->MDCTL12, PSC_MDCTL12_NEXT, 0x2);
       		break;
       	case CSL_PSC_MODULE_UTOPIA :
       		CSL_FINS(hPsc->regs->MDCTL13, PSC_MDCTL13_NEXT, 0x2);
       		break;
       	default :
       		return CSL_ESYS_INVPARAMS;
    
  }

  return CSL_SOK;

}

/** ============================================================================
 *   @n@b CSL_PscPwrDomainTrans
 *
 *   @b Description
 *   @n This function enables GO transition bit of the selected power domain
 *   @b Arguments
 *   @verbatim
         hPsc                 Pointer to the object that holds reference to the
                              instance of PSC requested after the call

         CSL_PscRailcount   pointer to the structure
     @endverbatim
 *
 *   <b> Return Value </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None 
 *
 *   <b> Post Condition </b>
 *   @n  Go status bit is et
 *
 *   @b  Modifies
 *   @n  None
 *
 *   @b  Example
 *   @verbatim
        CSL_PscHandle          hPsc;
        CSL_PscPowerDomain     *pdomain;
        ...
        CSL_PscPwrDomainTrans(hPsc, &pdomain);
        ...
     @endverbatim
 * ============================================================================
 */
static inline
void CSL_PscPwrDomainTrans (
    CSL_PscHandle      hPsc,
    CSL_PscPowerDomain *pDomain
)
{
    CSL_FINS(hPsc->regs->PTCMD, PSC_PTCMD_GO, (0x1 << *pDomain) );
}


/** ===========================================================================
 *   @n@b CSL_pscGetModuleStat
 *
 *   @b Description
 *   @n This function gets the status of the module
 *
 *   @b Arguments
 *   @verbatim
         hPsc                Pointer to the object that holds reference to the
                             instance of PSC requested after the call
         CSL_PscmoduleState  ponter to the module status                    
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_PscmoduleState - TINTHI status value
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n Module status value
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim
        CSL_PscHandle      hPsc;
        CSL_PscmoduleState status;
        ...
        CSL_pscGetModuleStat(hPsc,&ststus);
        ...
     @endverbatim
 * ============================================================================
 */
static inline
CSL_Status CSL_pscGetModuleStat (
    CSL_PscHandle       hPsc,
    CSL_PscmoduleState  *pModuleStat
)
{
    
    switch(pModuleStat->module ){
    
       	case CSL_PSC_MODULE_GEM0 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT0, PSC_MDSTAT0_STATE);
       		break;
       	case CSL_PSC_MODULE_GEM1 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT1, PSC_MDSTAT1_STATE);
       		break;
       	case CSL_PSC_MODULE_GEM2 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT2, PSC_MDSTAT2_STATE);
       		break;
       	case CSL_PSC_MODULE_GEM3 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT3, PSC_MDSTAT3_STATE);
       		break;
       	case CSL_PSC_MODULE_GEM4 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT4, PSC_MDSTAT4_STATE);
       		break;
       	case CSL_PSC_MODULE_GEM5 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT5, PSC_MDSTAT5_STATE);
       		break;
       	case CSL_PSC_MODULE_SRIO :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT6, PSC_MDSTAT6_STATE);
       		break;
       	case CSL_PSC_MODULE_EMAC0 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT7, PSC_MDSTAT7_STATE);
       		break;
       	case CSL_PSC_MODULE_EMAC1 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT8, PSC_MDSTAT8_STATE);
       		break;
       	case CSL_PSC_MODULE_TSIP0 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT9, PSC_MDSTAT9_STATE);
       		break;
       	case CSL_PSC_MODULE_TSIP1 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT10, PSC_MDSTAT10_STATE);
       		break;
       	case CSL_PSC_MODULE_TSIP2 :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT11, PSC_MDSTAT11_STATE);
       		break;
       	case CSL_PSC_MODULE_HPI :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT12, PSC_MDSTAT12_STATE);
       		break;
       	case CSL_PSC_MODULE_UTOPIA :
       		pModuleStat->state = (CSL_PscPeriState )CSL_FEXT(\
                                     hPsc->regs->MDSTAT13, PSC_MDSTAT13_STATE);
       		break;
       	default :
       		return CSL_ESYS_INVPARAMS;
    
  }

  return CSL_SOK;
  
}

/** ===========================================================================
 *   @n@b CSL_pscGetPwrDmnTransStat
 *
 *   @b Description
 *   @n This function gets the transition status of the power domain
 *
 *   @b Arguments
 *   @verbatim
         hPsc                Pointer to the object that holds reference to the
                             instance of PSC requested after the call
     @endverbatim
 *
 *   <b> Return Value </b>  Uint32 - Power Domain status 
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *    @n Power domain transition status value is read
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim
        CSL_PscHandle      hPsc;
        Uint32             pstatus;
        ...
        CSL_pscGetPwrDmnTransStat(hPsc,&pststus);
        ...
     @endverbatim
 * ============================================================================
 */
static inline
Uint32 CSL_pscGetPwrDmnTransStat(
    CSL_PscHandle       hPsc 
)
{
    return (Uint32)hPsc->regs->PTSTAT ;
}

#ifdef __cplusplus
}
#endif

#endif /* CSL_PSCAUX_H_ */

