/* =============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ============================================================================
 *   @file  csl_smc.h
 *
 *   @path  $(CSLPATH)\inc
 *
 *   @desc  Header file for functional layer of SMC CSL
 *  - The different symbolic constants, enumerations, structure definitions
 *    and function prototype declarations
 *
 */
/* =============================================================================
 *  Revision History
 *  ===============
 *  23-May-2006 NG  File Created
 *
 * =============================================================================
 */

/** @mainpage SMC CSL 3.x
 *
 * @section Introduction
 *
 * @subsection xxx Purpose and Scope
 * The purpose of this document is to identify a set of common CSL APIs for
 * the SMC module across various devices. The CSL developer is expected to
 * refer to this document while designing APIs for these modules. Some of the
 * listed APIs may not be applicable to a given SMC module. While this list
 * of APIs may not be sufficient to cover all the features of a
 * particular SMC Module. The CSL developer should use his discretion designing
 * new APIs or extending the existing ones to cover these.
 *
 * @subsection aaa Terms and Abbreviations
 *   -# CSL:  Chip Support Library
 *   -# API:  Application Programmer Interface
 *
 * @subsection References
 *    -# CSL-001-DES, CSL 3.x Design Specification DocumentVersion 1.02
 *
 */
/**
 * @defgroup CSL_SMC_API SMC
 */
#ifndef _CSL_SMC_H_
#define _CSL_SMC_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <soc.h>
#include <csl_types.h>
#include <csl_error.h>
#include <cslr_smc.h>

/**************************************************************************\
* SMC global macro declarations
\**************************************************************************/
/** to set/reset page2 */
#define CSL_SMC_POWERDOWN_PAGE2    (0x00000004u)
/** to set/reset page3 */
#define CSL_SMC_POWERDOWN_PAGE3    (0x00000008u)

/** To enable/disable specific pages */
#define CSL_SMC_PREFETECH_PAGE(x)   (1 << x)

/** To enable/disable all the pages */
#define CSL_SMC_PREFETECH_PAGES_ALL   (0xFFFFFFFFu)


/**************************************************************************\
* SMC global typedef declarations
\**************************************************************************/

/**
 * @defgroup CSL_SMC_CONTROLCMD_ENUM Control commands
 * @ingroup CSL_SMC_CONTROL_API
 * @{ */
/**
 * This is the set of control commands that are passed to
 * @a CSL_smcHwControl(), with an optional argument type-casted to @a void*
 *
 * The arguments, if any, to be passed with each command are specified
 * next to that command.
 */
typedef enum {
    /**
     * Setup to enable the prefetch page
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMC_CMD_ENA_PREFETCH_PAGE = 0,

    /**
     * Setup to disable the prefetch page
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMC_CMD_DIS_PREFETCH_PAGE = 1,

    /**
     * Setup the prefetch flush
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMC_CMD_SET_PREFETCH_FLUSH = 2,

    /**
     * Setup to clear the MPFSR and MPFAR
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMC_CMD_CLR_MPFSR_MPFAR = 3,

    /**
     * Setup to page2 sleep
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMC_CMD_SET_SLEEP_PAGE2 = 4,

    /**
     * Setup to page2 wake
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMC_CMD_SET_WAKE_PAGE2 = 5,

    /**
     * Setup to page3 sleep
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMC_CMD_SET_SLEEP_PAGE3 = 6,

    /**
     * Setup to page3 wake
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMC_CMD_SET_WAKE_PAGE3 = 7

} CSL_SmcControlCmd;
/**
@} */


/**
 * @defgroup CSL_SMC_QUERYCMD_ENUM Query commands
 * @ingroup CSL_SMC_QUERY_API
 * @{ */
/**
 * This is the set of query commands to get the status of various
 * operations in SMC
 *
 * The arguments, if any, to be passed with each command are specified
 * next to that command. */
typedef enum {
    /**
     * Queries the prefetech page status
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_PREFETCH_PAGE_STS = 0,
    
    /**
     * Queries the mode
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_MODE = 1,

    /**
     * Queries the Cpu Id
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_CPU_ID  = 2,

    /**
     * Queries the Fault Address
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_FAULT_ADDRESS = 3,

    /**
     * Queries the powerdown page status
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_PWRDWN_PAGE_STS = 4,

    /**
     * Queries the AML link data status
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_AML_LINK_DATA_STS = 5,

    /**
     * Queries the atomic link
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_ATOMIC_LINK = 6,

    /**
     * Queries the link owner status
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_LINK_OWNER = 7,

    /**
     * Queries the link address
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_LINK_ADDR = 8,

    /**
     * Queries the link data
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMC_QUERY_AML_LINK_DATA = 9

} CSL_SmcHwStatusQuery;
/**
@} */


/** 
 * @brief This structure is used to hold the configuration/status information
 *        of different SMC pages
 */
typedef struct {
    /** Page selection */
    Uint32 index;
    
    /** Desired information in the registers */
    CSL_BitMask32 data;
} CSL_SmcData;

/**
 * @brief This structure will have the base-address information for the 
 *  peripheral
 *  instance
 */
typedef struct {
    /** Base-address of the Configuration registers of SMC.
     */
    CSL_SmcRegsOvly regs;
} CSL_SmcBaseAddress;


/**
 * @brief SMC specific parameters. Present implementation doesn't have
 *  any specific parameters.
 */
typedef struct {
    /** Bit mask to be used for module specific parameters.
     *  The below declaration is just a place-holder for future
     *  implementation.
     */
    CSL_BitMask16   flags;
} CSL_SmcParam;


/**
 * @brief SMC specific context information. Present implementation doesn't
 *  have any Context information.
 */

typedef struct {
    /** Context information of SMC.
     *  The below declaration is just a place-holder for future
     *  implementation.
     */
    Uint16  contextInfo;
} CSL_SmcContext;

/**
 * @brief This structure/object holds the context of the instance of SMC
 * opened using @a CSL_smcOpen() function.
 *
 * Pointer to this object is passed as SMC Handle to all SMC CSL APIs.
 * @a CSL_smcOpen() function intializes this structure based on the parameters
 * passed */
typedef struct CSL_SmcObj {
    /** Pointer to the register overlay structure of the SMC */
    CSL_SmcRegsOvly    regs;

    /** Instance of SMC being referred by this object  */
    CSL_InstNum         perNum;
} CSL_SmcObj;


/** @brief This is a pointer to @a CSL_SmcObj and is passed as the first
 * parameter to all SMC CSL APIs
 */
typedef struct CSL_SmcObj *CSL_SmcHandle;
/**
@} */


/* ************************************************************************\
 * SMC global function declarations
\* ************************************************************************/

/** ============================================================================
 *   @n@b CSL_smcInit
 *
 *   @b Description
 *   @n This function is idempotent i.e. calling it many times is same as
 *      calling it once. This function is only for book-keeping purpose
 *      and it doesn't touch the hardware (read/write registers) in any manner.
 *
 *   @b Arguments
     @verbatim
            pContext   Context information for the instance. Should be NULL

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK - Always returns
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  The CSL for SMC is initialized 
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim
        CSL_Status    status;
        ...
        status = CSL_smcInit(NULL);
        ...
 *   @endverbatim
 *  ============================================================================
 */
extern CSL_Status  CSL_smcInit (
    CSL_SmcContext    *pContext
);

/** ============================================================================
 *   @n@b CSL_smcOpen
 *
 *   @b Description
 *   @n Reserves the specified SMC for use. The device can be re-opened anytime
 *      after it has been normally closed, if so required. 
 *      The SMC handle returned by this call is input as an essential argument
 *      for the rest of the APIs in SMC module. 
 *
 *   @b Arguments
 *   @verbatim            
            pSmcObj        Pointer to SMC object that holds the context.
                           Memory for this object should be allocated by the 
                           user
 
            smcNum         The instance of SMC to be opened
 
            pSmcParam      Parameter for SMC
 
            pStatus        Pointer to the variable that holds the status of the 
                           open call

     @endverbatim
 *
 *   <b> Return Value </b>  
 *      CSL_SmcHandle
 *        CSL_SmcHandle to the requested instance of SMC if the call is 
 *        successful, otherwise @a NULL is returned.
 *
 *   <b> Pre Condition </b>
 *   @n  @a CSL_smcInit() must be called successfully.
 *       Memory for the @a CSL_SmcObj must be allocated outside this call. This 
 *       object must be retained while using this peripheral instance.
 *
 *   <b> Post Condition </b>
 *   @n   1.    The status is returned in the status variable. If status
 *              returned is
 *   @li            CSL_SOK            - Valid HECC handle is returned
 *   @li            CSL_ESYS_FAIL      - The HECC instance is invalid
 *   @li            CSL_ESYS_INVPARAMS - Invalid parameter
 *
 *        2.    HECC object structure is populated
 *
 *   @b  Modifies
 *   @n  1. The status variable
 *
 *       2. SMC object structure
 *
 *
 *   @b Example:
 *   @verbatim
        CSL_SmcHandle    hSmc;
        CSL_SmcObj       smcObj;
        CSL_Status       status;
        ...
        hSmc = CSL_smcOpen(&smcObj, CSL_SMC_0, NULL, &status);
        ...
 *   @endverbatim
 *
 * ===========================================================================
 */
extern CSL_SmcHandle CSL_smcOpen (
    CSL_SmcObj      *hSmcObj,
    CSL_InstNum      smcNum,
    CSL_SmcParam    *pSmcParam,
    CSL_Status      *pStatus
);

/** ============================================================================
 *   @n@b CSL_smcClose
 *
 *   @b Description
 *   @n Unreserves the SMC identified by the handle passed.
 *
 *   @b Arguments
 *   @verbatim
            hSmc            Handle to the SMC

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Close Successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_smcInit() and CSL_smcOpen() must be called successfully 
 *       in order before calling CSL_smcClose(). 
 * 
 *   <b> Post Condition </b>
 *   @n  The smc CSL APIs can not be called until the smc CSL is reopened 
 *       again using CSL_smcOpen()
 *
 *   @b Modifies
 *   @n SMC Handle
 *
 *   @b Example
 *   @verbatim
            CSL_Status       status;
            CSL_smcHandle    hSmc;
            ...
            status = CSL_smcClose(hSmc);
            ...

     @endverbatim
 *  ============================================================================
 */
extern CSL_Status  CSL_smcClose (
    CSL_SmcHandle    hSmc
);

/** ============================================================================
 *   @n@b CSL_smcHwControl
 *
 *   @b Description
 *   @n This function takes an input control command with an optional argument
 *      and accordingly controls the operation/configuration of SMC.
 *
 *   @b Arguments
 *   @verbatim
            hSmc       SMC handle returned by successful 'open'

            cmd         The command to this API indicates the action to be
                        taken on SMC.
                        Control command, refer @a CSL_SmcControlCmd for the 
                        list of commands supported

            arg         An optional argument.
                        Optional argument as per the control command, 
                        @a void * casted

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li         CSL_SOK               - Control command is successful.
 *   @li         CSL_ESYS_BADHANDLE    - Invalid handle
 *   @li         CSL_ESYS_INVCMD       - Invalid command
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_smcInit() and CSL_smcOpen() must be called successfully 
 *       in order before calling CSL_smcHwControl(). 
 *
 *       Refer to @a CSL_SmcHwControlCmd for the argument type (@a void*) that 
 *       needs to be passed with the control command
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n The hardware registers of SMC.
 *
 *   @b Example
 *   @verbatim
      CSL_Status    status;
      Uint32        arg;
      CSL_SmcHandle hSmc;
      ...
      // SMC object defined and HwSetup structure defined and initialized
      ...
      // Init successfully done
      ...
      // Open successfully done
      ...
      arg = CSL_SMC_PREFETECH_PAGE(1);
      status = CSL_smcHwControl(hSmc, CSL_SMC_CMD_ENA_PREFETCH_PAGE, &arg);
      ...
     @endverbatim
 * =============================================================================
 */
extern CSL_Status  CSL_smcHwControl (
    CSL_SmcHandle        hSmc,
    CSL_SmcControlCmd    cmd,
    void*                 arg
);

/** ============================================================================
 *   @n@b CSL_smcGetHwStatus
 *
 *   @b Description
 *   @n Gets the status of different operations or some setup-parameters of 
 *      SMC. The status is returned through the third parameter.
 *
 *   @b Arguments
 *   @verbatim
            hSmc     SMC handle returned by successful 'open'

            query     The query to this API of SMC which indicates the
                      status to be returned. 

                      Query command, refer @a CSL_SmcHwStatusQuery for the list 
                      of query commands supported 

            response  Response from the query. Pointer to appropriate object 
                      corresponding to the query command needs to be passed here

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li        CSL_SOK             - Status info return successful
 *   @li        CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li        CSL_ESYS_INVQUERY   - Invalid query command
 *   @li        CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_smcInit() and CSL_smcOpen() must be called successfully 
 *       in order before calling CSL_smcGetHwStatus(). 
 *       Refer to @a CSL_SmcHwStatusQuery for the argument to be passed along 
 *       with the corresponding query command.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n The third parameter 'response' will be modified.
 *
 *   @b Example
 *   @verbatim
        CSL_SmcHandle    hSmc;
        CSL_Status       status;
        Uint32           response;
        ...
        status = CSL_smcGetHwStatus(hSmc, CSL_SMC_QUERY_MODE, &response);
        ...
     @endverbatim
 * =============================================================================
 */
extern CSL_Status  CSL_smcGetHwStatus (
    CSL_SmcHandle           hSmc,
    CSL_SmcHwStatusQuery    myQuery,
    void*                   response
);


/** ============================================================================
 *   @n@b CSL_smcGetBaseAddress
 *
 *   @b Description
 *   @n  Function to get the base address of the peripheral instance.
 *       This function is used for getting the base address of the peripheral
 *       instance. This function will be called inside the CSL_smcOpen()
 *       function call. This function is open for re-implementing if the user
 *       wants to modify the base address of the peripheral object to point to
 *       a different location and there by allow CSL initiated write/reads into
 *       peripheral. MMR's go to an alternate location.
 *
 *   @b Arguments
 *   @verbatim
            smcNum        Specifies the instance of the SMC to be opened.

            pSmcParam     Module specific parameters.

            pBaseAddress  Pointer to baseaddress structure containing base
                          address details.

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li             CSL_OK            Successful on getting the base address 
 *                                     of SMC module.
 *   @li             CSL_ESYS_FAIL     The instance number is invalid.
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Base Address structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *
 *         2. Base address structure is modified.
 *
 *   @b Example
 *   @verbatim
        CSL_Status          status;
        CSL_SmcBaseAddress  baseAddress;
        ...
        status = CSL_smcGetBaseAddress(CSL_SMC_0, NULL, &baseAddress);
        ...
    @endverbatim
 * ===========================================================================
 */
extern CSL_Status CSL_smcGetBaseAddress (
    CSL_InstNum            smcNum,
    CSL_SmcParam           *pSmcParam,
    CSL_SmcBaseAddress     *pBaseAddress
);


#ifdef __cplusplus
}
#endif

#endif

