/* =============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ============================================================================
 *   @file  csl_smcp.h
 *
 *   @path  $(CSLPATH)\inc
 *
 *   @desc  Header file for functional layer of SMCP CSL
 *  - The different symbolic constants, enumerations, structure definitions
 *    and function prototype declarations
 *
 *    @date    10 June, 2005
 *    @author  Vinod Arumugham
 *
 */
/* =============================================================================
 *  Revision History
 *  ===============
 *  24-May-2006 NG  File created
 *
 * =============================================================================
 */

/** @mainpage SMCP CSL 3.x
 *
 * @section Introduction
 *
 * @subsection xxx Purpose and Scope
 * The purpose of this document is to identify a set of common CSL APIs for
 * the SMCP module across various devices. The CSL developer is expected to
 * refer to this document while designing APIs for these modules. Some of the
 * listed APIs may not be applicable to a given SMCP module. While this list
 * of APIs may not be sufficient to cover all the features of a
 * particular SMCP Module. The CSL developer should use his discretion designing
 * new APIs or extending the existing ones to cover these.
 *
 * @subsection aaa Terms and Abbreviations
 *   -# CSL:  Chip Support Library
 *   -# API:  Application Programmer Interface
 *
 * @subsection References
 *    -# CSL-001-DES, CSL 3.x Design Specification DocumentVersion 1.02
 *
 */
/**
 * @defgroup CSL_SMCP_API SMCP
 */
#ifndef _CSL_SMCP_H_
#define _CSL_SMCP_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <soc.h>
#include <csl_types.h>
#include <csl_error.h>
#include <cslr_smcp.h>

/**************************************************************************\
* SMCP global macro declarations
\**************************************************************************/

/** SMC read request was 0 wait state read request. */
#define CSL_SMCP_MASK_WS_0        (0x00000001u)
/** SMC read request was 1 wait state read request. */
#define CSL_SMCP_MASK_WS_1        (0x00000002u)
/** SMC read request was 2 wait state read request. */
#define CSL_SMCP_MASK_WS_2        (0x00000004u)
/** SMC read request was 3 wait state read request. */
#define CSL_SMCP_MASK_WS_3        (0x00000008u)
/** SMC read request was 4 wait state read request. */
#define CSL_SMCP_MASK_WS_4        (0x00000010u)
/** SMC read request was 5 wait state read request. */
#define CSL_SMCP_MASK_WS_5        (0x00000020u)
/** SMC read request was 6 wait state read request. */
#define CSL_SMCP_MASK_WS_6        (0x00000040u)
/** SMC read request was 7 wait state read request. */
#define CSL_SMCP_MASK_WS_7        (0x00000080u)

/** Mask value for bank0 */
#define CSL_SMCP_BANK0            (0x00000001u)
/** Mask value for bank1 */
#define CSL_SMCP_BANK1            (0x00000002u)
/** Mask value for bank2 */
#define CSL_SMCP_BANK2            (0x00000004u)
/** Mask value for bank3 */
#define CSL_SMCP_BANK3            (0x00000008u)


/**************************************************************************\
* SMCP global typedef declarations
\**************************************************************************/

/**
 * @defgroup CSL_SMCP_CONTROLCMD_ENUM Control commands
 * @ingroup CSL_SMCP_CONTROL_API
 * @{ */
/**
 * This is the set of control commands that are passed to
 * @a CSL_smcpHwControl(), with an optional argument type-casted to @a void*
 *
 * The arguments, if any, to be passed with each command are specified
 * next to that command.
 */
typedef enum {
    /**
     * Setup to set the bank
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMCP_CMD_SET_BANK = 0,
    /**
     * Setup the CLEAR bit of SL2PCMD
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMCP_CMD_CLR_PWS_CNT = 1,
    /**
     * Setup to enable / disable the profiler
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMCP_CMD_ENA_DIS_PROFILER = 2,

    /**
     * Setup combine WSx_EVENT to generate SMC profile event to GEM
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMCP_CMD_SET_MASK_WS = 3,
    /**
     * Setup does not combine WSx_EVENT to generate SMC profile event to GEM
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMCP_CMD_RESET_MASK_WS = 4,

    /**
     * Setup to reset the bank
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_SMCP_CMD_RESET_BANK = 5
} CSL_SmcpControlCmd;
/**
@} */


/**
 * @defgroup CSL_SMCP_QUERYCMD_ENUM Query commands
 * @ingroup CSL_SMCP_QUERY_API
 * @{ */
/**
 * This is the set of query commands to get the status of various
 * operations in SMCP
 *
 * The arguments, if any, to be passed with each command are specified
 * next to that command. */
typedef enum {
    /**
     * Queries the bank
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMCP_QUERY_BANK = 0,
    /**
     * Queries the CLEAR bit of SL2PCMD
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMCP_QUERY_PWS_CNT = 1,

    /**
     * Queries the wait x read state accesses
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMCP_QUERY_PFC_CNT = 2,
    /**
     * Queries the counter saturation state
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMCP_QUERY_PWS_CNT_SAT = 3,
    /**
     * Queries the prefetech count saturate state
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMCP_QUERY_PFC_CNT_SAT = 4,
    /**
     * Queries the profiler status
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMCP_QUERY_PROFILER_STS = 5,
    /**
     * Queries the wait read access state
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMCP_QUERY_MASK_WS = 6,
    
    /**
     * Queries the prefetch wait read access clear status
     * @param (Uint32 *)
     * @return CSL_SOK
     */
    CSL_SMCP_QUERY_PWS_CNT_CLR = 7 
    
} CSL_SmcpHwStatusQuery;
/**
@} */

/** 
 * @brief This structure is used to hold the configuration/status information
 *        of different SMC pages
 */
typedef struct {
    /** Page selection */
    Uint32 index;
    
    /** Desired information in the registers */
    CSL_BitMask32 data;
} CSL_SmcpData;

/**
 * @brief This structure will have the base-address information for the 
 *  peripheral instance
 */
typedef struct {
    /** Base-address of the Configuration registers of SMCP.
     */
    CSL_SmcpRegsOvly regs;
} CSL_SmcpBaseAddress;


/**
 * @brief SMCP specific parameters. Present implementation doesn't have
 *  any specific parameters.
 */
typedef struct {
    /** Bit mask to be used for module specific parameters.
     *  The below declaration is just a place-holder for future
     *  implementation.
     */
    CSL_BitMask16   flags;
} CSL_SmcpParam;


/**
 * @brief SMCP specific context information. Present implementation doesn't
 *  have any Context information.
 */

typedef struct {
    /** Context information of SMCP.
     *  The below declaration is just a place-holder for future
     *  implementation.
     */
    Uint16  contextInfo;
} CSL_SmcpContext;

/**
 * @brief This structure/object holds the context of the instance of SMCP
 * opened using @a CSL_smcpOpen() function.
 *
 * Pointer to this object is passed as SMCP Handle to all SMCP CSL APIs.
 * @a CSL_smcpOpen() function intializes this structure based on the parameters
 * passed */
typedef struct CSL_SmcpObj {
    /** Pointer to the register overlay structure of the SMCP */
    CSL_SmcpRegsOvly    regs;

    /** Instance of SMCP being referred by this object  */
    CSL_InstNum         perNum;
} CSL_SmcpObj;


/** @brief This is a pointer to @a CSL_SmcpObj and is passed as the first
 * parameter to all SMCP CSL APIs
 */
typedef struct CSL_SmcpObj *CSL_SmcpHandle;
/**
@} */


/* ************************************************************************\
 * SMCP global function declarations
\* ************************************************************************/

/** ============================================================================
 *   @n@b CSL_smcpInit
 *
 *   @b Description
 *   @n This function is idempotent i.e. calling it many times is same as
 *      calling it once. This function is only for book-keeping purpose
 *      and it doesn't touch the hardware (read/write registers) in any manner.
 *
 *   @b Arguments
     @verbatim
            pContext   Context information for the instance. Should be NULL

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK - Always returns
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim
        CSL_Status status;
        ...
        status = CSL_smcpInit(NULL);
        ...
     @endverbatim
 *  ===========================================================================
 */
extern CSL_Status  CSL_smcpInit (
    CSL_SmcpContext    *pContext
);

/** ============================================================================
 *   @n@b CSL_smcpOpen
 *
 *   @b Description
 *   @n Reserves the specified SMCP for use. The device can be re-opened anytime
 *      after it has been normally closed, if so required. 
 *      The SMCP handle returned by this call is input as an essential argument
 *      for the rest of the APIs in SMCP module. 
 *
 *   @b Arguments
 *   @verbatim            
            pSmcpObj        Pointer to SMCP object that holds the context.
                            Memory for this object should be allocated by the 
                            user
 
            smcpNum         The instance of SMCP to be opened
 
            pSmcpParam      Parameter for SMCP
 
            pStatus         Pointer to the variable that holds the status of the 
                            open call

     @endverbatim
 *
 *   <b> Return Value </b>  
 *      CSL_SmcpHandle
 *        CSL_SmcpHandle to the requested instance of SMCP if the call is 
 *        successful, otherwise @a NULL is returned.
 *
 *   <b> Pre Condition </b>
 *   @n  @a CSL_smcpInit() must be called successfully.
 *       Memory for the @a CSL_SmcpObj must be allocated outside this call. This 
 *       object must be retained while using this peripheral instance.
 *
 *   <b> Post Condition </b>
 *   @n   1.    The status is returned in the status variable. If status
 *              returned is
 *   @li            CSL_SOK            - Valid HECC handle is returned
 *   @li            CSL_ESYS_FAIL      - The HECC instance is invalid
 *   @li            CSL_ESYS_INVPARAMS - Invalid parameter
 *
 *        2.    SMCP object structure is populated
 *
 *   @b  Modifies
 *   @n  1. The status variable
 *
 *       2. SMCP object structure
 *
 *   @b Example:
 *   @verbatim
        CSL_SmcpHandle    hSmcp = NULL;
        CSL_SmcpObj       smcpObj;
        CSL_Status        status = CSL_SOK;
        ...
        hSmcp = CSL_smcpOpen(&smcpObj, CSL_SMCP_0, NULL, &status);
        ...
     @endverbatim
 *
 * ===========================================================================
 */
extern CSL_SmcpHandle CSL_smcpOpen (
    CSL_SmcpObj      *hSmcpObj,
    CSL_InstNum      smcpNum,
    CSL_SmcpParam    *pSmcpParam,
    CSL_Status       *pStatus
);

/** ============================================================================
 *   @n@b CSL_smcpClose
 *
 *   @b Description
 *   @n Unreserves the SMCP identified by the handle passed.
 *
 *   @b Arguments
 *   @verbatim
            hSmcp            Handle to the SMCP

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li           CSL_SOK             - Close Successful
 *   @li           CSL_ESYS_BADHANDLE  - Invalid handle
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_smcpInit() and CSL_smcpOpen() must be called successfully 
 *       in order before calling CSL_smcpClose(). 
 * 
 *   <b> Post Condition </b>
 *   @n  The smcp CSL APIs can not be called until the i2c CSL is reopened 
 *       again using CSL_smcpOpen()
 *
 *   @b Modifies
 *   @n SMCP Handle
 *
 *   @b Example
 *   @verbatim
            CSL_Status       status;
            CSL_smcpHandle   hSmcp;
            ...
            status = CSL_smcpClose(hSmcp);
            ...
     @endverbatim
 *  ============================================================================
 */
extern CSL_Status  CSL_smcpClose (
    CSL_SmcpHandle    hSmcp
);


/** ============================================================================
 *   @n@b CSL_smcpHwControl
 *
 *   @b Description
 *   @n This function takes an input control command with an optional argument
 *      and accordingly controls the operation/configuration of SMCP.
 *
 *   @b Arguments
 *   @verbatim
            hSmcp       SMCP handle returned by successful 'open'

            cmd         The command to this API indicates the action to be
                        taken on SMCP.
                        Control command, refer @a CSL_SmcpControlCmd for the 
                        list of commands supported

            arg         An optional argument.
                        Optional argument as per the control command, 
                        @a void * casted

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li         CSL_SOK              - Control command is successful
 *   @li         CSL_ESYS_BADHANDLE   - Invalid handle
 *   @li         CSL_ESYS_INVCMD      - Invalid command
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_smcpInit() and CSL_smcpOpen() must be called successfully 
 *       in order before calling CSL_smcpHwControl(). 
 *
 *       Refer to @a CSL_SmcpHwControlCmd for the argument type (@a void*) that 
 *       needs to be passed with the control command
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n The hardware registers of SMCP.
 *
 *   @b Example
 *   @verbatim
      CSL_Status     status;
      Uint32         arg;
      CSL_SmcpHandle hSmcp;
      ...
      // SMCP object defined and HwSetup structure defined and initialized
      ...
      // Init successfully done
      ...
      // Open successfully done
      ...
      arg = CSL_SMCP_BANK0;
      status = CSL_smcpHwControl(hSmcp,
                                 CSL_SMCP_CMD_SET_BANK,
                                 &arg);
      ...  
     @endverbatim
 * =============================================================================
 */
CSL_Status  CSL_smcpHwControl (
    CSL_SmcpHandle        hSmcp,
    CSL_SmcpControlCmd    cmd,
    void*                 arg
);

/** ============================================================================
 *   @n@b CSL_smcpGetHwStatus
 *
 *   @b Description
 *   @n Gets the status of different operations or some setup-parameters of 
 *      SMCP. The status is returned through the third parameter.
 *
 *   @b Arguments
 *   @verbatim
            hSmcp     SMCP handle returned by successful 'open'

            query     The query to this API of SMCP which indicates the
                      status to be returned. 

                      Query command, refer @a CSL_SmcpHwStatusQuery for the list 
                      of query commands supported 

            response  Response from the query. Pointer to appropriate object 
                      corresponding to the query command needs to be passed here

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li            CSL_SOK             - Status info return successful
 *   @li            CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li            CSL_ESYS_INVQUERY   - Invalid query command
 *   @li            CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_smcpInit() and CSL_smcpOpen() must be called successfully 
 *       in order before calling CSL_smcpGetHwStatus(). 
 *       Refer to @a CSL_SmcpHwStatusQuery for the argument to be passed along 
 *       with the corresponding query command.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n The third parameter 'response'
 *
 *   @b Example
 *   @verbatim
        CSL_SmcpHandle   hSmcp;
        CSL_Status       status;
        Uint32           response;
        ...
        status = CSL_smcpGetHwStatus(hSmcp, CSL_SMCP_QUERY_PFC_CNT, &response);
        ...
     @endverbatim
 * =============================================================================
 */
CSL_Status  CSL_smcpGetHwStatus (
    CSL_SmcpHandle           hSmcp,
    CSL_SmcpHwStatusQuery    myQuery,
    void*                    response
);


/** ============================================================================
 *   @n@b CSL_smcpGetBaseAddress
 *
 *   @b Description
 *   @n  Function to get the base address of the peripheral instance.
 *       This function is used for getting the base address of the peripheral
 *       instance. This function will be called inside the CSL_smcpOpen()
 *       function call. This function is open for re-implementing if the user
 *       wants to modify the base address of the peripheral object to point to
 *       a different location and there by allow CSL initiated write/reads into
 *       peripheral. MMR's go to an alternate location.
 *
 *   @b Arguments
 *   @verbatim
            smcpNum        Specifies the instance of the SMCP to be opened.

            pSmcpParam     Module specific parameters.

            pBaseAddress  Pointer to baseaddress structure containing base
                          address details.

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_OK            Successful on getting the base 
 *                                            address of SMCP
 *   @li                    CSL_ESYS_FAIL     The instance number is invalid.
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Base Address structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *
 *         2. Base address structure is modified.
 *
 *   @b Example
 *   @verbatim
        CSL_Status          status;
        CSL_SmcpBaseAddress  baseAddress;
        ...
        status = CSL_smcpGetBaseAddress(CSL_SMCP_0, NULL, &baseAddress);
        ...
    @endverbatim
 * ===========================================================================
 */
CSL_Status   CSL_smcpGetBaseAddress (
        CSL_InstNum             smcpNum,
        CSL_SmcpParam          *pSmcpParam,
        CSL_SmcpBaseAddress    *pBaseAddress
);


#ifdef __cplusplus
}
#endif

#endif

