/* =============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ============================================================================
 *   @file  csl_tsip.h
 *
 *   @path  $(CSLPATH)\inc
 *
 *   @desc  Header file for functional layer of TSIP CSL
 *  - The different symbolic constants, enumerations, structure definitions
 *    and function prototype declarations
 *                                  
 */
/* =============================================================================
 *  Revision History
 *  ===============
 *  23-Mar-2006 NG  File taken from TI code base and modified for c6486.
 *                  Updated according coding guidelines (doxygen compliant)
 *
 * =============================================================================
 */

/** @mainpage TSIP CSL 3.x
 *
 * @section Introduction
 *
 * @subsection xxx Purpose and Scope
 * The purpose of this document is to identify a set of common CSL APIs for
 * the TSIP module across various devices. The CSL developer is expected to
 * refer to this document while designing APIs for these modules. Some of the
 * listed APIs may not be applicable to a given TSIP module. While this list
 * of APIs may not be sufficient to cover all the features of a
 * particular TSIP Module. The CSL developer should use his discretion designing
 * new APIs or extending the existing ones to cover these.
 *
 * @subsection aaa Terms and Abbreviations
 *   -# CSL:  Chip Support Library
 *   -# API:  Application Programmer Interface
 *
 * @subsection References
 *    -# CSL-001-DES, CSL 3.x Design Specification DocumentVersion 1.02
 *
 */
#ifndef _CSL_TSIP_H_
#define _CSL_TSIP_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <soc.h>
#include <csl_types.h>
#include <csl_error.h>
#include <cslr_tsip.h>

/**************************************************************************\
* TSIP global macro declarations
\**************************************************************************/

/** @defgroup CSL_ETSIP_SYMBOL Error codes
 * @ingroup CSL_TSIP_SYMBOL
 *
 * @{ */
/** Invalid Control Command */
#define CSL_ETSIP_INVCNTLCMD                         (CSL_ETSIP_FIRST - 0)

/** Invalid Query */
#define CSL_ETSIP_INVQUERY                           (CSL_ETSIP_FIRST - 1)

/** Invalid Parameter */
#define CSL_ETSIP_INVPARAMS                          (CSL_ETSIP_FIRST - 2)

/** Invalid Size */
#define CSL_ETSIP_INVSIZE                            (CSL_ETSIP_FIRST - 3)

/** 'Does not exist' */
#define CSL_ETSIP_NOTEXIST                           (CSL_ETSIP_FIRST - 4)

/** Invalid mode to conduct operation */
#define CSL_ETSIP_INVMODE                            (CSL_ETSIP_FIRST - 5)
/**
@} */

/** @defgroup CSL_TSIP_NUM_CHAN Number of Channels
 * @ingroup CSL_TSIP_SYMBOL
 *
 * @{ */
#define CSL_TSIP_NUM_CHAN                            6
/**
@} */


/** @defgroup CSL_TSIP_RCV_CHAN_MASK Receive Mask
 * @ingroup CSL_TSIP_SYMBOL
 *
 * @{ */
#define CSL_TSIP_RCV_CHAN_MASK                       0x8000
/**
@} */


/** @defgroup CSL_TSIP_CHAN_CFGB Configuration B
 * @ingroup CSL_TSIP_SYMBOL
 *
 * @{ */
#define CSL_TSIP_CHAN_CFGB                          0x1000
/**
@} */

/** @defgroup CSL_TSIP_CHAN_CFGA Configuration A
 * @ingroup CSL_TSIP_SYMBOL
 *
 * @{ */
#define CSL_TSIP_CHAN_CFGA                          0
/**
@} */

/** @defgroup CSL_TSIP_BITMAP_SIZE Bitmap Size
 * @ingroup CSL_TSIP_SYMBOL
 *
 * @{ */
#define CSL_TSIP_BITMAP_SIZE                        64
/**
@} */


/**************************************************************************\
* TSIP global typedef declarations
\**************************************************************************/

/**
 * @defgroup CSL_TSIP_DATARATE_ENUM Data Rate
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Data Rate supported on TSIP
 *
 * Use this symbol for setting Data rate
 * @{ */
typedef enum {
    /** Data rate is 8 Mbps */
    CSL_TSIP_DATARATE_8M         = 0,
    /** Data rate is 16 Mbps */
    CSL_TSIP_DATARATE_16M        = 1,
    /** Data rate is 32 Mbps */
    CSL_TSIP_DATARATE_32M        = 2
} CSL_TsipDataRate;
/**
@} */

/**
 * @defgroup CSL_TSIP_XMTRCV_ENUM Transmit or Receive Channel
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Transmit or Receive Channel of TSIP
 *
 * Use this symbol for repereseting a Transmit or a Receive channel
 * @{ */
typedef enum {
    /** Transmit Channel */
    CSL_TSIP_XMT_CHAN         = 0,
    /** Receive Channel */
    CSL_TSIP_RCV_CHAN         = 0x8000 
} CSL_TsipXmtRcv;
/**
@} */

/**
 * @defgroup CSL_TSIP_PRI_ENUM Priority 0 to 7
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Priority 0 to 7
 *
 * Use this symbol for repereseting DMA priority
 * @{ */
typedef enum {
    /** Priority 0 */
    CSL_TSIP_PRI_0         = 0,
    /** Priority 1 */
    CSL_TSIP_PRI_1         = 1,
    /** Priority 2 */
    CSL_TSIP_PRI_2         = 2,
    /** Priority 3 */
    CSL_TSIP_PRI_3         = 3,
    /** Priority 4 */
    CSL_TSIP_PRI_4         = 4,
    /** Priority 5 */
    CSL_TSIP_PRI_5         = 5,
    /** Priority 6 */
    CSL_TSIP_PRI_6         = 6,
    /** Priority 7 */
    CSL_TSIP_PRI_7         = 7
} CSL_TsipPri;
/**
@} */

/**
 * @defgroup CSL_TSIP_CHANUM_ENUM Channel Numbers
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Transmit and Receive channels supported on TSIP.
 *
 * Use this symbol for Transmit and Receive channel number representation
 * @{ */
typedef enum {
    /** Transmit Channel 0 */
    CSL_TSIP_XMT_CHA0         = 0x0000,
    /** Transmit Channel 1 */
    CSL_TSIP_XMT_CHA1         = 0x0001,
    /** Transmit Channel 2 */
    CSL_TSIP_XMT_CHA2         = 0x0002,
    /** Transmit Channel 3 */
    CSL_TSIP_XMT_CHA3         = 0x0003,
    /** Transmit Channel 4 */
    CSL_TSIP_XMT_CHA4         = 0x0004,
    /** Transmit Channel 5 */
    CSL_TSIP_XMT_CHA5         = 0x0005,
    /** Receive Channel 0 */
    CSL_TSIP_RCV_CHA0         = 0x8000,
    /** Receive Channel 1 */
    CSL_TSIP_RCV_CHA1         = 0x8001,
    /** Receive Channel 2 */
    CSL_TSIP_RCV_CHA2         = 0x8002,
    /** Receive Channel 3 */
    CSL_TSIP_RCV_CHA3         = 0x8003,
    /** Receive Channel 4 */
    CSL_TSIP_RCV_CHA4         = 0x8004,
    /** Receive Channel 5 */
    CSL_TSIP_RCV_CHA5         = 0x8005
} CSL_TsipChanum;
/**
@} */

/**
 * @defgroup CSL_TSIP_CHACFG_ENUM Channel and Configuration Number
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Transmit and Receive channels supported on TSIP.
 *
 * Use this symbol for Transmit and Receive channel number representation
 * @{ */
typedef enum {
    /** Transmit Channel 0 Config A*/
    CSL_TSIP_XMT_CHA0_CFGA         = 0x0000,
    /** Transmit Channel 1 Config A*/
    CSL_TSIP_XMT_CHA1_CFGA         = 0x0001,
    /** Transmit Channel 2 Config A*/
    CSL_TSIP_XMT_CHA2_CFGA         = 0x0002,
    /** Transmit Channel 3 Config A*/
    CSL_TSIP_XMT_CHA3_CFGA         = 0x0003,
    /** Transmit Channel 4 Config A*/
    CSL_TSIP_XMT_CHA4_CFGA         = 0x0004,
    /** Transmit Channel 5 Config A*/
    CSL_TSIP_XMT_CHA5_CFGA         = 0x0005,
    /** Transmit Channel 0 Config B*/
    CSL_TSIP_XMT_CHA0_CFGB         = 0x1000,
    /** Transmit Channel 1 Config B*/
    CSL_TSIP_XMT_CHA1_CFGB         = 0x1001,
    /** Transmit Channel 2 Config B*/
    CSL_TSIP_XMT_CHA2_CFGB         = 0x1002,
    /** Transmit Channel 3 Config B*/
    CSL_TSIP_XMT_CHA3_CFGB         = 0x1003,
    /** Transmit Channel 4 Config B*/
    CSL_TSIP_XMT_CHA4_CFGB         = 0x1004,
    /** Transmit Channel 5 Config B*/
    CSL_TSIP_XMT_CHA5_CFGB         = 0x1005,
    /** Receive Channel 0 Config A*/
    CSL_TSIP_RCV_CHA0_CFGA         = 0x8000,
    /** Receive Channel 1 Config A*/
    CSL_TSIP_RCV_CHA1_CFGA         = 0x8001,
    /** Receive Channel 2 Config A*/
    CSL_TSIP_RCV_CHA2_CFGA         = 0x8002,
    /** Receive Channel 3 Config A*/
    CSL_TSIP_RCV_CHA3_CFGA         = 0x8003,
    /** Receive Channel 4 Config A*/
    CSL_TSIP_RCV_CHA4_CFGA         = 0x8004,
    /** Receive Channel 5 Config A*/
    CSL_TSIP_RCV_CHA5_CFGA         = 0x8005,
    /** Receive Channel 0 Config B*/
    CSL_TSIP_RCV_CHA0_CFGB         = 0x9000,
    /** Receive Channel 1 Config B*/
    CSL_TSIP_RCV_CHA1_CFGB         = 0x9001,
    /** Receive Channel 2 Config B*/
    CSL_TSIP_RCV_CHA2_CFGB         = 0x9002,
    /** Receive Channel 3 Config B*/
    CSL_TSIP_RCV_CHA3_CFGB         = 0x9003,
    /** Receive Channel 4 Config B*/
    CSL_TSIP_RCV_CHA4_CFGB         = 0x9004,
    /** Receive Channel 5 Config B*/
    CSL_TSIP_RCV_CHA5_CFGB         = 0x9005
} CSL_TsipChanCfg;
/**
@} */

/**
 * @defgroup CSL_TSIP_CLKM_ENUM Clock mode options
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief TSIP companding options
 *
 * Use this symbol to set Clock mode
 * @{ */
typedef enum {
    /** Double rate clock */
    CSL_TSIP_CLKM_DBL = 0,
    /** Single rate clock */
    CSL_TSIP_CLKM_SGL = 1
} CSL_TsipClkm;
/**
@} */

/**
 * @defgroup CSL_TSIP_FRAMECOUNT_ENUM Frame Count options
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief TSIP companding options
 *
 * Use this symbol to represent Frame Count
 * @{ */
typedef enum {
    /** Frame Count is 0 */
    CSL_TSIP_FRAMECOUNT_1   = 0,
    /** Frame Count is 255 */
    CSL_TSIP_FRAMECOUNT_256 = 255
} CSL_TsipFramecount;
/**
@} */

/**
 * @defgroup CSL_TSIP_FRAMESIZE_ENUM Frame Count options
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief TSIP companding options
 *
 * Use this symbol to represent Frame Size
 * @{ */
typedef enum {
    /** Frame Count is 0 */
    CSL_TSIP_FRAMESIZE_1   = 0,
    /** Frame Count is 255 */
    CSL_TSIP_FRAMESIZE_128 = 127
} CSL_TsipFramesize;
/**
@} */

/**
 * @defgroup CSL_TSIP_TIMESLOT_ENUM Frame Count options
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief TSIP companding options
 *
 * Use this symbol to represent Timeslot configuration
 * @{ */
typedef enum {
    /** Timeslot Disabled */
    CSL_TSIP_TIMESLOT_DISABLED   = 0,
    /** Linear */
    CSL_TSIP_TIMESLOT_LINEAR     = 1,
    /** u-law */
    CSL_TSIP_TIMESLOT_ULAW       = 2,
    /** a-law */
    CSL_TSIP_TIMESLOT_ALAW       = 3
} CSL_TsipTimeslot;
/**
@} */

/**
 * @defgroup CSL_TSIP_CLKSRC_ENUM Frame and Clock Source
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Clock Source.... A/B
 *
 * Use this symbol to select the clock source
 * @{ */
typedef enum {
    /** Clock and Frame sync Source A  */
    CSL_TSIP_CLKSRC_A      = 0,
    /** Clock and Frame sync Source B  */
    CSL_TSIP_CLKSRC_B      = 1
} CSL_TsipClkSrc;
/**
@} */

/**
 * @defgroup CSL_TSIP_CLKP_ENUM Data Clock Polarity
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Clock Polarity.... rising/falling
 *
 * Use this symbol to represent data/fsync clock polarity
 * @{ */
typedef enum {
    /** Rising Polarity  */
    CSL_TSIP_CLKP_RISING     = 0,
    /** Falling Polarity */
    CSL_TSIP_CLKP_FALLING    = 1
} CSL_TsipClkp;
/**
@} */

/**
 * @defgroup CSL_TSIP_ENDIAN_ENUM Endian Selection
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Endian Selection.... Little/Big
 *
 * Use this symbol to represent the endianness
 * @{ */
typedef enum {
    /** Little Endian */
    CSL_TSIP_ENDIAN_LITTLE    = 0,
    /** Big Endian */
    CSL_TSIP_ENDIAN_BIG       = 1
} CSL_TsipEndian;
/**
@} */

/**
 * @defgroup CSL_TSIP_FSYNCP_ENUM Frame Sync Polarity
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief TSIP Frame Sync Polarity
 *
 * Use this symbol to represent Frame Sync Polarity
 * This symbol is used on both RCV and XMT
 * @{ */
typedef enum {
    /** Active Low */
    CSL_TSIP_FSYNCP_ALOW     = 0,
    /** Active High */
    CSL_TSIP_FSYNCP_AHIGH    = 1
} CSL_TsipFsyncp;
/**
@} */

/**
 * @defgroup CSL_TSIP_INT_ENUM Interrupt selection
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief TSIP Interrupt selection
 *
 * Use this symbol to set the interrupt selection time
 * This symbol could be used for both RCV and XMT
 * @{ */
typedef enum {
    /** Interrupt on Acknowledgement */
    CSL_TSIP_INT_ACK        = 0,
    /** Interrupt on Delay */
    CSL_TSIP_INT_DLY        = 1,
    /** Interrupt on either Acknowledgement or Delay */
    CSL_TSIP_INT_ACKORDLY   = 2,
    /** Interrupt on both Acknowledgement and Delay */
    CSL_TSIP_INT_ACKNDLY    = 3
} CSL_TsipInt;
/**
@} */

/**
 * @defgroup CSL_TSIP_XMTDIS_ENUM Transmit Output Disable
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Transmit Output Disable State
 *
 * Use this symbol to represent the transmit output disable state
 * @{ */
typedef enum {
    /** High Impedence */
    CSL_TSIP_XMTDIS_HIGHIMP    = 0,
    /** Driven Low */
    CSL_TSIP_XMTDIS_LOW        = 2,
    /** Driven HIGH */
    CSL_TSIP_XMTDIS_HIGH       = 3
} CSL_TsipXmtDis;
/**
@} */

/**
 * @defgroup CSL_TSIP_CLKD_ENUM Clock Mode
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Clock Redundancy selection source
 *
 * Use this symbol to set the CLKD for redundancy clock selection
 * @{ */
typedef enum {
    /** Redundant Clock Mode */
    CSL_TSIP_CLKD_REDUN    = 0,
    /** Dual Clock Mode */
    CSL_TSIP_CLKD_DUAL     = 1
} CSL_TsipClkd;
/**
@} */


/**
 * @defgroup CSL_TSIP_CHST_ENUM Channel Active Status
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Channel Status: inactive or a config or b config
 *
 * Use this symbol to represent the Channel Status
 * @{ */
typedef enum {
    /** Channel is Inactive */
    CSL_TSIP_CHST_INACTIVE        = 0,
    /** A Config/Bitmap is used */
    CSL_TSIP_CHST_AACTIVE         = 1,
    /** B Config/Bitmap is used */
    CSL_TSIP_CHST_BACTIVE         = 2
} CSL_TsipChst;
/**
@} */


/**
 * @defgroup CSL_TSIP_EMU_ENUM Emulation mode setting
 * @ingroup CSL_TSIP_ENUM
 *
 * @brief Emulation mode setting
 *
 * Use this symbol to set the Emulation Mode
 * @{ */
typedef enum {
    /** Emulation mode stop */
    CSL_TSIP_EMU_STOP             = 0,
    /** Emulation mode TX stop */
    CSL_TSIP_EMU_TX_STOP          = 1,
    /** Emulation free run mode */
    CSL_TSIP_EMU_FREERUN          = 2
} CSL_TsipEmu;
/**
@} */


/**
 * @defgroup CSL_TSIP_CONTROLCMD_ENUM Control commands
 * @ingroup CSL_TSIP_CONTROL_API
 * @{ */
/**
 * This is the set of control commands that are passed to
 * @a CSL_tsipHwControl(), with an optional argument type-casted to @a void*
 *
 * The arguments, if any, to be passed with each command are specified
 * next to that command.
 */
typedef enum {
    /**
     * Setup Transmit Clock and Frame Sync Source
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_XCLK_SRC        = 0,
    /**
     * Setup Transmit Clock Mode...Double vs Single Clock
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_XCLK_MODE       = 1,

    /**
     * Setup Transmit Data rate 8/16/32
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_XDATA_RATE      = 2,

    /**
     * Setup Transmit Output when transmit is disabled
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_XDIS_OUTPUT     = 3,

    /**
     * Setup Data Delay
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_XDATA_DELAY     = 4,

    /**
     * Setup Transmit data clock polarity
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_XDCLK_POL       = 5,

    /**
     * Setup Receive Frame clock polarity
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_XFCLK_POL       = 6,

    /**
     * Setup Transmit Frame Sync Polarity
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_XFSYNC_POL      = 7,

    /**
     * Setup Receive Clock Source
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_RCLK_SRC        = 8,
    /**
     * Setup Receive Clock Mode
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_RCLK_MODE       = 9,

    /**
     * Setup Receive Data Rate
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_RDATA_RATE      = 10,

    /**
     * Setup Receive Data Delay
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_RDATA_DELAY     = 11,

    /**
     * Setup Receive Data Clock Polarity
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_RDCLK_POL       = 12,

    /**
     * Setup Receive Frame Clock Polarity
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_RFCLK_POL       = 13,

    /**
     * Setup Receive Frame Sync Polarity
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_RFSYNC_POL      = 14,

    /**
     * Setup Clock redundancy selection
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_CLKD            = 15,

    /**
     * Reset SIU
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_RST_SIU             = 16,

    /**
     * Enable SIU Receive
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_ENA_SIU_RCV         = 17,

    /**
     * Enable SIU Transmit
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_ENA_SIU_XMT         = 18,

    /**
     * Disable SIU Receive
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_DIS_SIU_RCV         = 19,

    /**
     * Disable SIU Transmit
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_DIS_SIU_XMT         = 20,

    /**
     * Set the Priority for Dma
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_PRI             = 21,

    /**
     * Set the maximum priority for the Priority to be set
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_MAXPRI          = 22,

    /**
     * Clear the Error Queue Overflow flag
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_CLR_QOV             = 23,

    /**
     * Clear the Error Queue
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_CLRQ                = 24,

    /**
     * Pop the top from the Error Queue
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_POPQ                = 25,

    /**
     * Enable a Transmit or Receive Channel
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_ENA_CHAN            = 26,

    /**
     * Disable a Transmit or Receive Channel
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_DIS_CHAN            = 27,

    /**
     * Configure a timeslot in the bitmap
     *
     * @param (CSL_TsipTslotCfg *)
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_CFG_TIMESLOT        = 28,

    /**
     * Enable DMATCU
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_ENA_DMA             = 29,

    /**
     * Disable DMATCU
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_DIS_DMA             = 30,

    /**
     * Setup the word Endianiness for incoming data
     *
     * @param None
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_SET_ENDIAN          = 31,

    /**
     * Enable SIU Transmit
     *
     * @return CSL_SOK
     */
    CSL_TSIP_CMD_RST_DMA             = 32

} CSL_TsipControlCmd;
/**
@} */


/**
 * @defgroup CSL_TSIP_QUERYCMD_ENUM Query commands
 * @ingroup CSL_TSIP_QUERY_API
 * @{ */
/**
 * This is the set of query commands to get the status of various
 * operations in TSIP
 *
 * The arguments, if any, to be passed with each command are specified
 * next to that command. */
typedef enum {
    /**
     * Queries the current XMT block
     * @param (CSL_TsipPeriId *)
     * @return CSL_SOK
     */
    CSL_TSIP_QUERY_PID         = 0,

    /**
     * Queries the current XMT block
     * @param  (Uint32 *)
     * @return CSL_SOK
     */
    CSL_TSIP_QUERY_XFRFC       = 1,

    /**
     * Queries the current XMT block
     * @param  (Uint32 *)
     * @return CSL_SOK
     */
    CSL_TSIP_QUERY_RFRFC       = 2,

    /**
     * Queries the current status of the bitmap
     * @param  (CSL_TsipChst *)
     * @return CSL_SOK
     */
    CSL_TSIP_QUERY_BMST        = 3,

    /**
     * Queries the current status of a channel
     * @param  (CSL_TsipChstat *)
     * @return CSL_SOK
     */
    CSL_TSIP_QUERY_CHST        = 4,

    /**
     * Queries the current Channel Configuration parameters
     * @param  (CSL_TsipDrchRegs * or CSL_TsipDxchRegs *)
     * @return CSL_SOK
     */
    CSL_TSIP_QUERY_CHANCFG     = 5,

    /**
     * Queries the Error Information for a channel
     * @param  (CSL_TsipErrInfo *)
     * @return CSL_SOK
     s*/     
    CSL_TSIP_QUERY_ERR_INFO    = 6

} CSL_TsipHwStatusQuery;
/**
@} */


/**
 * @brief This structure will have the base-address information for the 
 *  peripheral instance
 */
typedef struct {
    /** Base-address of the Configuration registers of TSIP. */
    CSL_TsipRegsOvly regs;
} CSL_TsipBaseAddress;


/**
 * @brief TSIP specific parameters. Present implementation doesn't have
 *  any specific parameters.
 */
typedef struct {
    /** Bit mask to be used for module specific parameters.
     *  The below declaration is just a place-holder for future implementation.
     */
    CSL_BitMask16   flags;
} CSL_TsipParam;


/**
 * @brief TSIP specific context information. Present implementation doesn't
 *  have any Context information.
 */
typedef struct {
    /** Context information of TSIP.
     *  The below declaration is just a place-holder for future
     *  implementation.
     */
    Uint16  contextInfo;
} CSL_TsipContext;


/**
 * @brief Pointer to this structure is used as the third argument in
 * @a CSL_tsipGetHwStatus () for getting revison, type and class info
 * of TSIP  */
typedef struct CSL_TsipPerId {
    /** Module Identifier of the peripheral */
    Uint16    modid; 
    /**  Major Revision */
    Uint16    major; 
    /** Minor Revision */
    Uint16    minor; 
} CSL_TsipPerId;
/**
@} */

/** @addtogroup CSL_TSIP_DATASTRUCT
*
* @{ */
/**
 * @brief Pointer to this structure is used as the third argument in
 * @a CSL_tsipGetHwStatus () for getting revison, type and class info
 * of TSIP  */
typedef struct CSL_TsipTimeslotCfg{
    /** Channel number */
    CSL_TsipChanCfg   chanum; 
    /** Slot number */
    Uint16            slotnum; 
    /** Time slot */
    CSL_TsipTimeslot  slot;
} CSL_TsipTimeslotCfg;

/** @addtogroup CSL_TSIP_DATASTRUCT
*
* @{ */
/**
 * @brief This is a sub-structure in @a CSL_TsipHwSetup. This structure is used 
 *        for configuring Transmit Clock and Frame Sync generation parameters.
 */
typedef struct CSL_TsipXclkSetup {
    /** Clock and Frame Sync source */
    CSL_TsipClkSrc        clksrc;
    /**  Data delay */
    Uint32                datd;
    /**  Output disable state */
    CSL_TsipXmtDis        outdis;
    /**  Transmit Frame sync Polarity */
    CSL_TsipFsyncp        fsyncp;
    /**  Transmit Frame Clock Polarity */
    CSL_TsipClkp          fclkp;
    /**  Transmit Data Clock Polarity */
    CSL_TsipClkp          dclkp;
    /**  Transmit Data Rate */
    CSL_TsipDataRate      drate;
    /**  Transmit Data Rate */
    CSL_TsipClkm          clkmode;
}CSL_TsipXclkSetup;


/** @addtogroup CSL_TSIP_DATASTRUCT
*
* @{ */
/**
 * @brief This is a sub-structure in @a CSL_TsipHwSetup. This structure is used 
 *        for configuring Receive Clock and Frame Sync generation parameters.
 */
typedef struct CSL_TsipRclkSetup {
    /** RCV frame sync mode */
    CSL_TsipClkSrc      clksrc;
    /**  Data delay */
    Uint32              datd;
    /**  Receive Frame sync Polarity */
    CSL_TsipFsyncp      fsyncp;
    /**  Receive Frame Clock Polarity */
    CSL_TsipClkp        fclkp;
    /**  Receive Data Clock Polarity */
    CSL_TsipClkp        dclkp;
    /**  Receive Data Rate */
    CSL_TsipDataRate    drate;
    /**  Receive Clock Mode */
    CSL_TsipClkm        clkmode;

} CSL_TsipRclkSetup;


/** @addtogroup CSL_TSIP_DATASTRUCT
*
* @{ */
/**
 * @brief This is a sub-structure in @a CSL_TsipHwSetup. This structure is used
 *        for configuring the parameters global to TSIP
 */
typedef struct CSL_TsipGblSetup {
    /** Tsip Clock Redundancy selection */
    CSL_TsipClkd      clkd;
    /** endian selection */
    CSL_TsipEndian    endian;
    /** DMA priority */
    CSL_TsipPri       pri;
    /** Maximum DMA priority */
    CSL_TsipPri       maxpri;

} CSL_TsipGblSetup;
/**
@} */

/**
 * @brief This structure is used for obtaining the error information for a 
 *        particular channel. Both the receive and transmit errors are logged 
 *        in the same fifo
 */
typedef struct CSL_TsipErrInfo {
    /** Tsip Clock Redundancy selection */
    CSL_TsipChanum    chanum;
    /** Tsip Error Queue Overflow */
    Uint32            errqov;
    /** Tsip Error Count */
    Uint32            errcnt;
    /** Tsip Error Code */
    Uint32            errcode;
    /** Tsip Channel Error Queue Register information */
    Uint32            info;

} CSL_TsipErrInfo;
/**
@} */


/**
 * @brief This structure is used for obtaining the status of a channel.
 */
typedef struct CSL_TsipChanstat {
    /** Tsip channel number */
    CSL_TsipChanum    chanum;
    /** Tsip channel status */
    CSL_TsipChst      stat;

} CSL_TsipChanstat;
/**
@} */


/** @addtogroup CSL_TSIP_DATASTRUCT
*
* @{ */
/**
 * @brief This is a sub-structure in @a CSL_TsipHwSetup. This structure is used 
 *        for configuring the parameters global to TSIP
 */
typedef struct CSL_TsipFrameSetup {
    /** Number of frames in a superframe */
    CSL_TsipFramecount    fcount;
    /** Frame Size */
    CSL_TsipFramesize     fsize;

} CSL_TsipFrameSetup;
/**
@} */


/** @addtogroup CSL_TSIP_DATASTRUCT
*
* @{ */
/**
 * @brief This is a sub-structure in @a CSL_TsipHwSetup. This structure is used 
 *        for configuring the parameters global to TSIP
 */
typedef struct CSL_TsipIntSetup {
    /** Interrupt selection for Super frame */
    CSL_TsipInt    sfint;

    /** Interrupt selection for Frame Transfer */
    CSL_TsipInt    frint;

    /** Delay for the interrupt */
    CSL_TsipInt    fdelay;
} CSL_TsipIntSetup;
/**
@} */


/** @addtogroup CSL_TSIP_DATASTRUCT
*
* @{ */
/**
 * @brief This is a sub-structure in @a CSL_TsipHwSetup. This structure is used 
 *        for configuring the parameters of a channel configuration
 */
typedef struct CSL_TsipChanSetup {
    /** Memory base Address */
    Uint32    baseaddr;

    /** Frame Allocation Register */
    Uint32    falloc;

    /** Frame Size Register */
    Uint32    fsize;

    /** Frame Count Register */
    Uint32    fcount;

    /** Assign a filled up bitmap */
    Uint32    *bitmap;
} CSL_TsipChanSetup;
/**
@} */


/** @addtogroup CSL_TSIP_DATASTRUCT
*
* @{ */
/**
 * @brief This is the Setup structure for configuring TSIP using  
 *        @a CSL_tsipHwSetup() function.
 */
typedef struct CSL_TsipHwSetup {
    /** Global configuration parameters */
    CSL_TsipGblSetup     *gbl;

    /** Xmt Clock setup */
    CSL_TsipXclkSetup    *xclk;

    /** Rcv Clock setup*/
    CSL_TsipRclkSetup    *rclk;

    /** Transmit Frame Setup */
    CSL_TsipFrameSetup   *xframe;

    /** Receive Frame Setup */
    CSL_TsipFrameSetup   *rframe;

    /** Receive Interrupt Setup */
    CSL_TsipIntSetup     *xint;

    /** Receive Interrupt Setup */
    CSL_TsipIntSetup     *rint;

    /** Xmt Channel Configuration A */
    CSL_TsipChanSetup    *xchana;

    /** Xmt Channel Configuration B */
    CSL_TsipChanSetup    *xchanb;

    /** Rcv Channel Configuration A */
    CSL_TsipChanSetup    *rchana;

    /** Rcv Channel Configuration B */
    CSL_TsipChanSetup    *rchanb;

    /** Channel Number */
    Uint32*              ChannelNumber;

    /** Any extra parameters, for future use */
    void*                extendSetup;
} CSL_TsipHwSetup;
/**
@} */


/**
 * @brief This structure/object holds the context of the instance of TSIP
 * opened using @a CSL_tsipOpen() function.
 *
 * Pointer to this object is passed as TSIP Handle to all TSIP CSL APIs.
 * @a CSL_tsipOpen() function intializes this structure based on the parameters
 * passed */
typedef struct CSL_TsipObj {
    /** Pointer to the register overlay structure of the TSIP */
    CSL_TsipRegsOvly    regs;

    /** Instance of TSIP being referred by this object  */
    CSL_InstNum         perNum;
} CSL_TsipObj;


/** @brief This is a pointer to @a CSL_TsipObj and is passed as the first
 * parameter to all TSIP CSL APIs
 */
typedef struct CSL_TsipObj *CSL_TsipHandle;
/**
@} */

/* Default Values */
/**  Global setup default values */
#define CSL_TSIP_GBLSETUP_DEFAULTS { \
CSL_TSIP_CLKD_REDUN,   \
CSL_TSIP_ENDIAN_LITTLE, \
CSL_TSIP_PRI_0, \
CSL_TSIP_PRI_7  \
}

/**  Transmit clock setup default values */
#define CSL_TSIP_XCLK_SETUP_DEFAULTS { \
CSL_TSIP_CLKSRC_A, \
0, \
CSL_TSIP_XMTDIS_HIGHIMP, \
CSL_TSIP_FSYNCP_ALOW, \
CSL_TSIP_CLKP_RISING, \
CSL_TSIP_CLKP_RISING, \
CSL_TSIP_DATARATE_8M, \
CSL_TSIP_CLKM_DBL \
}

/**  Receive clock setup default values */
#define CSL_TSIP_RCLK_SETUP_DEFAULTS { \
CSL_TSIP_CLKSRC_A, \
0, \
CSL_TSIP_FSYNCP_ALOW, \
CSL_TSIP_CLKP_RISING, \
CSL_TSIP_CLKP_RISING, \
CSL_TSIP_DATARATE_8M, \
CSL_TSIP_CLKM_DBL \
}

#define CSL_TSIP_HWSETUP_DEFAULTS { \
CSL_TSIP_GBLSETUP_DEFAULTS, \
CSL_TSIP_XCLK_SETUP_DEFAULTS, \
CSL_TSIP_RCLK_SETUP_DEFAULTS, \
{ CSL_TSIP_FRAMECOUNT_1, CSL_TSIP_FRAMESIZE_1 }, \
{ CSL_TSIP_FRAMECOUNT_1, CSL_TSIP_FRAMESIZE_1 }, \
{ CSL_TSIP_INT_ACK, CSL_TSIP_INT_ACK, CSL_TSIP_INT_ACK }, \
{ CSL_TSIP_INT_ACK, CSL_TSIP_INT_ACK, CSL_TSIP_INT_ACK }, \
      { /* TX-Ch-A */ \
      0x00000000, /* Memory base Address */ \
      0x0,  /* Frame Allocation Register */ \
      0x0,  /* Frame Size Register */ \
      0x0, /* Frame Count Register */ \
      0x0 /* Assign a filled up bitmap */ \
    }, \
    { /* TX-Ch-B */ \
      0x00000000, /* Memory base Address */ \
      0x0,  /* Frame Allocation Register */ \
      0x0,  /* Frame Size Register */ \
      0x0, /* Frame Count Register */ \
      0x0 /* Assign a filled up bitmap */ \
    }, \
    { /* RX-Ch-A */ \
      0x00000000, /* Memory base Address */ \
      0x0,  /* Frame Allocation Register */ \
      0x0,  /* Frame Size Register */ \
      0x0, /* Frame Count Register */ \
      0x0 /* Assign a filled up bitmap */ \
    }, \
    { /* RX-Ch-B */ \
      0x00000000, /* Memory base Address */ \
      0x0,  /* Frame Allocation Register */ \
      0x0,  /* Frame Size Register */ \
      0x0, /* Frame Count Register */ \
      0x0 /* Assign a filled up bitmap */ \
    }, \
    0x0,  /* Rcv Channel Configuration B*/ \
    0x0 \
}


/* ************************************************************************\
 * TSIP global function declarations
\* ************************************************************************/

/** ============================================================================
 *   @n@b CSL_tsipInit
 *
 *   @b Description
 *   @n This is the initialization function for the TSIP CSL. The function  
 *      must be called before calling any other API from this CSL. This function 
 *      does not modify any registers or check status. It returns status  
 *      CSL_SOK. It has been kept for future use.
 *
 *   @b Arguments
     @verbatim
         pContext   Pointer to module-context.  As TSIP doesn't have
                    any context based information user is expected to pass NULL.

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK - Always returns
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  The CSL for TSIP is initialized 
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim
        CSL_Status status;
        ...
   
        status = CSL_tsipInit(NULL);
   
     @endverbatim
 *  ===========================================================================
 */
CSL_Status  CSL_tsipInit (
    CSL_TsipContext    *pContext
);

/** ============================================================================
 *   @n@b CSL_tsipOpen
 *
 *   @b Description
 *   @n This function returns the handle to the HPI controller instance.
 *      This handle is passed to all other CSL APIs.
 *
 *   @b Arguments
 *   @verbatim            
            pTsipObj        Pointer to the object that holds reference to the
                            instance of HPI requested after the call.
                            Memory for this object should be allocated by the 
                            user
 
            tsipNum         Instance of TSIP to be opened
 
            pTsipParam      Module specific parameters.
 
            pStatus         Pointer to the variable that holds the status of the 
                            open call

     @endverbatim
 *
 *   <b> Return Value </b>  
 *      CSL_TsipHandle
 *        CSL_TsipHandle to the requested instance of TSIP if the call is 
 *        successful, otherwise @a NULL is returned.
 *
 *   <b> Pre Condition </b>
 *   @n  @a CSL_tsipInit() must be called before calling this function.
 *       Memory for the @a CSL_TsipObj must be allocated outside this call. This 
 *       object must be retained while using this peripheral instance.
 *
 *   <b> Post Condition </b>
 *   @n   1.    The status is returned in the status variable. If status
 *              returned is
 *   @li            CSL_SOK            - Valid TSIP handle is returned
 *   @li            CSL_ESYS_FAIL      - The TSIP instance is invalid
 *   @li            CSL_ESYS_INVPARAMS - Invalid parameter
 *
 *        2.    TSIP object structure is populated
 *
 *   @b  Modifies
 *   @n  None
 *
 *   @b Example:
 *   @verbatim

        CSL_TsipHandle  hTsip;
        CSL_TsipObj     tsipObj;
        CSL_Status      status;
        ...
        hTsip = CSL_tsipOpen(&tsipObj, CSL_TSIP_0, NULL, &status);
        ...

     @endverbatim
 *
 * ===========================================================================
 */
CSL_TsipHandle CSL_tsipOpen (
    CSL_TsipObj      *hTsipObj,
    CSL_InstNum      tsipNum,
    CSL_TsipParam    *pTsipParam,
    CSL_Status       *pStatus
);

/** ============================================================================
 *   @n@b CSL_tsipClose
 *
 *   @b Description
 *   @n Unreserves the TSIP identified by the handle passed.
 *
 *   @b Arguments
 *   @verbatim
            hTsip            Handle to the TSIP

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Close Successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_tsipInit() and CSL_tsipOpen() must be called successfully 
 *       in order before calling CSL_tsipClose(). 
 * 
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n TSIP Handle
 *
 *   @b Example
 *   @verbatim
            CSL_Status       status;
            CSL_tsipHandle   hTsip;

            ...

            status = CSL_tsipClose(hTsip);

            ...

     @endverbatim
 *  ============================================================================
 */
CSL_Status  CSL_tsipClose (
    CSL_TsipHandle    hTsip
);

/** ============================================================================
 *   @n@b CSL_tsipHwSetup
 *
 *   @b Description
 *   @n Configures the TSIP using the values passed in the setup structure.
 *
 *   @b Arguments
 *   @verbatim
            hTsip           TSIP handle returned by successful 'open'

            setup           Pointer to the intialized setup structure 

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - HwSetup Successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_tsipInit() and CSL_tsipOpen() must be called successfully 
 *       in order before calling CSL_tsipHwSetup(). The setup structure
 *       consists of the logically grouped sub-structures. The main setup 
 *       structure as well as these sub-structures must be filled up before 
 *       calling this function
 *
 *   <b> Post Condition </b>
 *   @n The registers of the specified TSIP instance will be setup
 *      according to value passed.
 *
 *   @b Modifies
 *   @n Hardware registers of the specified TSIP instance.
 *
 *   @b Example
 * @verbatim
        CSL_tsipHandle hTsip;
        CSL_TsipHwSetup hwSetup = CSL_TSIP_HWSETUP_DEFAULTS;
        ...
 
        // Init Successfully done
        ...
        // Open Successfully done
        ...
        CSL_tsipHwSetup(hTsip, &hwSetup);
        ...
  @endverbatim
 * =============================================================================
 */
CSL_Status  CSL_tsipHwSetup (
    CSL_TsipHandle     hTsip,
    CSL_TsipHwSetup    *setup
);

/** ============================================================================
 *   @n@b CSL_tsipHwControl
 *
 *   @b Description
 *   @n This function takes an input control command with an optional argument
 *      and accordingly controls the operation/configuration of TSIP.
 *
 *   @b Arguments
 *   @verbatim
            hTsip       TSIP handle returned by successful 'open'

            cmd         The command to this API indicates the action to be
                        taken on TSIP.
                        Control command, refer @a CSL_TsipControlCmd for the 
                        list of commands supported

            arg         An optional argument.
                        Optional argument as per the control command, 
                        @a void * casted

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li         CSL_SOK               - Status info return successful.
 *   @li         CSL_ESYS_BADHANDLE    - Invalid handle
 *   @li         CSL_ESYS_INVCMD       - Invalid command
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_tsipInit() and CSL_tsipOpen() must be called successfully 
 *       in order before calling CSL_tsipHwControl(). 
 *
 *       Refer to @a CSL_TsipHwControlCmd for the argument type (@a void*) that 
 *       needs to be passed with the control command
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n The hardware registers of TSIP.
 *
 *   @b Example
 *   @verbatim
      CSL_Status status;
      CSL_BitMask16 ctrlMask;
      CSL_TsipHandle hTsip;
      ...
      // TSIP object defined and HwSetup structure defined and initialized
      ...
     
      // Init successfully done
      ...
      // Open successfully done
      ...
      // HwSetup sucessfully done
      ...
     
      ctrlMask = 1;
      status = CSL_tsipHwControl(hTsip,
                                 CSL_TSIP_CMD_SET_PRI,
                                 &ctrlMask);

     @endverbatim
 * =============================================================================
 */
CSL_Status  CSL_tsipHwControl (
    CSL_TsipHandle        hTsip,
    CSL_TsipControlCmd    cmd,
    void*                 arg
);

/** ============================================================================
 *   @n@b CSL_tsipGetHwStatus
 *
 *   @b Description
 *   @n Gets the status of different operations or some setup-parameters of 
 *      TSIP. The status is returned through the third parameter.
 *
 *   @b Arguments
 *   @verbatim
            hTsip     TSIP handle returned by successful 'open'

            query     The query to this API of TSIP which indicates the
                      status to be returned. 

                      Query command, refer @a CSL_TsipHwStatusQuery for the list 
                      of query commands supported 

            response  Response from the query. Pointer to appropriate object 
                      corresponding to the query command needs to be passed here

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Status info return successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li                    CSL_ESYS_INVQUERY   - Invalid query command
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_tsipInit() and CSL_tsipOpen() must be called successfully 
 *       in order before calling CSL_tsipGetHwStatus(). 
 *       Refer to @a CSL_TsipHwStatusQuery for the argument to be passed along 
 *       with the corresponding query command.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n Modifies third parameter response.
 *
 *   @b Example
 *   @verbatim
        CSL_TsipHandle hTsip;
        CSL_Status status;
        CSL_TsipPid response;
        ...
        status = CSL_tsipGetHwStatus(hTsip,
                                     CSL_TSIP_QUERY_PID,
                                     &response);
        ...

     @endverbatim
 * =============================================================================
 */
CSL_Status  CSL_tsipGetHwStatus (
    CSL_TsipHandle           hTsip,
    CSL_TsipHwStatusQuery    query,
    void*                    response
);

/** ============================================================================
 *   @n@b CSL_tsipGetHwSetup
 *
 *   @b Description
 *   @n Gets the status of some or all of the setup-parameters of TSIP.
 *      To get the status of complete TSIP h/w setup, all the sub-structure
 *      pointers inside the main HwSetup structure, should be non-NULL. 
 *
 *   @b Arguments
 *   @verbatim
            hTsip           TSIP handle returned by successful 'open'

            hwSetup         Pointer to @a CSL_TsipHwSetup structure to read the 
                            setup parameters

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Retrieving the hardware setup
 *                                                parameters is successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_tsipInit() and CSL_tsipOpen() must be called successfully 
 *       in order before calling CSL_tsipGetHwSetup(). 
 *
 *   <b> Post Condition </b>
 *   @n  The hardware setup structure is populated with the hardware setup
 *       parameters
 *
 *   @b Modifies
 *   @n hwSetup variable
 *
 *   @b Example
 *   @verbatim
        CSL_TsipHandle hTsip;
        CSL_Status status;

        CSL_TsipGblSetup gblSetup;
        CSL_TsipFrameSetup frameSetup;
        CSL_TsipClkSetup clkSetup;
        CSL_TsipEmu emuMode;
        CSL_TsipHwSetup readSetup;
        ...
        status = CSL_tsipGetHwSetup(hTsip, &readSetup);
   

     @endverbatim
 * ===========================================================================
 */
CSL_Status  CSL_tsipGetHwSetup (
    CSL_TsipHandle     hTsip,
    CSL_TsipHwSetup    *hwSetup
);

/** ============================================================================
 *   @n@b CSL_tsipChHwSetup
 *
 *   @b Description
 *   @n Configures the TSIP using the values passed in the setup structure.
 *
 *   @b Arguments
 *   @verbatim
            hTsip            TSIP handle returned by successful 'open'

            setup            Pointer to the intialized setup structure 

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Tsip Channel setup Successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_tsipInit() and CSL_tsipOpen() must be called successfully 
 *       in order before calling CSL_tsipChHwSetup(). 
 *
 *   <b> Post Condition </b>
 *   @n  TSIP registers are configured according to the hardware setup 
 *       parameters.
 *
 *   @b Modifies
 *   @n TSIP registers
 *
 *   @b Example
 *   @verbatim
     CSL_tsipHandle hTsip;
     CSL_TsipChHwSetup hwSetup = CSL_TSIP_HWSETUP_DEFAULTS;
     ...

     // Init Successfully done
     ...
     // Open Successfully done
     ...
     CSL_tsipChHwSetup(hTsip, &hwSetup);
     ...

     @endverbatim
 *  ============================================================================
 */
CSL_Status  CSL_tsipChHwSetup (
    CSL_TsipHandle     hTsip,
    CSL_TsipHwSetup    *setup
);

/** ============================================================================
 *   @n@b CSL_tsipGetBaseAddress
 *
 *   @b Description
 *   @n  Function to get the base address of the peripheral instance.
 *       This function is used for getting the base address of the peripheral
 *       instance. This function will be called inside the CSL_tsipOpen()
 *       function call. This function is open for re-implementing if the user
 *       wants to modify the base address of the peripheral object to point to
 *       a different location and there by allow CSL initiated write/reads into
 *       peripheral. MMR's go to an alternate location.
 *
 *   @b Arguments
 *   @verbatim
            tsipNum        Specifies the instance of the TSIP to be opened.

            pTsipParam     Module specific parameters.

            pBaseAddress  Pointer to baseaddress structure containing base
                          address details.

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_OK            Open call is successful
 *   @li                    CSL_ESYS_FAIL     The instance number is invalid.
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Base Address structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *
 *         2. Base address structure is modified.
 *
 *   @b Example
 *   @verbatim
        CSL_Status          status;
        CSL_TsipBaseAddress  baseAddress;

       ...
      status = CSL_tsipGetBaseAddress(CSL_TSIP, NULL, &baseAddress);

    @endverbatim
 * ===========================================================================
 */
CSL_Status   CSL_tsipGetBaseAddress (
        CSL_InstNum              tsipNum,
        CSL_TsipParam           *pTsipParam,
        CSL_TsipBaseAddress     *pBaseAddress
);


#ifdef __cplusplus
}
#endif

#endif

