/* =============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ============================================================================
 *   @file  csl_tsipAux.h
 *
 *   @path  $(CSLPATH)\inc
 *
 *   @desc  Header file for functional layer of CSL
 *    - This decribes the definitions of the inline functions
 *
 */
/* =============================================================================
 *  Revision History
 *  ===============
 *  23-Mar-2006 NG  File taken from TI code base and modified for c6486.
 *                  Updated according coding guidelines (doxygen compliant)
 *  
 *  10-Nov-2006 KNG File is Updated to add a function CSL_tsipGetChcfg().
 *
 *
 * =============================================================================
 */
#ifndef _CSL_TSIPAUX_H_
#define _CSL_TSIPAUX_H_

#include <csl_tsip.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Status Query Functions of tsip. */

/** ============================================================================
 *   @n@b CSL_tsipGetXfrfc
 *
 *   @b Description
 *   @n This function gets the current Transmit Free Running Frame Count 
 *
 *   @b Arguments
 *   @verbatim

            hTsip           Handle to TSIP instance

            response        Placeholder to return status.

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Tsip must be initialized and opened properly
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_tsipGetXfrfc(hTsip, response);

     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_tsipGetXfrfc (
    CSL_TsipHandle    hTsip,
    void              *response
)
{
    *(Uint32 *)response = (CSL_FEXT( hTsip->regs->XFRFC, TSIP_XFRFC_FRFC));
}

/** ============================================================================
 *   @n@b CSL_tsipGetRfrfc
 *
 *   @b Description
 *   @n This function gets the current Receive Free Running Frame Count 
 *
 *   @b Arguments
 *   @verbatim

            hTsip          Handle to TSIP instance

            response       Placeholder to return the status. @a void* casted

     @endverbatim
 *
 *   <b> Return Value </b>
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Tsip must be initialized and opened properly
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_tsipGetRfrfc(hTsip, response);

     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_tsipGetRfrfc (
    CSL_TsipHandle    hTsip,
    void*             response
)
{
    *(Uint32 *)response = (CSL_FEXT( hTsip->regs->RFRFC, TSIP_RFRFC_FRFC));
}

/** ============================================================================
 *   @n@b CSL_tsipGetXbitmap
 *
 *   @b Description
 *   @n This function gets the status of the channel bitmap, 
 *      inactive or a or b
 *
 *   @b Arguments
 *   @verbatim

            hTsip          Handle to TSIP instance

            response       Placeholder to return the status. @a void* casted

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Tsip must be initialized and opened properly 
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_tsipGetBmst(hTsip, response);

     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_tsipGetBmst (
    CSL_TsipHandle    hTsip,
    void*             response
)
{
    Uint32 chanum;
    chanum = ((CSL_TsipChanstat *)response)->chanum;
    if (chanum & CSL_TSIP_RCV_CHAN) {
        ((CSL_TsipChanstat *)response)->stat = 
            (CSL_TsipChst)((hTsip->regs->RBMST >> (chanum & 0x7)) & 0x3);
    }
    else {
        ((CSL_TsipChanstat *)response)->stat = 
            (CSL_TsipChst)((hTsip->regs->XBMST >> (chanum & 0x7)) & 0x3);
    }
}

/** ============================================================================
 *   @n@b CSL_tsipGetChst
 *
 *   @b Description
 *   @n This function gets the status of the Configuration ...
        inactive or a or b
 *
 *   @b Arguments
 *   @verbatim
            hTsip          Handle to TSIP instance
            
            response       Placeholder to return the status. @a void* casted
     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Tsip must be initialized and opened properly 
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_tsipGetCfgst(hTsip, response);

     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_tsipGetChst (
    CSL_TsipHandle    hTsip,
    void*             response
)
{
    Uint32 chanum;
    chanum = ((CSL_TsipChanstat *)response)->chanum;
    if (chanum & CSL_TSIP_RCV_CHAN) {
        ((CSL_TsipChanstat *)response)->stat = 
            (CSL_TsipChst)((hTsip->regs->RCHST >> (2*(chanum & 0x7))) & 0x3);
    }
    else {
        ((CSL_TsipChanstat *)response)->stat = 
            (CSL_TsipChst)((hTsip->regs->XCHST >> (2*(chanum & 0x7))) & 0x3);
    }
}


/** ============================================================================
 *   @n@b CSL_tsipGetChcfg
 *
 *   @b Description
 *   @n This function gets the Configuration of the channel...
        configurations contains baseaddress of the bitmap, frame size,
        frame allocation, frame count.
 *
 *   @b Arguments
 *   @verbatim
            hTsip          Handle to TSIP instance
            
            response       Placeholder to return the status. @a void* casted
     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Tsip must be initialized and opened properly 
 *
 *   <b> Post Condition </b>
 *    @n The place holder 'response' will be populated
 *
 *   @b Modifies
 *   @n  The parameter 'response' will be modified.
 *
 *   @b Example
 *   @verbatim
        CSL_tsipGetChcfg(hTsip, response);

     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_tsipGetChcfg(
    CSL_TsipHandle    hTsip,
    void*             response
)
{
    Uint32 chanNo;
    chanNo = ((CSL_TsipChanstat *)response)->chanum;
    
    /* Recieve channel configuration */
    if (chanNo & CSL_TSIP_RCV_CHAN){
        ((CSL_TsipDrchRegs *)response)->ABASE = 
                             hTsip->regs->DRCH[(chanNo & 0x7)].ABASE;
        ((CSL_TsipDrchRegs *)response)->AFALLOC =
                             hTsip->regs->DRCH[(chanNo & 0x7)].AFALLOC;
        ((CSL_TsipDrchRegs *)response)->AFSIZE = 
                             hTsip->regs->DRCH[(chanNo & 0x7)].AFSIZE;
        ((CSL_TsipDrchRegs *)response)->AFCNT = 
                             hTsip->regs->DRCH[(chanNo & 0x7)].AFCNT;
        ((CSL_TsipDrchRegs *)response)->BBASE =
                             hTsip->regs->DRCH[(chanNo & 0x7)].BBASE;
        ((CSL_TsipDrchRegs *)response)->BBASE =
                             hTsip->regs->DRCH[(chanNo & 0x7)].BFALLOC;
        ((CSL_TsipDrchRegs *)response)->BBASE =
                             hTsip->regs->DRCH[(chanNo & 0x7)].BFSIZE;
        ((CSL_TsipDrchRegs *)response)->BBASE =
                             hTsip->regs->DRCH[(chanNo & 0x7)].BFCNT;
    }
    /* Transmit channel Configuration */
    else {
        ((CSL_TsipDxchRegs *)response)->ABASE = 
                             hTsip->regs->DXCH[(chanNo & 0x7)].ABASE;
        ((CSL_TsipDxchRegs *)response)->AFALLOC =
                             hTsip->regs->DXCH[(chanNo & 0x7)].AFALLOC;
        ((CSL_TsipDxchRegs *)response)->AFSIZE = 
                             hTsip->regs->DXCH[(chanNo & 0x7)].AFSIZE;
        ((CSL_TsipDxchRegs *)response)->AFCNT = 
                             hTsip->regs->DXCH[(chanNo & 0x7)].AFCNT;
        ((CSL_TsipDxchRegs *)response)->BBASE =
                             hTsip->regs->DXCH[(chanNo & 0x7)].BBASE;
        ((CSL_TsipDxchRegs *)response)->BBASE =
                             hTsip->regs->DXCH[(chanNo & 0x7)].BFALLOC;
        ((CSL_TsipDxchRegs *)response)->BBASE =
                             hTsip->regs->DXCH[(chanNo & 0x7)].BFSIZE;
        ((CSL_TsipDxchRegs *)response)->BBASE =
                             hTsip->regs->DXCH[(chanNo & 0x7)].BFCNT;
    }
}

/** ============================================================================
 *   @n@b CSL_tsipGetErrInfo
 *
 *   @b Description
 *   @n This function gets the error information for a particular channel
 *
 *   @b Arguments
 *   @verbatim

            hTsip          Handle to TSIP instance

            response       Placeholder to return the status. @a void* casted

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Tsip must be initialized and opened properly 
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

        CSL_tsipGetErrInfo(hTsip, response);

     @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE
void CSL_tsipGetErrInfo (
    CSL_TsipHandle    hTsip,
    void              *response
)
{
    Uint32 chanum = ((CSL_TsipErrInfo *)response)->chanum;
    ((CSL_TsipErrInfo *)response)->errqov = 
        CSL_FEXT(hTsip->regs->ERR[chanum].ERRCNT, TSIP_ERRCNT_EQOV);
    ((CSL_TsipErrInfo *)response)->errcnt = 
        CSL_FEXT(hTsip->regs->ERR[chanum].ERRCNT, TSIP_ERRCNT_ECNT);
    ((CSL_TsipErrInfo *)response)->errcode = 
        CSL_FEXT(hTsip->regs->ERR[chanum].ERRQ, TSIP_ERRQ_ERRC);
    ((CSL_TsipErrInfo *)response)->info = 
        CSL_FEXT(hTsip->regs->ERR[chanum].ERRQ, TSIP_ERRQ_INFO);
}


/** ============================================================================
 *   @n@b CSL_tsipGetPid 
 *
 *   @b Description
 *   @n This function returns the PID of TSIP module.
 *
 *   @b Arguments
 *   @verbatim

            hTsip          Handle to TSIP instance

            response       Placeholder to return the status. @a void* casted

     @endverbatim
 *
 *   <b> Return Value </b>  
 *	 @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  Tsip must be initialized and opened properly 
 *
 *   <b> Post Condition </b>
 *    @n None
 *
 *   @b Modifies
 *   @n  None
 *
 *   @b Example
 *   @verbatim

       CSL_tsipGetPid(hTsip, response);

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE
void CSL_tsipGetPid (
    CSL_TsipHandle    hTsip,
    void              *response
)
{
    ((CSL_TsipPerId *)response)->modid = CSL_FEXT(hTsip->regs->PID, 
                                                  TSIP_PID_MODID); 
    ((CSL_TsipPerId *)response)->major = CSL_FEXT(hTsip->regs->PID, 
                                                  TSIP_PID_MAJOR);
    ((CSL_TsipPerId *)response)->minor = CSL_FEXT(hTsip->regs->PID, 
                                                  TSIP_PID_MINOR);
}


#ifdef __cplusplus
extern "C" {
#endif

#endif /* CSL_TSIPAUX_H_ */

