/* =============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ============================================================================
 *   @file  csl_utopia2.h
 *
 *   @path  $(CSLPATH)\inc
 *
 *   @desc  UTOPIA2 functional layer APIs header file
 *
 */
/** ============================================================================
 *  @mainpage UTOPIA2 CSL 3.x
 *
 *  @section Introduction
 *
 *  @subsection xxx Purpose and Scope
 *  The purpose of this document is to identify a set of common CSL APIs for
 *  the UTOPIA2 module across various devices. The CSL developer is expected to
 *  refer to this document while designing APIs for this module. Some of the
 *  listed APIs may not be applicable to a given UTOPIA2 module. While other 
 *  cases this list of APIs may not be sufficient to cover all the features of a
 *  particular UTOPIA2 Module. The CSL developer should use his discretion 
 *  designing new APIs or extending the existing ones to cover these.
 *
 *  @subsection aaa Terms and Abbreviations
 *   -# CSL:  Chip Support Library
 *   -# API:  Application Programmer Interface
 *
 *  @subsection References
 *    -# ul2_feature_spec.pdf - Universal Test and Operations Interface for ATM 
 *                              (UTOPIA2) Peripheral Requirements 
 */

/* =============================================================================
 *  Revision History
 *  ===============
 *  19-Jan-2006 NG  File taken from TI code base and modified for c6486.
 *                  Updated according to coding guidelines (doxygen compliant)
 *
 *  09-Feb-2006 NG  Removed enums CSL_Utopia2RecMstrStatus, 
 *                  CSL_Utopia2XmtMstrStatus, CSL_Utopia2LoopBack, 
 *                  CSL_Utopia2GetIntrStatus, CSL_Utopia2GetIntrPendingStatus;
 *
 * =============================================================================
 */
#ifndef _CSL_UTOPIA2_H_
#define _CSL_UTOPIA2_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <csl.h>
#include <cslr_utopia2.h>

#include <cslr.h>
#include <soc.h>
#include <csl_error.h>
#include <csl_types.h>

/*---- Global macro defns for Errors ----*/
/** Command for Utopia2 Transmitter */ 
#define CSL_UTOPIA2_XMT_CMD    (1)
/** Command for Utopia2 Receiver */ 
#define CSL_UTOPIA2_REC_CMD    (2)

/* typedef enumerations for Utopia2 */ 

/** 
 * @brief Enumeration for commands passed to @a CSL_utopia2HwControl()
 *
 * This is used to select the commands to control the operations
 * existing setup of UTOPIA2. The arguments to be passed with each
 * enumeration if any are specified next to the enumeration.
 */
typedef enum {
    /** Command to select Utopia2 Error Clear Configuration */
	CSL_UTOPIA2_CMD_ERROR_CLEAR = 0,
    /** Command to select Utopia2 Error Enable Configuration */
	CSL_UTOPIA2_CMD_ERROR_ENABLE = 1,
    /** Command to select Utopia2 Error Disable Configuration */
	CSL_UTOPIA2_CMD_ERROR_DISABLE = 2,
    /** Command to select Utopia2 Interrupt Clear Configuration */
	CSL_UTOPIA2_CMD_INTERUPT_CLEAR = 3,
    /** Command to select Utopia2 Interrupt Enable Configuration */
	CSL_UTOPIA2_CMD_INTERUPT_ENABLE = 4,
    /** Command to select Utopia2 Interrupt Disable Configuration */
	CSL_UTOPIA2_CMD_INTERUPT_DISABLE = 5,
    /** Command to enable Utopia2 Transmitter  */
	CSL_UTOPIA2_CMD_XMT_ENABLE = 6,
    /** Command to disable Utopia2 Transmitter  */
	CSL_UTOPIA2_CMD_XMT_DISABLE = 7,
    /** Command to enable Utopia2 Receiver  */
	CSL_UTOPIA2_CMD_REC_ENABLE = 8,
    /** Command to disable Utopia2 Receiver  */
	CSL_UTOPIA2_CMD_REC_DISABLE = 9,

    /** Command to Utopia2 Reset */
	CSL_UTOPIA2_CMD_RESET = 10,
    /** Command to set PHY Disable Register */
	CSL_UTOPIA2_CMD_SET_PHY = 11
} CSL_Utopia2ControlCmd;			

/** 
 * @brief Enumeration for queries passed to @a CSL_utopia2GetHwStatus()
 *
 * This is used to get the status of different operations 
 */
typedef enum {
    /** Query to get Utopia2 Error Status */
	CSL_UTOPIA2_QUERY_ERR_STATUS = 0,
    /** Query to get Utopia2 Error Pending Status */		 
	CSL_UTOPIA2_QUERY_ERR_PENDING_STATUS = 1,
    /** Query to get Utopia2 Interrupt Status */
	CSL_UTOPIA2_QUERY_INTR_STATUS = 2,
    /** Query to get Utopia2 Interrupt pending Status */		
	CSL_UTOPIA2_QUERY_INTR_PENDING_STATUS = 3,
    /** Query to get Utopia2 Transmitter Status */
	CSL_UTOPIA2_QUERY_XMT_STATUS = 4,
    /** Query to get Utopia2 Receiver Status */
	CSL_UTOPIA2_QUERY_REC_STATUS = 5,
    /** Query to get Utopia2 Reset Status */
	CSL_UTOPIA2_QUERY_RESET_STATUS = 6,
    /** Query to get Transmit Cell Available for PHY Register Status */
	CSL_UTOPIA2_QUERY_TX_CELL_AVAIL_STATUS = 7,
    /** Query to get PHY Disable Register Status */
	CSL_UTOPIA2_QUERY_PHY_STATUS = 8
} CSL_Utopia2HwStatusQuery;

/** 
 * @brief Utopia2 Receiver Control enumeration
 */
typedef enum {
    /** Utopia2 Receiver Disable */
    CSL_UTOPIA2_UREN_DISABLE = 0,
    /** Utopia2 Receiver Enable */
    CSL_UTOPIA2_UREN_ENABLE = 1
} CSL_Utopia2UrenStatus;            

/** 
 * @brief Utopia2 transmitter Control enumeration
 */
typedef enum {
    /** Utopia2 Transmitter Disable */
	CSL_UTOPIA2_UXEN_DISABLE = 0,
    /** Utopia2 Transmitter Enable */
    CSL_UTOPIA2_UXEN_ENABLE = 1
} CSL_Utopia2UxenStatus; 

/** @brief Utopia2 Multi - PHY Control Enumeration */
typedef enum {
    /** Utopia2 Multi - PHY Disable */
	CSL_UTOPIA2_MPHY_DISABLE = 0,
    /** Utopia2 Multi - PHY Enable */
	CSL_UTOPIA2_MPHY_ENABLE = 1
} CSL_Utopia2Mphy;

/** @brief Utopia2 Bit - Mode Enumeration */
typedef enum {
    /** Utopia2 8 Bit Mode Select */
	CSL_UTOPIA2_8BIT_MODE = 0,
    /** Utopia2 16 Bit Mode Select */
	CSL_UTOPIA2_16BIT_MODE = 1
} CSL_Utopia2BitMode;

/** @brief Utopia2 polling Mode Select Enumeration */
typedef enum {
    /** Utopia2 RoundRobin Polling Mode Select */
	CSL_UTOPIA2_ROUNDROBIN_POLLING = 0,
    /** Utopia2 Fixed Polling Mode Select */
	CSL_UTOPIA2_FIXED_POLLING = 1
} CSL_Utopia2PollingMode;

/** @brief Utopia2 Endianness Select enumeration */
typedef enum {
    /** Utopia2 Little Endian Mode Select */   
	CSL_UTOPIA2_LITTLE_ENDIAN_MODE = 0,
    /** Utopia2 Big Endian Mode Select */
	CSL_UTOPIA2_BIG_ENDIAN_MODE = 1
} CSL_Utopia2EndianMode;

/** @brief Utopia2 slave Id select enumeration */
typedef enum {
    /** Utopia2 slave Id */
	CSL_UTOPIA2_SLID_SLEND = 0
} CSL_Utopia2SlidSlendMode;

/** @brief Utopia2 clock mode setup enumeration */
typedef enum {
    /** Utopia2 Receive clock mode selection */
	CSL_UTOPIA2_REC_CLOCK = 0,
    /** Utopia2 Transmit clock mode selection */
	CSL_UTOPIA2_TX_CLOCK = 1
} CSL_Utopia2SetupMode;

/** @brief Utopia2 error interrupt status enumeration */
typedef enum {
    /** Utopia2 transmit clock present status */
    CSL_UTOPIA2_TX_CLK_PRES_INTR_ENABLE = 0,
    /** Utopia2 transmit clock fail status */
	CSL_UTOPIA2_TX_CLK_FAIL_INTR_ENABLE = 1,
    /** Utopia2 transmit queue stall status */
	CSL_UTOPIA2_TX_QUEUE_STALL_INTR_ENABLE = 2,
    /** Utopia2 receive clock present status */
	CSL_UTOPIA2_RX_CLK_PRES_INTR_ENABLE = 3,
    /** Utopia2 receive clock fail status */
	CSL_UTOPIA2_RX_CLK_FAIL_INTR_ENABLE = 4,
    /** Utopia2 receive queue stall status */
	CSL_UTOPIA2_RX_QUEUE_STALL_INTR_ENABLE = 5,
	
    /** Utopia2 transmit clock present disable setup */
	CSL_UTOPIA2_TX_CLK_PRES_INTR_DISABLE = 6,
    /** Utopia2 transmit clock fail disable setup */
	CSL_UTOPIA2_TX_CLK_FAIL_INTR_DISABLE = 7,
    /** Utopia2 transmit queue stall disable setup */
	CSL_UTOPIA2_TX_QUEUE_STALL_INTR_DISABLE = 8,
    /** Utopia2 receive clock present disable setup */
	CSL_UTOPIA2_RX_CLK_PRES_INTR_DISABLE = 9,
    /** Utopia2 receive clock fail disable setup */
	CSL_UTOPIA2_RX_CLK_FAIL_INTR_DISABLE = 10,
    /** Utopia2 receive queue stall disable setup */
	CSL_UTOPIA2_RX_QUEUE_STALL_INTR_DISABLE = 11
}CSL_Utopia2GetErrorSetup;

/** @brief Utopia2 error status enumeration */
typedef enum {
    /** Utopia2 transmit clock present enable setup */
	 CSL_UTOPIA2_TX_CLK_PRES_ENABLE = 0,
    /** Utopia2 transmit clock fail enable setup */
	CSL_UTOPIA2_TX_CLK_FAIL_ENABLE = 1, 
    /** Utopia2 receive clock present enable setup */
	CSL_UTOPIA2_RX_CLK_PRES_ENABLE = 2,
    /** Utopia2 receive clock fail enable setup */
	CSL_UTOPIA2_RX_CLK_FAIL_ENABLE = 3,  
	
    /** Utopia2 transmit clock present disable setup */
	CSL_UTOPIA2_TX_CLK_PRES_DISABLE = 4,
    /** Utopia2 transmit clock fail disable setup */
	CSL_UTOPIA2_TX_CLK_FAIL_DISABLE = 5, 
    /** Utopia2 receive clock present disable setup */
	CSL_UTOPIA2_RX_CLK_PRES_DISABLE = 6,
    /** Utopia2 receive clock fail disable setup */
	CSL_UTOPIA2_RX_CLK_FAIL_DISABLE = 7
} CSL_Utopia2ErrorStatCmd;


typedef enum {
    /** Command for Utopia2 XCFE */
    CSL_UTOPIA2_CTRL_XCFE    = (0x0001),
    /** Command for Utopia2 XQSE */
    CSL_UTOPIA2_CTRL_XQSE    = (0x0002),
    /** Command for Utopia2 XCPE */
    CSL_UTOPIA2_CTRL_XCPE    = (0x0004),

    /** Command for Utopia2 RCFE */
    CSL_UTOPIA2_CTRL_RCFE    = (0x0008),
    /** Command for Utopia2 RQSE */
    CSL_UTOPIA2_CTRL_RQSE    = (0x0010),
    /** Command for Utopia2 RCPE */   
    CSL_UTOPIA2_CTRL_RCPE    = (0x0020)
} CSL_Utopia2ClkErrSts;			

typedef enum {
    /** Command for Utopia2 XCPP */                                       
    CSL_UTOPIA2_CTRL_XCPP = (0x0001),
    /** Command for Utopia2 XCFP */
    CSL_UTOPIA2_CTRL_XCFP = (0x0002),

    /** Command for Utopia2 RCPP */
    CSL_UTOPIA2_CTRL_RCPP = (0x0004),
    /** Command for Utopia2 RCFP */
    CSL_UTOPIA2_CTRL_RCFP =	(0x0008)
} CSL_Utopia2ClkPendSts;			


typedef enum {
    /*---- Global macro defns for Interrupts ----*/
    /** Command for Utopia2 XQIE0 */
    CSL_UTOPIA2_CTRL_XQIE0	= (0x0001),
    /** Command for Utopia2 XQIE1 */
    CSL_UTOPIA2_CTRL_XQIE1	= (0x0002),
    /** Command for Utopia2 XQIE2 */
    CSL_UTOPIA2_CTRL_XQIE2	= (0x0004),
    /** Command for Utopia2 XQIE3 */
    CSL_UTOPIA2_CTRL_XQIE3	= (0x0008),
    /** Command for Utopia2 XQIE4 */
    CSL_UTOPIA2_CTRL_XQIE4	= (0x0010),
    /** Command for Utopia2 XQIE5 */
    CSL_UTOPIA2_CTRL_XQIE5	= (0x0020),
    /** Command for Utopia2 XQIE6 */
    CSL_UTOPIA2_CTRL_XQIE6	= (0x0040),
    /** Command for Utopia2 XQIE7 */
    CSL_UTOPIA2_CTRL_XQIE7	= (0x0080),

    /** Command for Utopia2 RQIE0 */                              
    CSL_UTOPIA2_CTRL_RQIE0	= (0x0100),
    /** Command for Utopia2 RQIE1 */
    CSL_UTOPIA2_CTRL_RQIE1	= (0x0200),
    /** Command for Utopia2 RQIE2 */
    CSL_UTOPIA2_CTRL_RQIE2	= (0x0400),
    /** Command for Utopia2 RQIE3 */
    CSL_UTOPIA2_CTRL_RQIE3	= (0x0800),
    /** Command for Utopia2 RQIE4 */
    CSL_UTOPIA2_CTRL_RQIE4	= (0x1000),
    /** Command for Utopia2 RQIE5 */
    CSL_UTOPIA2_CTRL_RQIE5	= (0x2000),
    /** Command for Utopia2 RQIE6 */
    CSL_UTOPIA2_CTRL_RQIE6	= (0x4000),
    /** Command for Utopia2 RQIE7 */
    CSL_UTOPIA2_CTRL_RQIE7	= (0x8000)

} CSL_Utopia2IntrErrSts;			

typedef enum {
    /** Command for Utopia2 XQIP0 */
    CSL_UTOPIA2_CTRL_XQIP0	= (0x0001),
    /** Command for Utopia2 XQIP1 */
    CSL_UTOPIA2_CTRL_XQIP1	= (0x0002),
    /** Command for Utopia2 XQIP2 */
    CSL_UTOPIA2_CTRL_XQIP2	= (0x0004),
    /** Command for Utopia2 XQIP3 */
    CSL_UTOPIA2_CTRL_XQIP3	= (0x0008),
    /** Command for Utopia2 XQIP4 */
    CSL_UTOPIA2_CTRL_XQIP4	= (0x0010),
    /** Command for Utopia2 XQIP5 */
    CSL_UTOPIA2_CTRL_XQIP5	= (0x0020),
    /** Command for Utopia2 XQIP6 */
    CSL_UTOPIA2_CTRL_XQIP6	= (0x0040),
    /** Command for Utopia2 XQIP7 */
    CSL_UTOPIA2_CTRL_XQIP7	= (0x0080),

    /** Command for Utopia2 RQIP0 */   
    CSL_UTOPIA2_CTRL_RQIP0	= (0x0100),
    /** Command for Utopia2 RQIP1 */
    CSL_UTOPIA2_CTRL_RQIP1	= (0x0200),
    /** Command for Utopia2 RQIP2 */
    CSL_UTOPIA2_CTRL_RQIP2	= (0x0400),
    /** Command for Utopia2 RQIP3 */
    CSL_UTOPIA2_CTRL_RQIP3	= (0x0800),
    /** Command for Utopia2 RQIP4 */
    CSL_UTOPIA2_CTRL_RQIP4	= (0x1000),
    /** Command for Utopia2 RQIP5 */
    CSL_UTOPIA2_CTRL_RQIP5	= (0x2000),
    /** Command for Utopia2 RQIP6 */
    CSL_UTOPIA2_CTRL_RQIP6	= (0x4000),
    /** Command for Utopia2 RQIP7 */
    CSL_UTOPIA2_CTRL_RQIP7	= (0x8000)
} CSL_Utopia2IntrPendSts;			

/** @brief Utopia2 receive user defined cell enumeration */
typedef enum {
    /** Utopia2 receive user defined cell extra bits 0*/
    CSL_UTOPIA2_RUDC_EXTRA0     = (0x0000),
    /** Utopia2 receive user defined cell extra bits 1*/
    CSL_UTOPIA2_RUDC_EXTRA1     = (0x0001),
    /** Utopia2 receive user defined cell extra bits 2*/	
    CSL_UTOPIA2_RUDC_EXTRA2     = (0x0002),
    /** Utopia2 receive user defined cell extra bits 3*/		
    CSL_UTOPIA2_RUDC_EXTRA3     = (0x0003),
    /** Utopia2 receive user defined cell extra bits 4*/
    CSL_UTOPIA2_RUDC_EXTRA4     = (0x0004),
    /** Utopia2 receive user defined cell extra bits 5*/
    CSL_UTOPIA2_RUDC_EXTRA5     = (0x0005),
    /** Utopia2 receive user defined cell extra bits 6*/
    CSL_UTOPIA2_RUDC_EXTRA6     = (0x0006),
    /** Utopia2 receive user defined cell extra bits 7*/
    CSL_UTOPIA2_RUDC_EXTRA7     = (0x0007),
    /** Utopia2 receive user defined cell extra bits 8*/
    CSL_UTOPIA2_RUDC_EXTRA8     = (0x0008),
    /** Utopia2 receive user defined cell extra bits 9*/
    CSL_UTOPIA2_RUDC_EXTRA9     = (0x0009),
    /** Utopia2 receive user defined cell extra bits 10*/
    CSL_UTOPIA2_RUDC_EXTRA10    = (0x000A),
    /** Utopia2 receive user defined cell extra bits 11*/
    CSL_UTOPIA2_RUDC_EXTRA11    = (0x000B)
} CSL_Utopia2RecUdc;

/** @brief Utopia2 transmit user defined cell enumeration */
typedef enum {
    /** Utopia2 transmit user defined cell extra bits 0*/
    CSL_UTOPIA2_XUDC_EXTRA0     = (0x0000),
    /** Utopia2 transmit user defined cell extra bits 1*/
    CSL_UTOPIA2_XUDC_EXTRA1     = (0x0001),
    /** Utopia2 transmit user defined cell extra bits 2*/	
    CSL_UTOPIA2_XUDC_EXTRA2     = (0x0001),
    /** Utopia2 transmit user defined cell extra bits 3*/		
    CSL_UTOPIA2_XUDC_EXTRA3     = (0x0003),
    /** Utopia2 transmit user defined cell extra bits 4*/
    CSL_UTOPIA2_XUDC_EXTRA4     = (0x0004),
    /** Utopia2 transmit user defined cell extra bits 5*/
    CSL_UTOPIA2_XUDC_EXTRA5     = (0x0005),
    /** Utopia2 transmit user defined cell extra bits 6*/
    CSL_UTOPIA2_XUDC_EXTRA6     = (0x0006),
    /** Utopia2 transmit user defined cell extra bits 7*/
    CSL_UTOPIA2_XUDC_EXTRA7     = (0x0007),
    /** Utopia2 transmit user defined cell extra bits 8*/
    CSL_UTOPIA2_XUDC_EXTRA8     = (0x0008),
    /** Utopia2 transmit user defined cell extra bits 9*/
    CSL_UTOPIA2_XUDC_EXTRA9     = (0x0009),
    /** Utopia2 transmit user defined cell extra bits 10*/
    CSL_UTOPIA2_XUDC_EXTRA10    = (0x000A),
    /** Utopia2 transmit user defined cell extra bits 11*/
    CSL_UTOPIA2_XUDC_EXTRA11    = (0x000B)
} CSL_Utopia2TxUdc;

/** @brief UTOPIA2 specific context information. Present implementation doesn't
 *  have any Context information.
 */
typedef struct {
    /** Context information of Utopia2.
     *  The below declaration is just a place-holder for future implementation.
     */
    Uint16 contextInfo;
} CSL_Utopia2Context;


/** @brief Utopia2 Clock Setup Command Structure */
typedef struct CSL_Utopia2ClkSetup {
    /** XCCNT Transmit Clock Count */
	Uint16	txClkCount ;
    /** RCCNT Receive Clock Count */            
	Uint16	rxClkCount;             
} CSL_Utopia2ClkSetup;

/* @brief Utopia2 Error Setup Structure */ 
typedef struct CSL_Utopia2ErrorSetup {
    /** XCPP Enable/Disable */
    Uint16	txClkPresent;            
    /** XCPP Enable/Disable */
    Uint16 	rxClkPresent;            
    /** XCFP Enable/Disable */
    Uint16 	txClkFail;               
    /** RCFP Enable/Disable */
    Uint16 	rxClkFail;               
    /** XCPE Enable/Disable */
    Uint16 	txClkPresentIntEnb;      
    /** RCPE Enable/Disable */
    Uint16 	rxClkPresentIntEnb;      
    /** XCFE Enable/Disable */
    Uint16 	txClkFailIntEnb;         
    /** RCFE Enable/Disable */
    Uint16 	rxClkFailIntEnb;         
    /** XQSE Enable/Disable */
    Uint16 	txQueueStallIntEnb;      
    /** RQSE Enable/Disable */
    Uint16 	rxQueueStallIntEnb;       
} CSL_Utopia2ErrorSetup;

/** @brief Utopia2 'Utopia2 Control Register' Setup Command Structure */
typedef struct CSL_Utopia2UcrSetup {
    /** BEND Little Endian or Big Endian */
	Uint32	Endianness;        	
    /** UXEN UTOPIA2 Trasmit port Enable */
	Uint16  txUtopia2Enable;   	
    /** UREN UTOPIA2 Receive Enable */
	Uint16  rxUtopia2Enable ;  	
    /** MPHY PHY Mode Enable */
	Uint16  phyModeEnable;    	
    /** U16M 8 bit or 16 bit mode */
	Uint16  utopia2BitMode;    	
    /** UPM Valid only when master mode is enable */
	Uint16  utopia2PollingMode;	
    /** XUDC Transmit User defined cell */
	Uint16  txUdc;            	
    /** RUDC Receive User Defined cell */
	Uint16  rxUdc;            	
    /** Slave ID in Master/MPHY mode */
	Uint16	slendSlid;			
} CSL_Utopia2UcrSetup;

/** @brief Utopia2 Receive routing select Structure */
typedef struct CSL_Utopia2Rrs {
    /** Receive byte */
    Uint8   rbyt;
    /** Receive bit */
    Uint8   rbit;
}CSL_Utopia2Rrsr;


/** @brief Utopia2 Clock Setup Command Structure */
typedef struct CSL_Utopia2RruSetup {
    /** Mask bits of RMMR0 - RMMR7  */
	Uint16	rmmrMask0;
	Uint16	rmmrMask1;
	Uint16	rmmrMask2;
	Uint16	rmmrMask3;
	Uint16	rmmrMask4;
	Uint16	rmmrMask5;
	Uint16	rmmrMask6;
	Uint16	rmmrMask7;

    /** Match bits of RMMR0 - RMMR7 */
	Uint16	rmmrMatch0;
	Uint16	rmmrMatch1;
	Uint16	rmmrMatch2;
	Uint16	rmmrMatch3;
	Uint16	rmmrMatch4;
	Uint16	rmmrMatch5;
	Uint16	rmmrMatch6;
	Uint16	rmmrMatch7;

    /** Receive routing select */            
	CSL_Utopia2Rrsr	*rrsr;             
} CSL_Utopia2RruSetup;

/** @brief Utopia2 Hardware Setup structure */ 
typedef struct CSL_Utopia2HwSetup {
    /** Pointer to the Utopia2 Control Register Setup */
	CSL_Utopia2UcrSetup      *ucrSetup;
    /** Pointer to a Utopia2ClkSetup structure */
	CSL_Utopia2ClkSetup   	*clkSetup;
    /** Pointer to Error Setup structure */
	CSL_Utopia2ErrorSetup    *errorSetup;
    /** Pointer to Receive Routing Unit structure */
	CSL_Utopia2RruSetup     *rruSetup;
    /** VOID For later use */
	void                	*extendSetup;  		
} CSL_Utopia2HwSetup;

/** @brief Utopia2 Hardware Setup Structure */
typedef struct CSL_Utopia2GlobalGetSetup {
    /** Pointer to the Utopia2 Control Register Setup */
	CSL_Utopia2UcrSetup		getUcrSetup;
    /** Pointer to a Utopia2ClkSetup structure */
	CSL_Utopia2ClkSetup      getClkSetup;
    /** Pointer to Error Setup structure */
    CSL_Utopia2ErrorSetup    getErrorSetup;
} CSL_Utopia2GlobalGetSetup;
    
/** @brief This structure/object holds the context of the instance of UTOPIA2 
  *        opened using CSL_utopia2Open() function. Pointer to this object is 
  *        passed as UTOPIA2 Handle to all UTOPIA2 CSL APIs. CSL_utopia2Open() 
  *        function initializes this structure based on the parameters passed.
  */
typedef struct CSL_Utopia2Obj {
    /** Pointer to the register overlay structure of the utopia2 */
    CSL_Utopia2RegsOvly	   regs;
    /** Instance of utopia2 being referred by this object  */
    CSL_InstNum            perNum;
} CSL_Utopia2Obj;

/** @brief Holds the base-address information for UTOPIA2 peripheral instance */
typedef struct {
    /** Base-address of the Configuration registers of UTOPIA2. */
    CSL_Utopia2RegsOvly    regs;
} CSL_Utopia2BaseAddress;

/** @brief UTOPIA2 specific parameters. Present implementation doesn't have
 *  any specific parameters.
 */
typedef struct {
    /** Bit mask to be used for module specific parameters.
     *  The below declaration is just a place-holder for future
     *  implementation.
     */
    CSL_BitMask16   flags;
} CSL_Utopia2Param;

typedef struct CSL_Utopia2Obj *CSL_Utopia2Handle;

/** ============================================================================
 *   @n@b CSL_utopia2Init
 *
 *   @b Description
 *   @n This is the initialization function for the UTOPIA2 CSL. The function  
 *      must be called before calling any other API from this CSL. This function 
 *      does not modify any registers or check status. It returns status  
 *      CSL_SOK. It has been kept for future use.
 *
 *   @b Arguments
 *   @verbatim
        pContext    Pointer to module-context.  As UTOPIA2 doesn't have
                    any context based information user is expected to pass NULL.
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK - Always returns
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  The CSL for UTOPIA2 is initialized 
 *
 *   @b Modifies
 *   @n  None  
 *
 *   @b Example
 *   @verbatim
            CSL_Status status;
            ...
            status = CSL_utopia2Init(NULL);
            ...
     @endverbatim
 * =============================================================================
 */
extern CSL_Status  CSL_utopia2Init (
    CSL_Utopia2Context    *pContext
);

/** ============================================================================
 *   @n@b CSL_hpiOpen
 *
 *   @b Description
 *   @n This function reserves the specified UTOPIA2 for use. The device can be 
 *      re-opened anytime after it has been normally closed, if so required. 
 *      The UTOPIA2 handle returned by this call is input as an essential  
 *      argument for the rest of the APIs in UTOPIA2 module.
 *      If @a hwSetup parameter is non-NULL, it configures UTOPIA2 depending on 
 *      the paramters passed in this structure, after successful open

 *
 *   @b Arguments
 *   @verbatim
            pUtopia2Obj      Pointer to UTOPIA2 object that holds the context.
                             Memory for this object should be allocated by the 
                             user.
 
            utopia2Num       Instance of UTOPIA2 to which a handle is requested.
                             There is only one instance of the utopia2 available.
                             So, the value for this parameter will be 
                             CSL_UTOPIA2 always.
 
            pUtopia2Param    Pointer to module specific parameters
 
            pStatus          This returns the status (success/error)  of the 
                             call. The user may pass 'NULL', if status 
                             information is not required.
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Utopia2Handle
 *   @n                     Valid UTOPIA2 handle will be returned if status
 *                          value is equal to CSL_SOK.
 *
 *   <b> Pre Condition </b>
 *   @n  The UTOPIA2 must be successfully initialized via CSL_utopia2Init() 
 *       before  calling this function.
 *
 *   <b> Post Condition </b>
 *   @n   1.    The status is returned in the status variable. If status
 *              returned is
 *   @li            CSL_SOK            - Valid UTOPIA2 handle is returned
 *   @li            CSL_ESYS_FAIL      - The UTOPIA2 instance is invalid
 *   @li            CSL_ESYS_INVPARAMS - Invalid parameter
 *
 *        2.    UTOPIA2 object structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *
 *         2. UTOPIA2 object structure
 *
 *   @b Example
 *   @verbatim
            CSL_status            status;
            CSL_Utopia2Obj        utopia2Obj;
            CSL_Utopia2Handle     hUtopia2;
            ...
                
            hUtopia2 = CSL_utopia2Open(&utopia2Obj, CSL_UTOPIA2, NULL, 
                                     &status);
            
            ...
    @endverbatim
 * =============================================================================
 */
extern CSL_Utopia2Handle CSL_utopia2Open (
    CSL_Utopia2Obj      *pUtopia2Obj,
    CSL_InstNum         utopia2Num,
    CSL_Utopia2Param    *pUtopia2Param,
    CSL_Status          *pStatus
);

/** ============================================================================
 *   @n@b CSL_utopia2Close
 *
 *   @b Description
 *   @n This function closes the specified instance of UTOPIA2.
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2         Pointer to the object that holds reference to the
                            instance of Utopia2 requested after the call

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Close successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *
 *   <b> Pre Condition </b>
 *   @n  The UTOPIA2 instance should be opened before this close operation.
 *
 *   <b> Post Condition </b>
 *   @n  The UTOPIA2 CSL APIs can not be called until the UTOPIA2 CSL is 
 *       reopened again using CSL_utopia2Open().
 *
 *   @b Modifies
 *   @n None
 *
 *   @b Example
 *   @verbatim
            CSL_Utopia2Handle    hUtopia2;
            CSL_Status          status;
            ...
            status = CSL_utopia2Close(hUtopia2);
            ...
            
     @endverbatim
 * =============================================================================
 */ 
extern CSL_Status CSL_utopia2Close (
    CSL_Utopia2Handle    hUtopia2
);

/** ============================================================================
 *   @n@b CSL_utopia2HwSetup
 *
 *   @b Description
 *   @n It configures the Utopia2 registers as per the values passed in the
 *      hardware setup structure.
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2            Handle to the utopia2
 
            utopia2Setup        Pointer to harware setup structure
 
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Hardware setup successful
 *   @li                    CSL_ESYS_BADHANDLE  - The handle passed is invalid
 *   @li                    CSL_ESYS_INVPARAMS  - The parameter passed is 
 *                                                invalid
 *
 *   <b> Pre Condition </b>
 *   @n  Both CSL_utopia2Init() and CSL_utopia2Open() must be called 
 *       successfully in order before calling this function.
 *
 *   <b> Post Condition </b>
 *   @n  UTOPIA2 registers are configured according to the hardware setup 
 *       parameters.
 *
 *   @b Modifies
 *   @n UTOPIA2 registers
 *
 *   @b Example
 *   @verbatim
            CSL_status             status;
            CSL_Utopia2HwSetup     hwSetup;            
            CSL_Utopia2Handle      hUtopia2;
            ...
            hwSetup.utopia2Ctrl = (CSL_Utopia2Ctrl)0x80;
            .....
            
            status = CSL_utopia2HwSetup(hUtopia2, &hwSetup);

     @endverbatim
 * =============================================================================
 */
extern CSL_Status CSL_utopia2HwSetup (
    CSL_Utopia2Handle     hUtopia2,
    CSL_Utopia2HwSetup    *utopia2Setup
);

/** ============================================================================
 *   @n@b CSL_utopia2GetHwSetup
 *
 *   @b Description
 *   @n It retrives the hardware setup parameters of the Utopia2 specified by 
 *      the given handle.
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2    Handle to the utopia2
 
            hwSetup    Pointer to the harware setup structure
 
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Retrieving the hardware setup
 *                                                parameters is successful
 *   @li                    CSL_ESYS_BADHANDLE  - The handle passed is invalid
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_utopia2Init() and CSL_utopia2Open() must be called successfully in 
 *       order before calling CSL_utopia2GetHwSetup().
 *
 *   <b> Post Condition </b>
 *   @n  The hardware setup structure is populated with the hardware setup
 *       parameters
 *
 *   @b Modifies
 *   @n hwSetup variable
 *
 *   @b Example
 *   @verbatim
            CSL_Utopia2Handle   hUtopia2;
            CSL_Utopia2HwSetup  hwSetup;

            ...
            
            status = CSL_utopia2GetHwSetup(hUtopia2, &hwSetup);
            
            ...
            
     @endverbatim
 * =============================================================================
 */
extern CSL_Status CSL_utopia2GetHwSetup (
    CSL_Utopia2Handle	hUtopia2,
    CSL_Utopia2HwSetup  *hwSetup
);

/** ============================================================================
 *   @n@b CSL_utopia2GetHwStatus
 *
 *   @b Description
 *   @n Gets the status of the different operations of UTOPIA2.
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2       Handle to the UTOPIA2 instance

            utopia2Query  The query to this API of UTOPIA2 which indicates the
                          status to be returned.

            response      Placeholder to return the status.

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Query successful
 *   @li                    CSL_ESYS_BADHANDLE  - Invalid handle
 *   @li                    CSL_ESYS_INVQUERY   - The Query passed is invalid
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *  
 *   <b> Pre Condition </b>
 *   @n  CSL_utopia2Init() and CSL_utopia2Open() must be called successfully in  
 *       order before calling CSL_utopia2GetHwStatus().
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n Third parameter response vlaue
 *
 *   @b Example
 *   @verbatim
        CSL_Utopia2Handle           hUtopia2;
        CSL_Utopia2HwStatusQuery    query;
        void                        reponse;
        ...
        status = CSL_utopia2GetHwStatus(hUtopia2, query, &response);
        ...

     @endverbatim
 * =============================================================================
 */
extern CSL_Status CSL_utopia2GetHwStatus (
    CSL_Utopia2Handle        hUtopia2,
    CSL_Utopia2HwStatusQuery	utopia2Query,
    void                    *response
);

/** ============================================================================
 *   @n@b CSL_utopia2HwControl
 *
 *   @b Description
 *   @n This function takes an input control command with an optional argument 
 *      and accordingly controls the operation/configuration of UTOPIA2.
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2        Handle to the UTOPIA2 instance

            ctrlCmd         The command to this API indicates the action to be
                            taken on UTOPIA2.

            arg             An optional argument.

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li         CSL_SOK               - Command successful.
 *   @li         CSL_ESYS_BADHANDLE    - The handle passed is invalid
 *   @li         CSL_ESYS_INVCMD       - Invalid command
 *   @li         CSL_ESYS_INVPARAMS    - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_utopia2Init() and CSL_utopia2Open() must be called successfully in 
 *       order before calling CSL_utopia2HwControl().
 *
 *   <b> Post Condition </b>
 *   @n  UTOPIA2 registers are configured according to the command passed.
 *
 *   @b Modifies
 *   @n The hardware registers of UTOPIA2.
 *
 *   @b Example
 *   @verbatim
        CSL_Utopia2Handle        hUtopia2;
        CSL_Utopia2HwControlCmd  cmd = CSL_UTOPIA2_XMT_ENABLE_CMD;
        void                    arg;
        ...
        status = CSL_utopia2HwControl(hUtopia2, cmd, &arg);
        ...

     @endverbatim
 * =============================================================================
 */
extern CSL_Status CSL_utopia2HwControl (
	CSL_Utopia2Handle		hUtopia2,
	CSL_Utopia2ControlCmd	ctrlCmd,
	void 					*arg
);

/** ============================================================================
 *   @n@b CSL_utopia2GetBaseAddress
 *
 *   @b Description
 *   @n  Function to get the base address of the peripheral instance.
 *       This function is used for getting the base address of the peripheral
 *       instance. This function will be called inside the CSL_utopia2Open()
 *       function call. This function is open for re-implementing if the user
 *       wants to modify the base address of the peripheral object to point to
 *       a different location and there by allow CSL initiated write/reads into
 *       peripheral. MMR's go to an alternate location.
 *
 *   @b Arguments
 *   @verbatim
            utopia2Num      Specifies the instance of the UTOPIA2 to be opened.

            pUtopia2Param   Module specific parameters.

            pBaseAddress    Pointer to baseaddress structure containing base
                            address details.

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             Open call is successful
 *   @li                    CSL_ESYS_FAIL       The instance number is invalid.
 *   @li                    CSL_ESYS_INVPARAMS  Invalid parameter.
 
 
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  Base Address structure is populated
 *
 *   @b Modifies
 *   @n    1. The status variable
 *
 *         2. Base address structure is modified.
 *
 *   @b Example
 *   @verbatim
        CSL_Status          status;
        CSL_Utopia2BaseAddress  baseAddress;

       ...
      status = CSL_utopia2GetBaseAddress(CSL_UTOPIA2, NULL, &baseAddress);

    @endverbatim
 * ===========================================================================
 */
extern CSL_Status CSL_utopia2GetBaseAddress ( 
    CSL_InstNum               utopia2Num,
    CSL_Utopia2Param          *pUtopia2Param,
    CSL_Utopia2BaseAddress    *pBaseAddress
);

#ifdef __cplusplus
}
#endif

#endif    /* _CSL_UTOPIA2_H_ */

