/* =============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2002, 2003, 2004, 2005, 2006
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/** ============================================================================
 *   @file  csl_utopia2Aux.h
 *
 *   @path  $(CSLPATH)\inc
 *
 *   @desc  The defines inline function definitions for control commands & 
 *          status queries.
 *
 */
/* =============================================================================
 *  Revision History
 *  ===============
 *  19-Jan-2006 NG  File taken from TI code base and updated.
 *                  Updated according to coding guidelines (doxygen compliant)
 *
 *  03-Feb-2006 NG  Removed in-line functions as it is not supported.
 * =============================================================================
 */
#ifndef _CSL_UTOPIA2AUX_H_	
#define _CSL_UTOPIA2AUX_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <csl_utopia2.h>
#include <cslr_utopia2.h>

#define CSL_IDEF_INLINE static inline

/** ============================================================================
 *   @n@b CSL_utopia2ErrorClearCmd
 *
 *   @b Description
 *   @n It clears the specified error in EIPR register
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2             Handle to the utopia2
            utopia2ErrorClear    value of error number to clear

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK    - clear successful
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  Error Interrupt Pend Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;
            CSL_BitMask16       utopia2ErrorClear = 1;
            ...
            status = CSL_utopia2ErrorClearCmd(hUtopia2, utopia2ErrorClear);
            ...

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void CSL_utopia2ErrorClearCmd (
    CSL_Utopia2Handle    hUtopia2,    
    CSL_BitMask16        utopia2ErrorClear
)
{
	CSL_Utopia2RegsOvly utopia2Regs = hUtopia2->regs;

	if (utopia2ErrorClear & CSL_UTOPIA2_CTRL_XCPP) {
		CSL_FINST(utopia2Regs->EIPR, UTOPIA2_EIPR_XCPP, CLEAR);
	}
			
	if (utopia2ErrorClear & CSL_UTOPIA2_CTRL_XCFP) {
		CSL_FINST(utopia2Regs->EIPR, UTOPIA2_EIPR_XCFP, CLEAR);
	}
		
	if (utopia2ErrorClear & CSL_UTOPIA2_CTRL_RCPP) {
		CSL_FINST(utopia2Regs->EIPR, UTOPIA2_EIPR_RCPP, CLEAR);
	}
			
	if (utopia2ErrorClear & CSL_UTOPIA2_CTRL_RCFP) {
		CSL_FINST(utopia2Regs->EIPR, UTOPIA2_EIPR_RCFP, CLEAR);
	}
}

/** ============================================================================
 *   @n@b CSL_utopia2ErrorEnableCmd
 *
 *   @b Description
 *   @n It enables the specified bit in EIER register
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2             Handle to the utopia2
            utopia2ErrorEnable   value of error number to enable

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK    - enable successful
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *         
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  Error Interrupt Enable Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;
            CSL_BitMask16       utopia2ErrorEnable = 1;
            ...
            status = CSL_utopia2ErrorEnableCmd(hUtopia2, utopia2ErrorEnable);
            ...

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void CSL_utopia2ErrorEnableCmd (
    CSL_Utopia2Handle    hUtopia2,    
    CSL_BitMask16        utopia2ErrorEnable
)
{
	CSL_Utopia2RegsOvly utopia2Regs =  hUtopia2->regs;

	if (utopia2ErrorEnable & CSL_UTOPIA2_CTRL_XCFE) {
		CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_XCFE, ENABLE);
	}
		
    if (utopia2ErrorEnable & CSL_UTOPIA2_CTRL_XQSE) {
		CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_XQSE, ENABLE);
	}
	
	if (utopia2ErrorEnable & CSL_UTOPIA2_CTRL_XCPE) {
        CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_XCPE, ENABLE);
    }
                
    if (utopia2ErrorEnable & CSL_UTOPIA2_CTRL_RCFE) {
		CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_RCFE, ENABLE);
	}
    
	if (utopia2ErrorEnable & CSL_UTOPIA2_CTRL_RQSE) {
        CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_RQSE, ENABLE);
    }    
 	
	if (utopia2ErrorEnable & CSL_UTOPIA2_CTRL_RCPE) {
        CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_RCPE, ENABLE);
    }
}

/** ============================================================================
 *   @n@b CSL_utopia2Reset
 *
 *   @b Description
 *   @n It brings the UTOPIA2 out of reset state.
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2            Handle to the utopia2
 
     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK             - Reset successful
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  UTOPIA2  Control Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;
            .....
            
            status = CSL_utopia2Reset(hUtopia2);

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void CSL_utopia2Reset (
    CSL_Utopia2Handle    hUtopia2
)
{   
	CSL_Utopia2RegsOvly utopia2Regs =  hUtopia2->regs;

	CSL_FINST(utopia2Regs->UCR, UTOPIA2_UCR_UXEN, RESETVAL);
	CSL_FINST(utopia2Regs->UCR, UTOPIA2_UCR_UREN, RESETVAL);   
}

/** ============================================================================
 *   @n@b CSL_utopia2RecEnableCmd
 *
 *   @b Description
 *   @n It enables the receive port
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2            Handle to the utopia2

     @endverbatim
 *
 *   <b> Return Value </b>  
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  UTOPIA2  Control Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;

            ...
            
            CSL_utopia2RecEnableCmd(hUtopia2);

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void CSL_utopia2RecEnableCmd (
    CSL_Utopia2Handle    hUtopia2
)
{
	CSL_FINST(hUtopia2->regs->UCR,UTOPIA2_UCR_UREN, RX_PORT_ENABLE);
}                     

/** ============================================================================
 *   @n@b CSL_utopia2RecDisableCmd
 *
 *   @b Description
 *   @n It disables the receive port
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2            Handle to the utopia2

     @endverbatim
 *
 *   <b> Return Value </b>  
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  UTOPIA2  Control Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;

            ...
            
            CSL_utopia2RecDisableCmd(hUtopia2);

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void CSL_utopia2RecDisableCmd (
    CSL_Utopia2Handle    hUtopia2
)
{
	CSL_FINST(hUtopia2->regs->UCR,UTOPIA2_UCR_UREN, RX_PORT_DISABLE);
}                     

/** ============================================================================
 *   @n@b CSL_utopia2XmtEnableCmd
 *
 *   @b Description
 *   @n It enables the transmit port
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2            Handle to the utopia2

     @endverbatim
 *
 *   <b> Return Value </b>  
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  UTOPIA2  Control Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;

            .....
            
            CSL_utopia2XmtEnableCmd(hUtopia2);

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void CSL_utopia2XmtEnableCmd ( 
    CSL_Utopia2Handle    hUtopia2
)
{
	CSL_FINST(hUtopia2->regs->UCR, UTOPIA2_UCR_UXEN, TX_PORT_ENABLE);
}

/** ============================================================================
 *   @n@b CSL_utopia2XmtDisableCmd
 *
 *   @b Description
 *   @n It disables the transmit port
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2            Handle to the utopia2

     @endverbatim
 *
 *   <b> Return Value </b>  
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  UTOPIA2  Control Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;

            .....
            
            CSL_utopia2XmtDisableCmd(hUtopia2);

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void CSL_utopia2XmtDisableCmd ( 
    CSL_Utopia2Handle    hUtopia2
)
{
	CSL_FINST(hUtopia2->regs->UCR, UTOPIA2_UCR_UXEN, TX_PORT_DISABLE);
}

/** ============================================================================
 *   @n@b CSL_utopia2ErrorDisableCmd
 *
 *   @b Description
 *   @n It disables the specified bit in EIER register
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2             Handle to the utopia2
            utopia2ErrorDisable  value of error number to disable

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK    - disable successful
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *         
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  Error Interrupt Enable Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;
            CSL_BitMask16       utopia2ErrorDisable = 1;
            ...
            status = CSL_utopia2ErrorDisableCmd(hUtopia2, utopia2ErrorDisable);
            ...

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void CSL_utopia2ErrorDisableCmd (
    CSL_Utopia2Handle    hUtopia2,    
    CSL_BitMask16        utopia2ErrorDisable
)
{
	CSL_Utopia2RegsOvly utopia2Regs =  hUtopia2->regs;

	if (utopia2ErrorDisable & CSL_UTOPIA2_CTRL_XCFE) {
		CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_XCFE, DISABLE);
	}
    	
	if (utopia2ErrorDisable & CSL_UTOPIA2_CTRL_XQSE) {
		CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_XQSE, DISABLE);
	}
 	
	if (utopia2ErrorDisable & CSL_UTOPIA2_CTRL_XCPE) {
		CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_XCPE, DISABLE);
	}
        
    if (utopia2ErrorDisable & CSL_UTOPIA2_CTRL_RCFE) {
		CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_RCFE, DISABLE);
	}
    
	if (utopia2ErrorDisable & CSL_UTOPIA2_CTRL_RQSE) {
		CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_RQSE, DISABLE);
	}
 	
	if (utopia2ErrorDisable & CSL_UTOPIA2_CTRL_RCPE) {
		CSL_FINST(utopia2Regs->EIER, UTOPIA2_EIER_RCPE, DISABLE);
	}

}
//////////////////////////////////////////////////////////////////////////////

/** ============================================================================
 *   @n@b CSL_utopia2IntrClearCmd
 *
 *   @b Description
 *   @n It clears the specified error in EIPR register
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2             Handle to the utopia2
            utopia2ErrorClear    value of error number to clear

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK    - clear successful
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  Error Interrupt Pend Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;
            CSL_BitMask16       utopia2IntrClear = 1;
            ...
            status = CSL_utopia2IntrClearCmd(hUtopia2, utopia2IntrClear);
            ...

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void	CSL_utopia2IntrClearCmd (
    CSL_Utopia2Handle hUtopia2,
    CSL_BitMask16     utopia2IntrClear
)
{
	CSL_Utopia2RegsOvly utopia2Regs =  hUtopia2->regs;
	
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_XQIP0) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_XQIP0, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_XQIP1) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_XQIP1, CLEAR);
	}
	
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_XQIP2) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_XQIP2, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_XQIP3) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_XQIP3, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_XQIP4) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_XQIP4, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_XQIP5) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_XQIP5, CLEAR);
	}
    	
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_XQIP6) {
	   	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_XQIP6, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_XQIP7) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_XQIP7, CLEAR);
	}

	/* Receive Queue */
	
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_RQIP0) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_RQIP0, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_RQIP1) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_RQIP1, CLEAR);
	}
	
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_RQIP2) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_RQIP2, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_RQIP3) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_RQIP3, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_RQIP4) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_RQIP4, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_RQIP5) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_RQIP5, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_RQIP6) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_RQIP6, CLEAR);
	}
    
	if (utopia2IntrClear & CSL_UTOPIA2_CTRL_RQIP7) {
    	CSL_FINST(utopia2Regs->UIPR,UTOPIA2_UIPR_RQIP7, CLEAR);
	}
}

/** ============================================================================
 *   @n@b CSL_utopia2IntrEnableCmd
 *
 *   @b Description
 *   @n It enables the specified bit in EIER register
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2             Handle to the utopia2
            utopia2ErrorEnable   value of error number to enable

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK    - enable successful
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *         
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  Error Interrupt Enable Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;
            CSL_BitMask16       utopia2IntrEnable = 1;
            ...
            status = CSL_utopia2IntrEnableCmd(hUtopia2, utopia2IntrEnable);
            ...

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void	CSL_utopia2IntrEnableCmd (
    CSL_Utopia2Handle hUtopia2,
    CSL_BitMask16    utopia2IntrEnable
)
{
	CSL_Utopia2RegsOvly utopia2Regs =  hUtopia2->regs;
    
    if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_XQIE0) {
    	CSL_FINST(utopia2Regs->UIER, UTOPIA2_UIER_XQIE0, ENABLE);
   	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_XQIE1) {
    	CSL_FINST(utopia2Regs->UIER, UTOPIA2_UIER_XQIE1, ENABLE);
	}
	
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_XQIE2) {
    	CSL_FINST(utopia2Regs->UIER, UTOPIA2_UIER_XQIE2, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_XQIE3) {
    	CSL_FINST(utopia2Regs->UIER, UTOPIA2_UIER_XQIE3, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_XQIE4) {
    	CSL_FINST(utopia2Regs->UIER, UTOPIA2_UIER_XQIE4, ENABLE);
	}
	
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_XQIE5) {
    	CSL_FINST(utopia2Regs->UIER, UTOPIA2_UIER_XQIE5, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_XQIE6) {
    	CSL_FINST(utopia2Regs->UIER, UTOPIA2_UIER_XQIE6, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_XQIE7) {
    	CSL_FINST(utopia2Regs->UIER, UTOPIA2_UIER_XQIE7, ENABLE);
	}
	/*-------------------------------------------------------*/
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_RQIE0) {
    	CSL_FINST(utopia2Regs->UIER, UTOPIA2_UIER_RQIE0, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_RQIE1) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE1, ENABLE);
	}
	
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_RQIE2) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE2, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_RQIE3) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE3, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_RQIE4) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE4, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_RQIE5) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE5, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_RQIE6) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE6, ENABLE);
	}
    
	if (utopia2IntrEnable & CSL_UTOPIA2_CTRL_RQIE7) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE7, ENABLE);
	}
	
}

/** ============================================================================
 *   @n@b CSL_utopia2IntrDisableCmd
 *
 *   @b Description
 *   @n It disables the specified bit in EIER register
 *
 *   @b Arguments
 *   @verbatim
            hUtopia2             Handle to the utopia2
            utopia2ErrorDisable  value of error number to disable

     @endverbatim
 *
 *   <b> Return Value </b>  CSL_Status
 *   @li                    CSL_SOK    - disable successful
 *   @li                    CSL_ESYS_INVPARAMS  - Invalid parameter
 *         
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Modifies
 *   @n  Error Interrupt Enable Register
 *
 *   @b Example
 *   @verbatim
            CSL_status          status;
            CSL_Utopia2Handle    hUtopia2;
            CSL_BitMask16       utopia2IntrDisable = 1;
            ...
            status = CSL_utopia2IntrDisableCmd(hUtopia2, utopia2IntrDisable);
            ...

     @endverbatim
 * =============================================================================
 */
CSL_IDEF_INLINE 
void CSL_utopia2IntrDisableCmd (
    CSL_Utopia2Handle hUtopia2,
    CSL_BitMask16     utopia2IntrDisable
)
{
	CSL_Utopia2RegsOvly utopia2Regs =  hUtopia2->regs;

	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_XQIE0) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_XQIE0, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_XQIE1) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_XQIE1, DISABLE);
	}
	
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_XQIE2) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_XQIE2, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_XQIE3) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_XQIE3, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_XQIE4) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_XQIE4, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_XQIE5) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_XQIE5, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_XQIE6) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_XQIE6, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_XQIE7) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_XQIE7, DISABLE);
	}
	/*-------------------------------------------------------*/    			
	
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_RQIE0) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE0, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_RQIE1) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE1, DISABLE);
	}
	
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_RQIE2) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE2, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_RQIE3) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE3, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_RQIE4) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE4, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_RQIE5) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE5, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_RQIE6) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE6, DISABLE);
	}
    
	if (utopia2IntrDisable & CSL_UTOPIA2_CTRL_RQIE7) {
    	CSL_FINST(utopia2Regs->UIER,UTOPIA2_UIER_RQIE7, DISABLE);
	}
	
}

#ifdef __cplusplus
}
#endif

/*****************************************************************************/
#endif
/*****************************************************************************/

